Package: ryandexdirect
Type: Package
Title: Load Data From 'Yandex Direct'
Version: 3.6.6
Description: Load data from 'Yandex Direct' API V5 
    <https://yandex.ru/dev/direct/doc/dg/concepts/about-docpage> into R.
	Provide function for load lists of campaings, ads, keywords and other 
	objects from 'Yandex Direct' account. Also you can load statistic from
	API 'Reports Service' <https://yandex.ru/dev/direct/doc/reports/reports-docpage>.
	And allows keyword bids management.
License: GPL-2
Authors@R: 
    c(person(given  = "Alexey",
             family = "Seleznev",
             role   = c("aut", "cre"),
             email  = "selesnow@gmail.com",
             comment = c(ORCID = "0000-0003-0410-7385")))
Author: Alexey Seleznev [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0410-7385>)
Maintainer: Alexey Seleznev <selesnow@gmail.com>
Depends: R (>= 3.5.0)
BugReports: https://github.com/selesnow/ryandexdirect/issues
URL: https://selesnow.github.io/ryandexdirect/,
        https://t.me/R4marketing,
        https://www.youtube.com/playlist?list=PLD2LDq8edf4oUo0L9Kw77ZXf0KcV1hu67
Imports: utils, httr, bitops, jsonlite, xml2, data.table, readr,
        magrittr, dplyr, tidyr (>= 1.0.0), purrr, stringr, httr2
Suggests: knitr, rmarkdown, kableExtra, googleAnalyticsR, rym
Encoding: UTF-8
Language: ru
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-19 19:33:05 UTC; User
Repository: CRAN
Date/Publication: 2026-05-20 06:20:15 UTC
Built: R 4.5.2; ; 2026-05-21 23:34:28 UTC; unix

export(yadirGetCampaignList)
export(yadirGetCampaign)
export(yadirGetClientList)
export(yadirGetToken)
export(yadirAuth)
export(yadirGetLogins)
export(yadirSetLogin)
export(yadirSetAgencyAccount)
export(yadirGetDictionary)
export(yadirCurrencyRates)
export(yadirGetReport)
export(yadirGetAdGroups)
export(yadirGetAds)
export(yadirGetKeyWords)
export(yadirGetBalance)
export(yadirGetClientParam)
export(yadirGetSiteLinks)
export(yadirStopCampaigns)
export(yadirStartCampaigns)
export(yadirStartAds)
export(yadirStopAds)
export(yadirStopKeyWords)
export(yadirStartKeyWords)
export(yadirGetKeyWordsBids)
export(yadirSetAutoKeyWordsBids)
export(yadirGetWordStatReport)
export(yadirSetKeyWordsBids)
export(yadirGetForecast)
export(yadirGetCostData)
importFrom(utils,browseURL)
importFrom(utils,head)
importFrom(httr,POST)
importFrom(httr,GET)
importFrom(httr,stop_for_status)
importFrom(httr,content)
importFrom(httr,add_headers)
importFrom(httr, headers)
importFrom(tidyr, unnest_wider, unnest_longer, hoist, replace_na)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,fromJSON)
importFrom(xml2,xml_text)
importFrom(xml2,xml_find_all)
importFrom(dplyr, bind_rows, select, filter, tibble, rename, mutate, mutate_if, "%>%")
importFrom(purrr, map_df, map_dfr, flatten)
importFrom(stringr, str_glue)
import(bitops)
import(data.table)
import(magrittr)
import(httr2)
importFrom(httr2, request, req_body_form, req_perform, req_url_query, req_headers, resp_body_json)
importFrom(xml2, xml_find_all)
importFrom(xml2, xml_text)
import(readr)
S3method(print, yadir_token)

<p align="center">
<a href="https://selesnow.github.io/"><img src="https://alexeyseleznev.files.wordpress.com/2017/03/as.png" height="80"></a>
</p>


# ryandexdirect - пакет для работы с API Яндекс.Директ версии 4, Live 4 и 5 на языке R.<a href='https://selesnow.github.io/ryandexdirect/'><img src='https://raw.githubusercontent.com/selesnow/ryandexdirect/master/inst/ryandexdirect.png' align="right" height="139" /></a>

## CRAN
[Ссылка на страницу пакета на CRAN](https://CRAN.R-project.org/package=ryandexdirect)

### Бейджи
[![Rdoc](http://www.rdocumentation.org/badges/version/ryandexdirect)](https://www.rdocumentation.org/packages/ryandexdirect)
[![](https://cranlogs.r-pkg.org/badges/ryandexdirect)](https://cran.r-project.org/package=ryandexdirect)

## Краткое описание.
Пакет ryandexdirect предназначен для загрузки данных из Яндекс Директ в R, с помощью функций данного пакета вы можете работать с перечисленными ниже сервисами и службами API Яндекса с помощью готовых функций, не углубляясь при этом в документацию по работе с этими API сервисами.

+ [Сервис Reports](https://yandex.ru/dev/direct/doc/reports/reports-docpage) - Предназначен для получения статистики по аккаунту рекламодателя.
+ [API Директа версии 4 и Live 4](https://yandex.ru/dev/direct/doc/dg-v4/concepts/About-docpage) - Через API внешние приложения добавляют и редактируют кампании, объявления, фразы, задают ставки, получают статистику показов.
+ [API Директа версии 5](https://yandex.ru/dev/direct/doc/dg/concepts/about-docpage) - Через API внешние приложения добавляют и редактируют кампании, объявления, фразы, задают ставки, получают статистику показов.

Пакет позволяет вам выполнять следующие действия:

1. Авторизовываться в API.
2. Получать список различных объектов рекламного кабинета, рекламных кампаний, групп объявлений, объявлений, для агентских аккаунтов можно запрашивать список клиентов, и параметры каждого клиента.
3. Управлять показами на уровне рекламных кампаний, групп объявлений и объявлений.
4. Загружать статистику.
5. Упаравлять ставками ключевых слов и автотаргетингов.

## Поддержать проект
Вы можете поддержать проект любой произвольной суммой перейдя по этой [ссылке](https://secure.wayforpay.com/button/b6dd4a7083fe0).

## Установка пакета ryandexdirect.
Для установки из CRAN воспользуйтесь стандартной командой: `install.packages("ryandexdirect")`

Установить dev версию можно из репозитория GitHub, для этого сначала требуется установить и подключить пакет devtools.

```r
install.packages("devtools")
devtools::install_github('selesnow/ryandexdirect')
```

### Ссылки
1. [Документация по работе с пакетом ryandexdirect](https://selesnow.github.io/ryandexdirect/).
2. [Видео уроки по работе с ryandexdirect](https://www.youtube.com/playlist?list=PLD2LDq8edf4oUo0L9Kw77ZXf0KcV1hu67)
2. Баг репорты, предложения по доработке и улучшению функционала ryandexdirect оставлять [тут](https://github.com/selesnow/ryandexdirect/issues). 
3. [Список релизов](https://github.com/selesnow/ryandexdirect/releases).
4. [Телеграмм канал R4marketing](https://t.me/R4marketing).

### Автор пакета
Alexey Seleznev
<Br>Telegram канал автора: [R4marketing](https://t.me/R4marketing)
<Br>Сайт с документацией к пакетам: [selesnow.github.io](https://selesnow.github.io)
<Br>email: selesnow@gmail.com
<Br>skype: selesnow
<Br>facebook: [facebook.com/selesnow](https://www.facebook.com/selesnow)
<Br>linkedin: [linkedin.com/in/selesnow](https://www.linkedin.com/in/selesnow)
<Br>blog: [alexeyseleznev.wordpress.com](https://alexeyseleznev.wordpress.com/)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
# library(rfacebookstat)
# library(googleAnalyticsR)
# 
# # fb options
# options(ryandexdirect.username = 'your_yandex_username')
# 
# # auth
# fbAuth()
# ga_auth(email = "google_analytics@gmail.com")
# 
# # read data from facebook
# cost_data <- yadirGetCostData(DateFrom        = '2020-03-01',
#                               DateTo          = '2020-03-10',
#                               Source          = 'yandex',
#                               Medium          = 'cpc',
#                               IncludeVAT      = "YES",
#                               IncludeDiscount = "NO")
# 
# # upload into GA source
# ga_custom_upload_file(accountId          = 2222222,
#                       webPropertyId      = "UA-2222222-1",
#                       customDataSourceId = 'M_XXXXX_Qudk-8g',
#                       cost_data)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  eval = FALSE,
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
# library(ryandexdirect)

## ----eval = TRUE, results = "asis", echo=FALSE--------------------------------
library(magrittr)
table_report_types <- data.frame(reptype = c("ACCOUNT_PERFORMANCE_REPORT", 
                                             "CAMPAIGN_PERFORMANCE_REPORT", 
                                             "ADGROUP_PERFORMANCE_REPORT",
                                             "AD_PERFORMANCE_REPORT",
                                             "CRITERIA_PERFORMANCE_REPORT",
                                             "CUSTOM_REPORT",
                                             "REACH_AND_FREQUENCY_PERFORMANCE_REPORT",
                                             "SEARCH_QUERY_PERFORMANCE_REPORT"), 
                                 descr    = c("Статистика по аккаунту рекламодателя",
                                              "Статистика по кампаниям",
                                              "Статистика по группам объявлений",
                                              "Статистика по объявлениям",
                                              "Статистика по условиям показа",
                                              "Статистика с произвольными группировками",
                                              "Статистика по медийным кампаниям. Отчет содержит только данные по кампаниям с типом «Медийная кампания», кампании остальных типов игнорируются",
                                              "Статистика по поисковым запросам"),
                                 grouping = c("–", 
                                              "CampaignId", 
                                              "AdGroupId",
                                              "AdId",
                                              "AdGroupId, CriteriaId, CriteriaType",
                                              "–",
                                              "В запросе на формирование отчета необходимо указать в поле FieldNames значение CampaignId", 
                                              "AdGroupId, Query"))

names(table_report_types) <- c("Тип отчёта", "Описание", "Добавляется группировка")
table_report_types <- kableExtra::kable(table_report_types)
kableExtra::kable_styling(table_report_types, bootstrap_options = c("striped", "hover"))

## ----eval = TRUE, results = "asis", echo=FALSE--------------------------------
library(magrittr)
table_report_types <- data.frame(field = c("AdNetworkType, CampaignId, CampaignType", 
                                             "AdFormat, AdGroupId, AdId, Age, AudienceTargetId, CarrierType, ClickType, CriteriaType, CriterionType, Device, DynamicTextAdTargetId, ExternalNetworkName, Gender, LocationOfPresenceId, MatchType, MobilePlatform, Placement, RlAdjustmentId, Slot, SmartBannerFilterId, TargetingLocationId", 
                                             "Clicks, Conversions, ImpressionReach, Impressions",
                                             "AvgClickPosition, AvgCpc, AvgCpm, AvgImpressionFrequency, AvgImpressionPosition, AvgPageviews, AvgTrafficVolume, BounceRate, ConversionRate, Cost, CostPerConversion, Ctr, GoalsRoi, ImpressionShare, Profit, Revenue, WeightedCtr, WeightedImpressions, ",
                                             "Keyword, MatchedKeyword, Query"), 
                                 operators    = c("EQUALS, IN",
                                              "EQUALS, IN, NOT_EQUALS, NOT_IN",
                                              "EQUALS, IN, GREATER_THAN, LESS_THAN",
                                              "GREATER_THAN, LESS_THAN",
                                              "EQUALS, IN, NOT_EQUALS, NOT_IN, STARTS_WITH_IGNORE_CASE, STARTS_WITH_ANY_IGNORE_CASE, DOES_NOT_START_WITH_IGNORE_CASE, DOES_NOT_START_WITH_ALL_IGNORE_CASE"))

names(table_report_types) <- c("Имя поля", "Доступные операторы")
table_report_types <- kableExtra::kable(table_report_types)
kableExtra::kable_styling(table_report_types, bootstrap_options = c("striped", "hover"))

## -----------------------------------------------------------------------------
# library(ryandexdirect)
# 
# # создаём результирующий фрейм
# res   <- data.frame()
# 
# # список логинов
# log_list <- c("login1", "login2","login3", "login4")
# 
# # проверка лиситов
# # отмечаем что это первый запуск
# check <- "first"
# 
# # создаём последовательность уровней временной разбивки запросов
# fetching_seq <- c("OFF", "MONTH", "WEEK", "DAY")
# 
# # счётчик последовательностей разбивки
# fetch_id <- 1
# 
# # запускаем цикл загрузки данных с проверкой лимитов
# while ( ! is.null( log_list ) ) {
# 
#   # определяем уровень разбивки запроса
#   if ( fetching_seq[fetch_id] == "OFF" ) fetching <- NULL else fetching <- fetching_seq[fetch_id]
# 
#   # запускаем сбор данных
#   data <- yadirGetReport(DateRangeType = "CUSTOM_DATE",
#                          DateFrom      = "2018-06-01",
#                          DateTo        = "2019-05-31",
#                          FieldNames    = c("Date","CampaignName","Impressions","Clicks"),
#                          Login         = log_list,
#                          FetchBy       = fetching)
# 
#   # если загрузка была по одному аккаунту добавляем его логин
#   if ( length(log_list) == 1 ) {
#     data$Login <- log_list
#   }
# 
#   # проверяем список аккаунтов достигших лимита
#   log_list <- attr(data, "limit_reached")
# 
#   # выводим список аккаунтов достигших лимита
#   print(log_list)
# 
#   # если есть аккаунты достигшие лимита
#   if ( length(log_list) > 0 ) {
#     # очищаем от них общую таблицу
#     data <- data[ ! data$Login %in% log_list, ]
#     # переключаем уровень разбивки на более мелкий
#     fetch_id <- fetch_id + 1
#   }
# 
#   # дописываем в результирующий фрейм данные
#   # по тем аккаунтам которые не упёрлись в лимит
#   if ( nrow(data) > 0 ) {
#     res <- dplyr::bind_rows(res, data)
#   }
# 
#   # проверяем модно ли разбить запрос на более мелкие части
#   if ( fetch_id > length(fetching_seq) && length(log_list) > 0 ) {
#     message("Запрос невозможно разбить на меньшие части")
#     message("Аккаунты которые достигли лимита при загрузке данных по дням: ", paste(log_list, collapse = ", "))
#     limits_login <- log_list
#     break
#   }
# }

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  eval = FALSE,
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
# library(ryandexdirect)

## -----------------------------------------------------------------------------
# s_ws_rep <- yadirGetWordStatReport(c('купить смартфон',
#                                      'купить сотовый телефон',
#                                      'купить мобильный',
#                                      'samsung -(серого цвета)'),
#                                    Login = "selesnow")

## -----------------------------------------------------------------------------
# # Что искали с заданными ключевыми словами
# s_ws_rep$SearchedWith
# s_ws_rep[['SearchedWith']]
# 
# # Запросы, похожие на заданные ключевые слова
# s_ws_rep$SearchedAlso
# s_ws_rep[['SearchedAlso']]
# 

## -----------------------------------------------------------------------------
# library(dplyr)
# 
# # запрашиваем справочник регионов
# regions <- yadirGetDictionary(Login = "selesnow")
# 
# # оставляем толлько нужные регионы
# rep_regions <- regions %>%
#                   filter(GeoRegionName %in% c("Москва",
#                                               "Санкт-Петербург",
#                                               "Екатеринбург",
#                                               "Владивосток"))
# 
# # запрашиваем отчёт
# reg_ws <- yadirGetWordStatReport(Phrases = c('купить смартфон -xiaomi',
#                                              'купить xiaomi'),
#                                  GeoID   = rep_regions$GeoRegionId,
#                                  Login   = "selesnow")

## -----------------------------------------------------------------------------
# library(dplyr)
# 
# # запрашиваем справочник регионов
# regions <- yadirGetDictionary(Login = "selesnow")
# 
# # получаем идентификатор России
# regions <- regions %>%
#   filter(GeoRegionName %in% c("Россия"))
# 
# # получаем идентификатор минус регионов и помечаем их минусом
# minus_regions <- regions %>%
#   filter(GeoRegionName %in% c("Москва",
#                               "Санкт-Петербург",
#                               "Екатеринбург",
#                               "Владивосток")) %>%
#   mutate(GeoRegionId = paste0("-", GeoRegionId))
# 
# # запрашиваем прогноз с результататми торгов
# forecast <- yadirGetForecast2(Phrases = c('купить смартфон -xiaomi',
#                                          'купить xiaomi',
#                                          'самсунг -(серого цвета)'),
#                              GeoID   = c(regions$GeoRegionId, minus_regions$GeoRegionId),
#                              AuctionBids = 'Yes',
#                              Login   = "selesnow")

yadirStopCampaigns <-  function(Login = getOption("ryandexdirect.user"), 
                                Ids   = NULL,
                                Token = NULL,
                                AgencyAccount = getOption("ryandexdirect.agency_account"),
                                TokenPath     = yadirTokenPath()){
    
    # auth
    Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
    
    if(length(Ids) > 1000){
      stop(paste0("In the parameter Ids transferred numbers ",length(Ids), " of campaigns, maximum number of campaigns in one request is 1000."))
    }
    
    if(is.null(Ids)){
      stop("In the Ids argument, you must pass the vector containing the Id campaigns for which you want to stop the ads. You have not transferred any Id.")
    }
    
    # error counter
    CounErr <- 0
    
    # error vector
    errors_id <-  vector()
    
    # start time
    start_time  <- Sys.time()
    
    # start message
    packageStartupMessage("Processing", appendLF = T)
    
    IdsPast <- paste0(Ids, collapse = ",")
	
    # request body
    queryBody <- paste0("{
                          \"method\": \"suspend\",
                          \"params\": { 
                            \"SelectionCriteria\": {
                                              \"Ids\": [",IdsPast,"]}
                          }
                        }")
  
    # send request
    answer <- POST("https://api.direct.yandex.com/json/v5/campaigns", 
	               body = queryBody, 
				   add_headers(Authorization    = paste0("Bearer ",Token), 
				              'Accept-Language' = "ru",
							  "Client-Login"    = Login))
    # answer parser
    ans_pars <- content(answer)
    # check for error
    if(!is.null(ans_pars$error)){
      stop(paste0("Error: ", ans_pars$error$error_string,". Message: ",ans_pars$error$error_detail, ". Request ID: ",ans_pars$error$request_id))
    }
  
    # check missing campaings
    for(error_search in 1:length(ans_pars$result$SuspendResults)){
      if(!is.null(ans_pars$result$SuspendResults[[error_search]]$Errors)){
        CounErr <- CounErr + 1
        errors_id <- c(errors_id, Ids[error_search])
        packageStartupMessage(paste0("    CampId: ",Ids[error_search]," - ", ans_pars$result$SuspendResults[[error_search]]$Errors[[1]]$Details))
      }
    }
  
    # prepare result message
    out_message <- ""
  
    TotalCampStoped <- length(Ids) - CounErr
  
    if(TotalCampStoped %in% c(2,3,4) & !(TotalCampStoped %% 100 %in% c(12,13,14))){
      out_message <- "campaings stoped"
    } else if(TotalCampStoped %% 10 == 1 & TotalCampStoped %% 100 != 11){
     out_message <- "campaings stoped"
    } else {
      out_message <- "campaings stoped"
    }
  
    # result message
    packageStartupMessage(paste0(TotalCampStoped, " ", out_message))
    packageStartupMessage(paste0("Total time: ", as.integer(round(difftime(Sys.time(), start_time , units ="secs"),0)), " sec."))
    return(errors_id)}

yadirGetSiteLinks <- function(Login = getOption("ryandexdirect.user"),
                              Token = NULL,
                              Ids   = NULL,
                              AgencyAccount = getOption("ryandexdirect.agency_account"),
                              TokenPath     = yadirTokenPath()) {
  
  # result frame
  result <- data.frame()
  
  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  # set offset and limit
  offset <- 0
  limit  <- 10000
  
  dataRaw <- list()
  
  # start the cycle
  while (! is.null(dataRaw$result$LimitedBy) || offset == 0 ) {
    # body query
    queryBody <- paste0("{
                        \"method\": \"get\",
                        \"params\": {",
                          ifelse( is.null(Ids), "", paste0("\"SelectionCriteria\": { \"Ids\": [",paste0(Ids, collapse = ","),"]}," )),"
                          \"FieldNames\": [\"Id\", \"Sitelinks\"],
                          \"Page\": {  
                          \"Limit\": ",limit,",
                          \"Offset\": ",offset,"}
  }
  }")
    
    # send request API
    answer <- POST("https://api.direct.yandex.com/json/v5/sitelinks", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru",'Client-Login' = Login))
    stop_for_status(answer)
    
    # parsing
    dataRaw <- content(answer, "parsed", "application/json")
    
    # check error
    if(length(dataRaw$error) > 0){
      stop(paste0(dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
    }
    
    # parse json and to result frame
    for (i in seq_along(1:length(dataRaw$result$SitelinksSets))) {
      for (y in seq_along(1:length(dataRaw$result$SitelinksSets[[i]]$Sitelinks)))
        result <- rbind(result, 
                        data.frame(Id          = dataRaw$result$SitelinksSets[[i]]$Id,
                                   Title       = ifelse(is.null(dataRaw$result$SitelinksSets[[i]]$Sitelinks[[y]]$Title), NA, dataRaw$result$SitelinksSets[[i]]$Sitelinks[[y]]$Title),
                                   Href        = ifelse(is.null(dataRaw$result$SitelinksSets[[i]]$Sitelinks[[y]]$Href), NA, dataRaw$result$SitelinksSets[[i]]$Sitelinks[[y]]$Href),
                                   Description = ifelse(is.null(dataRaw$result$SitelinksSets[[i]]$Sitelinks[[y]]$Description), NA, dataRaw$result$SitelinksSets[[i]]$Sitelinks[[y]]$Description))
        )
    }
    
    # check of needly next iteration
    if (! is.null(dataRaw$result$LimitedBy)) {
      offset <- offset + limit
    } else {
      break
    }
  }
  
  # result
  return(result)
}

yadirGetToken <-
function(Login = NULL, TokenPath = yadirTokenPath()){
  # open browser
  browseURL("https://oauth.yandex.ru/authorize?response_type=token&client_id=365a2d0a675c462d90ac145d4f5948cc")
  
  # create list
  token <- list(token_type    = "bearer",
                access_token  = readline(prompt = "Enter your token: "),
                expires_in    = 26715505,
                refresh_token = NA,
                expire_at     = Sys.time() + as.numeric(26715505, units = "secs"))
  # set class
  class(token) <- "yadir_token"
  
  # ask for save credential
  # save token in file
  message("Do you want save API credential in local file (",paste0(TokenPath, "/", Login, ".yadirAuth.RData"),"), for use it between R sessions?")
  ans <- readline("y / n (recomedation - y): ")
  
  if ( tolower(ans) %in% c("y", "yes", "ok", "save") ) {
    # save into local file
    save(token, file = paste0(TokenPath, "/", Login, ".yadirAuth.RData"))
    message("Token saved in file ", paste0(TokenPath, "/", Login, ".yadirAuth.RData"))
  }
  return(token)
}

yadirGetBalance <- function(Logins        = getOption("ryandexdirect.user"), 
                            Token         = NULL,     
                            AgencyAccount = getOption("ryandexdirect.agency_account"),
                            TokenPath     = yadirTokenPath()){

  # result frame
  result <- data.table()
  
  # auth
  if (length(Logins) > 1 || is.null(Logins)) {
  Token <- tech_auth(login = AgencyAccount, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  } else {
  Token <- tech_auth(login = Logins, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)  
  }
 
  # start element
  start_element <- 1
  
  # limit
  lim <- 50
  
  # last iteraction
  ended <- FALSE
  
  while(ended == FALSE) {
  
   # get logins part
   logins_temp <- head(Logins[start_element:length(Logins)], lim)
   
   # for right json check number of logins
   # if number of login == 1 then convert him to list
   if(length(logins_temp)==1){
     logins_temp <- list(logins_temp)
    }
  
  # compose query body
	body_list <-  list(method = "AccountManagement",
					   param  = list(Action = "Get"),
					   locale = "ru",
					   token = Token)
    
  # if is agency account add selection 
	if ( ! is.null(AgencyAccount) ) {
	  body_list$param <- append(body_list$param, list(SelectionCriteria = list(Logins = logins_temp)))
	}

  # body to json format
  body_json <- toJSON(body_list, auto_unbox = T, pretty = TRUE)
  
  # send query
  answer <- POST("https://api.direct.yandex.ru/live/v4/json/", body = body_json)
  
  # check for error
  stop_for_status(answer)
  
  # parsing
  dataRaw <- content(answer, "parsed", "application/json")
  
  # check result for error
  if(!is.null(dataRaw$error_code)){
    stop(paste0("Error: code - ",dataRaw$error_code, ", message - ",dataRaw$error_str, ", detail - ",dataRaw$error_detail))
  }
  
  # Result to data frame
  result_temp <- fromJSON(content(answer, "text", "application/json", encoding = "UTF-8"),flatten = TRUE)$data$Accounts
  
  # check for all logins is loaded
  errors_list <- fromJSON(content(answer, "text", "application/json", encoding = "UTF-8"),flatten = TRUE)$data$ActionsResult
  
  if(length(errors_list) > 0){
  error <- data.frame(login = errors_list$Login, do.call(rbind.data.frame, errors_list$Errors))
  packageStartupMessage(paste0("Next ",nrow(error)," account",ifelse(nrow(error) > 1, "s","")," get error with try get ballance:"), appendLF = T)
  
  for(err in 1:nrow(error)){
	  packageStartupMessage(paste0("Login: ", error$login[err]), appendLF = T)
	  packageStartupMessage(paste0("....Code: ", error$FaultCode[err]), appendLF = T)
	  packageStartupMessage(paste0("....String: ", error$FaultString[err]), appendLF = T)  
	  packageStartupMessage(paste0("....Detail: ", error$FaultDetail[err]), appendLF = T)
  }}
  
  # bind to result data frame
  result <- rbind(result, result_temp, fill = TRUE)
  
  # to next iteraction
  start_element <- start_element + lim
  # check next part of logins
  if (start_element > length(Logins)) {
    ended <- TRUE
  }
 }    
  return(as.data.frame(result))}

yadirGetClientList <-
function(AgencyAccount = getOption("ryandexdirect.agency_account"),
         Token         = NULL,
         TokenPath     = yadirTokenPath()){
   
   # authorize
   Token <- tech_auth(token = Token, AgencyAccount = AgencyAccount, 
                      TokenPath = TokenPath)
   
   # prepare query body
   q <- list(method = "get",
             params = list(SelectionCriteria = NULL,
                           FieldNames = c( "AccountQuality", 
                                           "Archived", 
                                           "ClientId", 
                                           "ClientInfo", 
                                           "CountryId", 
                                           "CreatedAt", 
                                           "Currency", 
                                           "Grants", 
                                           "Login", 
                                           "Notification", 
                                           "OverdraftSumAvailable", 
                                           "Phone", 
                                           "Representatives", 
                                           "Restrictions", 
                                           "Settings", 
                                           "Type", 
                                           "VatRate"))) 
   # convert body to json format
   q_json <- toJSON(q, auto_unbox = T)
   
   # send HTTP query
   answer <- POST("https://api.direct.yandex.com/json/v5/agencyclients", 
                  body = q_json,
                  add_headers(Authorization = paste0("Bearer ",Token), 
                              'Accept-Language' = "ru"))
   
   # check query for error
   stop_for_status(answer)
   dataRaw <- content(answer, "parsed", "application/json")
   
   # check answer for error
   if ( !is.null(dataRaw$error ) ) {
     stop(dataRaw$error$error_detail, 
          call. = cat("Error detail:\n",
                      "request_id =",dataRaw$error$request_id, "\n",
                      "error_code =",dataRaw$error$error_code, "\n",
                      "error_detail =",dataRaw$error$error_detail, "\n",
                      "error_string =",dataRaw$error$error_string, "\n"))
   }
   
   # result data
   data <- list()
   
   # parse raw data
   for (i in dataRaw$result$Clients) {

     data <- append(data, list(
                            list(Login           = i$Login,
                                 FIO             = i$ClientInfo,
                                 StatusArch      = i$Archived,
                                 DateCreate      = i$CreatedAt,
                                 Role            = i$Type,
                                 Email           = i$Notification$Email,
                                 Phone           = i$Phone,
                                 Currency        = i$Currency,
                                 VatRate         = i$VatRate,
                                 ClientId        = i$ClientId,
                                 CountryId       = i$CountryId,
                                 AccountQuality  = ifelse( is.null(i$AccountQuality), NA, i$AccountQuality),
                                 Grants          = paste0(lapply(i$Grants, function(x) paste0(x[["Agency"]], ": ", x[["Privilege"]], " - ", x[["Value"]])), collapse = "; "),
                                 Representatives = paste0(lapply(i$Representatives, function( x ) paste0(x[["Login"]], ": ", x[["Role"]])), collapse = "; "),
                                 Restrictions    = paste0(lapply(i$Restrictions, function(x) paste0(x[["Element"]], ": ", x[["Value"]])), collapse = "; "),
                                 Setting         = paste0(lapply(i$Settings, function(x) paste0(x[["Option"]], ": ", x[["Value"]])), collapse = "; "))))
   }
   
   # binding to data frame
   dataTotal <- bind_rows(data)

   return(dataTotal)
 }
 

yadirGetKeyWords <- function(CampaignIds = NULL, 
                             AdGroupIds  = NA, 
                             Ids         = NA, 
                             States      = c("OFF","ON","SUSPENDED"), 
                             WithStats   = TRUE,
                             Login         = getOption("ryandexdirect.user"),
                             Token         = NULL,
                             AgencyAccount = getOption("ryandexdirect.agency_account"),
                             TokenPath     = yadirTokenPath()){
  
  
  #start time
  start_time  <- Sys.time()
   
  #auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  #check camp id
  if (is.null(CampaignIds)) {
    CampaignIds <-  yadirGetCampaign(Logins        = Login,
                                     AgencyAccount = AgencyAccount,
                                     Token         = Token,
                                     TokenPath     = TokenPath)$Id
  }
  
  #result frame
  result      <- data.frame(Id                            = integer(0), 
                            Keyword                       = character(0),
                            AdGroupId                     = integer(0),
                            CampaignId                    = integer(0),
                            ServingStatus                 = character(0),
                            State                         = character(0),
                            Status                        = character(0),
                            StrategyPriority              = character(0), 
                            StatisticsSearchImpressions   = integer(0),
                            StatisticsSearchClicks        = integer(0),
                            StatisticsNetworkImpressions  = integer(0),
                            StatisticsNetworkClicks       = integer(0),
                            UserParam1                    = character(0),
                            UserParam2                    = character(0),
                            ProductivityValue             = numeric(0),
                            ProductivityReferences        = character(0),
                            Bid                           = integer(0),
                            ContextBid                    = integer(0))
  
 
  States          <- paste("\"",States,"\"",collapse=", ",sep="")
  
  if ( as.integer(length(CampaignIds)) == 0 ) {
      warning(Login, ": This account have not any campaigns")
	  return(result)
  }
  camp_num     <- as.integer(length(CampaignIds))
  camp_start   <- 1
  camp_step    <- 10
  
  units_spent  <- 0
  packageStartupMessage("Processing", appendLF = F)

  while(camp_start <= camp_num){
    
    #composevector camps
    camp_step   <-  if(camp_num - camp_start > 10) camp_step else camp_num - camp_start + 1
    
    #ids
    Ids             <- ifelse(is.na(Ids), NA,paste0(Ids, collapse = ","))
    AdGroupIds      <- ifelse(is.na(AdGroupIds),NA,paste0(AdGroupIds, collapse = ","))
    CampaignIdsTmp  <- paste("\"",CampaignIds[camp_start:(camp_start + camp_step - 1)],"\"",collapse=", ",sep="")
    
    #limits
    lim <- 0
    
    while(lim != "stoped"){
      
      queryBody <- paste0("{
                          \"method\": \"get\",
                          \"params\": {
                          \"SelectionCriteria\": {
                          \"CampaignIds\": [",CampaignIdsTmp,"],
                          ",ifelse(is.na(Ids),"",paste0("\"Ids\": [",Ids,"],")),"        
                          ",ifelse(is.na(AdGroupIds),"",paste0("\"AdGroupIds\": [",AdGroupIds,"],")),"
                          \"States\": [",States,"]
    },
                          
                          \"FieldNames\": [
                              \"Id\",
                              \"CampaignId\",
                              \"AdGroupId\",
                              \"Keyword\",
                              \"UserParam1\",
                              \"UserParam2\",
                              \"Bid\",
                              \"ContextBid\",
                              \"StrategyPriority\",
                              \"Status\",
                              \"ServingStatus\",
                              \"State\",
                              \"Productivity\"
                              ",ifelse(WithStats == F, "",",\"StatisticsSearch\""),
                                ifelse(WithStats == F, "",",\"StatisticsNetwork\""),"],
                          \"Page\": {  
                              \"Limit\": 10000,
                              \"Offset\": ",lim,"}
    }
    }")

      answer <- POST("https://api.direct.yandex.com/json/v5/keywords", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru",'Client-Login' = Login))
      stop_for_status(answer)
      dataRaw <- content(answer, "parsed", "application/json")
      
      #
      if(length(dataRaw$error) > 0){
        stop(paste0(dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
      }
      
      
      #
      for(Keywords_i in 1:length(dataRaw$result$Keywords)){
        result      <- rbind(result,
                             data.frame(Id                            = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$Id), NA,dataRaw$result$Keywords[[Keywords_i]]$Id),
                                        Keyword                       = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$Keyword), NA,dataRaw$result$Keyword[[Keywords_i]]$Keyword),
                                        AdGroupId                     = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$AdGroupId), NA,dataRaw$result$Keywords[[Keywords_i]]$AdGroupId),
                                        CampaignId                    = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$CampaignId), NA,dataRaw$result$Keywords[[Keywords_i]]$CampaignId),
                                        ServingStatus                 = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$ServingStatus), NA,dataRaw$result$Keywords[[Keywords_i]]$ServingStatus),
                                        State                         = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$State), NA,dataRaw$result$Keywords[[Keywords_i]]$State),
                                        Status                        = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$Status), NA,dataRaw$result$Keywords[[Keywords_i]]$Status),
                                        StrategyPriority              = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$StrategyPriority), NA,dataRaw$result$Keywords[[Keywords_i]]$StrategyPriority), 
                                        StatisticsSearchImpressions   = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$StatisticsSearch$Impressions)|WithStats == F, NA,dataRaw$result$Keywords[[Keywords_i]]$StatisticsSearch$Impressions),
                                        StatisticsSearchClicks        = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$StatisticsSearch$Clicks)|WithStats == F, NA,dataRaw$result$Keywords[[Keywords_i]]$StatisticsSearch$Clicks),
                                        StatisticsNetworkImpressions  = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$StatisticsNetwork$Impressions)|WithStats == F, NA,dataRaw$result$Keywords[[Keywords_i]]$StatisticsNetwork$Impressions),
                                        StatisticsNetworkClicks       = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$StatisticsNetwork$Clicks|WithStats) == F, NA,dataRaw$result$Keywords[[Keywords_i]]$StatisticsNetwork$Clicks),
                                        UserParam1                    = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$UserParam1), NA,dataRaw$result$Keywords[[Keywords_i]]$UserParam1),
                                        UserParam2                    = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$UserParam2), NA,dataRaw$result$Keywords[[Keywords_i]]$UserParam2),
                                        ProductivityValue             = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$Productivity$Value), NA,dataRaw$result$Keywords[[Keywords_i]]$Productivity$Value),
                                        ProductivityReferences        = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$Productivity$References), NA,paste(dataRaw$result$Keywords[[Keywords_i]]$Productivity$References,collapse = "," )),
                                        Bid                           = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$Bid), NA,dataRaw$result$Keywords[[Keywords_i]]$Bid / 1000000),
                                        ContextBid                    = ifelse(is.null(dataRaw$result$Keywords[[Keywords_i]]$ContextBid), NA,dataRaw$result$Keywords[[Keywords_i]]$ContextBid / 1000000)))
      }
      #
      packageStartupMessage(".", appendLF = F)
      #
      lim <- ifelse(is.null(dataRaw$result$LimitedBy), "stoped",dataRaw$result$LimitedBy + 1)
	  units_spent <- units_spent + as.integer(strsplit(answer$headers$units, "/")[[1]][1])
    }
    
    #
    camp_start <- camp_start + camp_step
  }
  
  
  # delete NA rows
  result <- subset(result, !is.na(result$Id))
  
  #
  stop_time <- Sys.time()
  
  #
  #sucess meggage
  packageStartupMessage("Done", appendLF = T)
  packageStartupMessage(paste0("KeyWords number: ", nrow(result)), appendLF = T)
  packageStartupMessage(paste0("Number of API points spent when executing the request: " ,units_spent), appendLF = T)
  packageStartupMessage(paste0("Available balance of daily limit API points: " ,strsplit(answer$headers$units, "/")[[1]][2]), appendLF = T)
  packageStartupMessage(paste0("Daily limit of API points:" ,strsplit(answer$headers$units, "/")[[1]][3]), appendLF = T)
  packageStartupMessage(paste0("Duration: ", round(difftime(stop_time, start_time , units ="secs"),0), " sec."), appendLF = T)
  #result
  return(result)}

yadirAuth <- function(Login = getOption("ryandexdirect.user"), NewUser = FALSE, TokenPath = yadirTokenPath()) {
  
  # check dir
  # Create token directory if it does not exist
  if (!dir.exists(TokenPath)) dir.create(TokenPath)
  
  # Set login if not provided
  Login <- if (exists("Login") && !is.null(Login)) Login else getOption("ryandexdirect.user")

  # Normalize path
  TokenPath <- gsub("\\\\", "/", TokenPath)
  token_file <- file.path(TokenPath, paste0(Login, ".yadirAuth.RData"))
  
  gr_type <- "authorization_code"
  id_cl <- "365a2d0a675c462d90ac145d4f5948cc"
  secret_cl <- "f2074f4c312449fab9681942edaa5360"
  
  # Load token if it exists
  if (!NewUser && file.exists(token_file)) {
    message("Load token from ", token_file)
    load(token_file)

    # Check token expiration
    if (as.numeric(difftime(token$expire_at, Sys.time(), units = "days")) < 30) {
      message("Auto refresh token")
      token_raw <- httr::POST(
        "https://oauth.yandex.ru/token",
        body = list(
          grant_type = gr_type,
          refresh_token = token$refresh_token,
          client_id = id_cl,
          client_secret = secret_cl
        ),
        encode = "form"
      )
      
      # Check for errors
      token <- content(token_raw)
      if (!is.null(token$error_description)) stop(paste0(token$error, ": ", token$error_description))
      
      # Update token data
      token$expire_at <- Sys.time() + as.numeric(token$expires_in, units = "secs")
      class(token) <- "yadir_token"
      save(token, file = token_file)
      
      message("Token saved in file ", token_file)
      return(token)
    } else {
      message("Token expires in ", round(difftime(token$expire_at, Sys.time(), units = "days")), " days")
      return(token)
    }
  }
  
  # Check execution mode
  if (!interactive()) {
    stop(sprintf(
      "Function yadirAuth does not find the %s.yadirAuth.RData file in %s. 
    You must run this script in interactive mode (RStudio or RGui) to complete the authorization process. 
    Once authorized, the token file will be saved and can be used in batch mode. 

    More details:
    - Release notes: https://github.com/selesnow/ryandexdirect/releases/tag/3.0.0
    - R modes: https://www.r-bloggers.com/batch-processing-vs-interactive-sessions/",
      Login, TokenPath
    ))
  }
  
  # Start user authorization
  auth_url <- sprintf(
    "https://oauth.yandex.ru/authorize?response_type=code&client_id=365a2d0a675c462d90ac145d4f5948cc&redirect_uri=https://selesnow.github.io/ryandexdirect/getToken/get_code.html&force_confirm=%d%s",
    as.integer(NewUser),
    if (!is.null(Login)) paste0("&login_hint=", Login) else ""
  )
  browseURL(auth_url)
  
  # Enter authorization code
  repeat {
    temp_code <- readline("Enter authorize code:")
    if (nchar(temp_code) == 16) break
    message("The verification code is not 16 characters long, try again.")
  }

  body_data <- list(
    grant_type = gr_type,
    code = temp_code,
    client_id = id_cl,
    client_secret = secret_cl)
  
  response <- request("https://oauth.yandex.ru/token") %>%
    req_body_form(!!!body_data) %>%
    req_perform()
 
  token <- resp_body_json(response)
  token$expire_at <- Sys.time() + token$expires_in
  class(token) <- "yadir_token"

  # check error
  if (!is.null(token$error_description)) {
    stop(paste0(token$error, ": ", token$error_description))
  }

  # print save token file
  save_path <- paste0(TokenPath, "/", Login, ".yadirAuth.RData")
  message("Do you want to save the API credential in a local file (", save_path, ") for use between R sessions?")
  ans <- tolower(readline("y / n (recommendation - y): "))
  
  # save token
  if (ans %in% c("y", "yes", "ok", "save")) {
    save(token, file = save_path)
    message("Token saved in file ", save_path)
  }
  # return token
  return(token)

}

yadirCurrencyRates <- 
function(Login           = getOption("ryandexdirect.user"),
         AgencyAccount   = getOption("ryandexdirect.agency_account"),
         Token           = NULL,
         TokenPath       = yadirTokenPath()){
  
  queryBody <- "{
  \"method\": \"get\",
  \"params\": {
  \"DictionaryNames\": [ \"Currencies\" ]
   }
  }"

  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
answer <- POST("https://api.direct.yandex.com/json/v5/dictionaries", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru","Client-Login" = Login[1]))

stop_for_status(answer)
dataRaw <- content(answer, "parsed", "application/json")

cur <- data.frame(curName = character(),
                  fullName = character(),
                  RateWithVAT = double(),
                  Rate = double(),
                  stringsAsFactors = FALSE)

for(i in 1:length(dataRaw$result$Currencies)){
  cur[i,1] <-  dataRaw$result$Currencies[[i]]$Currency[1]
  cur[i,2] <-  dataRaw$result$Currencies[[i]]$Properties[[2]]$Value
  cur[i,3] <-  as.numeric(dataRaw$result$Currencies[[i]]$Properties[[12]]$Value)
  cur[i,4] <-  as.numeric(dataRaw$result$Currencies[[i]]$Properties[[11]]$Value)
	}
return(cur)
}

yadirGetCampaignList <-
  function (Logins          = getOption("ryandexdirect.user"), 
            States          = c("OFF","ON","SUSPENDED","ENDED","CONVERTED","ARCHIVED"),
            Types           = c("TEXT_CAMPAIGN","MOBILE_APP_CAMPAIGN","DYNAMIC_TEXT_CAMPAIGN","CPM_BANNER_CAMPAIGN"),
            Statuses        = c("ACCEPTED","DRAFT","MODERATION","REJECTED"),
            StatusesPayment = c("DISALLOWED","ALLOWED"),
            Token           = NULL,
            AgencyAccount = getOption("ryandexdirect.agency_account"),
            TokenPath     = yadirTokenPath()) {
    
	# depricating
    .Deprecated(new = "yadirGetCampaign()", 
                package = "ryandexdirect", 
                old = "yadirGetCampaignList()")
				
    # start time
    start_time  <- Sys.time()
    
    # result frame
    result       <- data.frame(Id = character(0),
                               Name = character(0),
                               Type = character(0),
                               Status = character(0),
                               State = character(0),
                               DailyBudgetAmount = double(0),
                               DailyBudgetMode = character(0),
                               Currency = character(0),
                               StartDate = as.Date(character(0)),
                               Impressions = integer(0),
                               Clicks = integer(0),
                               ClientInfo = character(0),
                               TextCampBidStrategySearchType   = character(0),
                               TextCampBidStrategyNetworkType  = character(0),
                               DynCampBidStrategySearchType    = character(0),
                               DynCampBidStrategyNetworkType   = character(0),
                               MobCampBidStrategySearchType    = character(0),
                               MobCampBidStrategyNetworkType   = character(0),
                               CpmBannerBidStrategySearchType  = character(0),
                               CpmBannerBidStrategyNetworkType = character(0),
                               Login = character(0),
                               stringsAsFactors=FALSE)
    
    # filters
    States          <- paste("\"",States,"\"",collapse=", ",sep="")
    Types           <- paste("\"",Types,"\"",collapse=", ",sep="")
    Statuses        <- paste("\"",Statuses,"\"",collapse=", ",sep="")
    StatusesPayment <- paste("\"",StatusesPayment,"\"",collapse=", ",sep="")
    
    # offset
    lim <- 0
    
    # start message
    packageStartupMessage("Processing", appendLF = F)
    
    while(lim != "stoped") {  
      # compose query body
      queryBody <- paste0("{
                          \"method\": \"get\",
                          \"params\": { 
                          \"SelectionCriteria\": {
                          \"States\": [",States,"],        
                          \"Types\": [",Types,"],
                          \"StatusesPayment\": [",StatusesPayment,"],
                          \"Statuses\": [",Statuses,"]},
                          \"FieldNames\": [
                                      \"Id\",
                                      \"Name\",
                                      \"Type\",
                                      \"StartDate\",
                                      \"Status\",
                                      \"State\",
                                      \"Statistics\",
                                      \"Currency\",
                                      \"DailyBudget\",
                                      \"ClientInfo\"],
                          \"TextCampaignFieldNames\": [\"BiddingStrategy\"],
                          \"MobileAppCampaignFieldNames\": [\"BiddingStrategy\"],
                          \"DynamicTextCampaignFieldNames\": [\"BiddingStrategy\"],
                          \"CpmBannerCampaignFieldNames\": [\"BiddingStrategy\"],
                          \"Page\": {  
                          \"Limit\": 10000,
                          \"Offset\": ",lim,"
    }
    }
    }")

      
      
      for(l in 1:length(Logins)){
        # auth
        Token <- tech_auth(login = Logins[l], token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
        
        answer <- POST("https://api.direct.yandex.com/json/v5/campaigns", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru","Client-Login" = Logins[l]))
        # check answer status
        stop_for_status(answer)
        dataRaw <- content(answer, "parsed", "application/json")
        
        if(length(dataRaw$error) > 0){
          stop(paste0(dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
        }
        
        # parsing
        for (i in 1:length(dataRaw$result$Campaigns)){
          
          try(result <- rbind(result,
                              data.frame(Id                              = dataRaw$result$Campaigns[[i]]$Id,
                                         Name                            = dataRaw$result$Campaigns[[i]]$Name,
                                         Type                            = dataRaw$result$Campaigns[[i]]$Type,
                                         Status                          = dataRaw$result$Campaigns[[i]]$Status,
                                         State                           = dataRaw$result$Campaigns[[i]]$State,
                                         DailyBudgetAmount               = ifelse(is.null(dataRaw$result$Campaigns[[i]]$DailyBudget$Amount), NA, dataRaw$result$Campaigns[[i]]$DailyBudget$Amount / 1000000),
                                         DailyBudgetMode                 = ifelse(is.null(dataRaw$result$Campaigns[[i]]$DailyBudget$Mode), NA, dataRaw$result$Campaigns[[i]]$DailyBudget$Mode),
                                         Currency                        = dataRaw$result$Campaigns[[i]]$Currency,
                                         StartDate                       = dataRaw$result$Campaigns[[i]]$StartDate,
                                         Impressions                     = ifelse(is.null(dataRaw$result$Campaigns[[i]]$Statistics$Impressions), NA,dataRaw$result$Campaigns[[i]]$Statistics$Impressions),
                                         Clicks                          = ifelse(is.null(dataRaw$result$Campaigns[[i]]$Statistics$Clicks), NA,dataRaw$result$Campaigns[[i]]$Statistics$Clicks),
                                         ClientInfo                      = dataRaw$result$Campaigns[[i]]$ClientInfo,
                                         TextCampBidStrategySearchType   = ifelse(is.null(dataRaw$result$Campaigns[[i]]$TextCampaign$BiddingStrategy$Search$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$TextCampaign$BiddingStrategy$Search$BiddingStrategyType),
                                         TextCampBidStrategyNetworkType  = ifelse(is.null(dataRaw$result$Campaigns[[i]]$TextCampaign$BiddingStrategy$Network$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$TextCampaign$BiddingStrategy$Network$BiddingStrategyType),
                                         DynCampBidStrategySearchType    = ifelse(is.null(dataRaw$result$Campaigns[[i]]$DynamicTextCampaign$BiddingStrategy$Search$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$DynamicTextCampaign$BiddingStrategy$Search$BiddingStrategyType),
                                         DynCampBidStrategyNetworkType   = ifelse(is.null(dataRaw$result$Campaigns[[i]]$DynamicTextCampaign$BiddingStrategy$Network$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$DynamicTextCampaign$BiddingStrategy$Network$BiddingStrategyType),
                                         MobCampBidStrategySearchType    = ifelse(is.null(dataRaw$result$Campaigns[[i]]$MobileAppCampaign$BiddingStrategy$Search$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$MobileAppCampaign$BiddingStrategy$Search$BiddingStrategyType),
                                         MobCampBidStrategyNetworkType   = ifelse(is.null(dataRaw$result$Campaigns[[i]]$MobileAppCampaign$BiddingStrategy$Network$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$MobileAppCampaign$BiddingStrategy$Network$BiddingStrategyType),
                                         CpmBannerBidStrategySearchType  = ifelse(is.null(dataRaw$result$Campaigns[[i]]$CpmBannerCampaign$BiddingStrategy$Search$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$CpmBannerCampaign$BiddingStrategy$Search$BiddingStrategyType),
                                         CpmBannerBidStrategyNetworkType = ifelse(is.null(dataRaw$result$Campaigns[[i]]$CpmBannerCampaign$BiddingStrategy$Network$BiddingStrategyType), "", dataRaw$result$Campaigns[[i]]$CpmBannerCampaign$BiddingStrategy$Network$BiddingStrategyType),
                                         Login                           = Logins[l])), silent = T)
          
        }
      }
      
      # add progres
      packageStartupMessage(".", appendLF = F)
      # check for next iteraction
      lim <- ifelse(is.null(dataRaw$result$LimitedBy), "stoped",dataRaw$result$LimitedBy + 1)
    }
    
    # convert to factor
    result$Type     <- as.factor(result$Type)
    result$Status   <- as.factor(result$Status)
    result$State    <- as.factor(result$State)
    result$Currency <- as.factor(result$Currency)
    result$SearchBidStrategyType  <- paste0(result$TextCampBidStrategySearchType, result$DynCampBidStrategySearchType, result$MobCampBidStrategySearchType, result$CpmBannerBidStrategySearchType)
    result$NetworkBidStrategyType <- paste0(result$TextCampBidStrategyNetworkType, result$DynCampBidStrategyNetworkType, result$MobCampBidStrategyNetworkType, result$CpmBannerBidStrategyNetworkType)
    
    # removing
    result$TextCampBidStrategySearchType   <- NULL
    result$TextCampBidStrategyNetworkType  <- NULL
    result$DynCampBidStrategySearchType    <- NULL
    result$DynCampBidStrategyNetworkType   <- NULL
    result$MobCampBidStrategySearchType    <- NULL
    result$MobCampBidStrategyNetworkType   <- NULL
    result$CpmBannerBidStrategySearchType  <- NULL
    result$CpmBannerBidStrategyNetworkType <- NULL
    
    # end timr
    stop_time <- Sys.time()
    
    # out message
    packageStartupMessage("Done", appendLF = T)
    packageStartupMessage(paste0("Number of load campaings: ", nrow(result)), appendLF = T)
    packageStartupMessage(paste0("Processing durations: ", round(difftime(stop_time, start_time , units ="secs"),0), " sec."), appendLF = T)
    
    # result
    return(result)
  }

# tech block auth
tech_auth <-  function(login = NULL, token = NULL, AgencyAccount = NULL, TokenPath = NULL) {
  
  # check options
  if ( is.null(token) ) {
    ## login
    if ( is.null(login) && ! is.null(getOption("ryandexdirect.user") ) ) {
      
      login <- getOption("ryandexdirect.user")
      
    }
    
    ## agency
    if ( is.null(AgencyAccount) && ! is.null(getOption("ryandexdirect.agency_account") ) ) {
      
      AgencyAccount <- getOption("ryandexdirect.agency_account")
      
    }
    
  }

  # if token not null skip auth
  if (! is.null(token) ) {
    # detect class of token object
    if(inherits(token, "list")) {
      Token <- token$access_token 
    } else {
      Token <- token
    }
  # if token is null go auth
  } else {
    # detect of account type, agency or client
    load_login <- ifelse(is.null(AgencyAccount) || is.na(AgencyAccount), login, AgencyAccount)
    # auth
    Token <- yadirAuth(Login = load_login, TokenPath = TokenPath, NewUser = FALSE)$access_token
  }
  
  # return token obj
  return(Token)
}

yadirTokenPath <- function() {
  
  if ( ! is.null(getOption("ryandexdirect.token_path")) ) {
    
    return(getOption("ryandexdirect.token_path"))
    
  } else {
    
    return(getwd())
    
  }
  
}

.onAttach <- function(lib, pkg,...){
  packageStartupMessage(ryandexdirectWelcomeMessage())
  
  if ( any( ! is.null(getOption("ryandexdirect.token_path") ),
            ! is.null(getOption("ryandexdirect.user") ),
			! is.null(getOption("ryandexdirect.agency_account") ) 
			) 
	 ) {
	 
	 packageStartupMessage("\n ..ryandexdirect Auth Presets:")
	 
	}
  
  ## token path
  if ( ! is.null(getOption("ryandexdirect.token_path") ) ) {
    packageStartupMessage("...Set ryandexdirect TokenPath: ", appendLF = F)
    packageStartupMessage(getOption("ryandexdirect.token_path"))
  } 
  
  ## username
  if ( ! is.null(getOption("ryandexdirect.user") ) ) {
    packageStartupMessage("...Set ryandexdirect Login: ", appendLF = F)
    packageStartupMessage(getOption("ryandexdirect.user"))
  } 
  
  ## agency
  if ( ! is.null(getOption("ryandexdirect.agency_account") ) ) {
    packageStartupMessage("...Set ryandexdirect AgencyAccount: ", appendLF = F)
    packageStartupMessage(getOption("ryandexdirect.agency_account"))
  } 
}

#
#

ryandexdirectWelcomeMessage <- function(){
  # library(utils)
  
  paste0("\n",
         "---------------------\n",
         "Welcome to ryandexdirect version ", utils::packageDescription("ryandexdirect")$Version, "\n",
         "\n",
         "Author:           Alexey Seleznev (Head of analytics dept at Netpeak).\n",
		     "Telegram channel: https://t.me/R4marketing \n",
		     "YouTube channel:  https://www.youtube.com/R4marketing/?sub_confirmation=1 \n",
         "Email:            selesnow@gmail.com\n",
		     "Site:             https://selesnow.github.io \n",
         "Blog:             https://alexeyseleznev.wordpress.com \n",
         "Facebook:         https://facebook.com/selesnown \n",
         "Linkedin:         https://www.linkedin.com/in/selesnow \n",
         "\n",
         "Type ?ryandexdirect for the main documentation.\n",
		     "The site of ryandexdirect: https://selesnow.github.io/ryandexdirect/ \n",
         "The github page is: https://github.com/selesnow/ryandexdirect/\n",
         "\n",
         "Suggestions and bug-reports can be submitted at: https://github.com/selesnow/ryandexdirect/issues\n",
         "Or contact: <selesnow@gmail.com>\n",
         "\n",
         "\tTo suppress this message use:  ", "suppressPackageStartupMessages(library(ryandexdirect))\n",
         "---------------------\n"
  )
}

.onLoad <- function(libname, pkgname) {
  
  op <- options()
  
  op.ryandexdirect <- list(ryandexdirect.user           = Sys.getenv("RYD_USER"),
                           ryandexdirect.token_path     = Sys.getenv("RYD_TOKEN_PATH"),
						               ryandexdirect.agency_account = Sys.getenv("RYD_AGENCY"))
  
  op.ryandexdirect <- lapply(op.ryandexdirect, function(x) if ( x == "" ) return(NULL) else return(x))
  
  toset <- !(names(op.ryandexdirect) %in% names(op))
  if (any(toset)) options(op.ryandexdirect[toset])
  
  invisible()
}

if (getRversion() >= "2.15.1") {
  utils::globalVariables(c("Currency", "State", "Status", "Type"))
}

yadirGetAdGroups <- function(CampaignIds   = NULL, 
                             Ids           = NA, 
                             Types         = c("TEXT_AD_GROUP", "MOBILE_APP_AD_GROUP", "DYNAMIC_TEXT_AD_GROUP",
                                               "CPM_BANNER_AD_GROUP", "CPM_VIDEO_AD_GROUP", "SMART_AD_GROUP"),
                             Statuses      = c( "ACCEPTED", "DRAFT", "MODERATION", "PREACCEPTED", "REJECTED"), 
                             Login         = getOption("ryandexdirect.user"),
                             AgencyAccount = getOption("ryandexdirect.agency_account"),
                             Token         = NULL,
                             TokenPath     = yadirTokenPath()){
  
  # set start time
  start_time  <- Sys.time()
  
  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  # result frame
  result      <- data.frame(Id                                                   = integer(0), 
                            Name                                                 = character(0),
                            CampaignId                                           = integer(0),
                            Type                                                 = character(0),
                            Subtype                                              = character(0),
                            Status                                               = character(0),
                            ServingStatus                                        = character(0),
                            NegativeKeywords                                     = character(0),
                            TrackingParams                                       = character(0),
                            RegionIds                                            = character(0),
                            RestrictedRegionIds                                  = character(0),
                            MobileAppAdGroupStoreUrl                             = character(0),
                            MobileAppAdGroupTargetDeviceType                     = character(0),
                            MobileAppAdGroupTargetCarrier                        = character(0),
                            MobileAppAdGroupTargetOperatingSystemVersion         = character(0),
                            MobileAppAdGroupAppIconModerationStatus              = character(0),
                            MobileAppAdGroupAppIconModerationStatusClarification = character(0),
                            MobileAppAdGroupAppOperatingSystemType               = character(0),
                            MobileAppAdGroupAppAvailabilityStatus                = character(0),
                            DynamicTextAdGroupDomainUrl                          = character(0),
                            DynamicTextAdGroupDomainUrlProcessingStatus          = character(0),
                            DynamicTextFeedAdGroupSource                         = character(0),
                            DynamicTextFeedAdGroupSourceType                     = character(0),
                            DynamicTextFeedAdGroupSourceProcessingStatus         = character(0),
                            SmartAdGroupFeedId                                   = character(0),
                            SmartAdGroupAdTitleSource                            = character(0),
                            SmartAdGroupAdBodySource                             = character(0))                 
  
  # check ids
  if (is.null(CampaignIds)) {
    CampaignIds <-  yadirGetCampaign(Logins        = Login,
                                     AgencyAccount = AgencyAccount,
                                     Token         = Token,
                                     TokenPath     = TokenPath)$Id
  }
  
  # convert filters to JSON
  Statuses          <- paste("\"",Statuses,"\"",collapse=", ",sep="")
  Types             <- paste("\"",Types,"\"",collapse=", ",sep="")
  
  # define camp numbers
  camp_num     <- as.integer(length(CampaignIds))
  camp_start   <- 1
  camp_step    <- 10
  
  packageStartupMessage("Processing", appendLF = F)
  # start cycle
  while(camp_start <= camp_num){
    
    #camp num
    camp_step   <-  if(camp_num - camp_start >= 10) camp_step else camp_num - camp_start + 1
    
    
    Ids             <- ifelse(is.na(Ids), NA, paste0(Ids, collapse = ","))
    CampaignIdsTmp  <- paste("\"",CampaignIds[camp_start:(camp_start + camp_step - 1)],"\"",collapse=", ",sep="")
    
    # set offset
    lim <- 0
    
    while(lim != "stoped"){
      
      queryBody <- paste0("{
                          \"method\": \"get\",
                          \"params\": {
                          \"SelectionCriteria\": {
                              \"CampaignIds\": [",CampaignIdsTmp,"],
                              ",ifelse(is.na(Ids),"",paste0("\"Ids\": [",Ids,"],")),"      
                              \"Types\": [",Types,"],
                              \"Statuses\": [",Statuses,"]},
                          \"FieldNames\": [
                              \"Id\",
                              \"Name\",
                              \"CampaignId\",
                              \"RegionIds\",
                              \"RestrictedRegionIds\",
                              \"NegativeKeywords\",
                              \"TrackingParams\",
                              \"Status\",
                              \"ServingStatus\",
                              \"Type\",
                              \"Subtype\"],
                         \"MobileAppAdGroupFieldNames\":[
                              \"StoreUrl\",
                              \"TargetDeviceType\",
                              \"TargetCarrier\",
                              \"TargetOperatingSystemVersion\",
                              \"AppIconModeration\",
                              \"AppOperatingSystemType\",
                              \"AppAvailabilityStatus\"],
                         \"DynamicTextAdGroupFieldNames\":[
                              \"DomainUrl\",
                              \"DomainUrlProcessingStatus\"],
                         \"DynamicTextFeedAdGroupFieldNames\":[
                              \"Source\",
                              \"SourceType\",
                              \"SourceProcessingStatus\"],
                         \"SmartAdGroupFieldNames\":[
                              \"FeedId\",
                              \"AdTitleSource\",
                              \"AdBodySource\"],
                          \"Page\": {  
                          \"Limit\": 10000,
                          \"Offset\": ",lim,"}
    }
    }")
      
      answer <- POST("https://api.direct.yandex.com/json/v5/adgroups", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru",'Client-Login' = Login))
      stop_for_status(answer)
      dataRaw <- content(answer, "parsed", "application/json")
      
      #check answer for error
      if(length(dataRaw$error) > 0){
        stop(paste0(dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
      }
      
      # pars
      for(adgroups_i in 1:length(dataRaw$result$AdGroups)){
        result      <- rbind(result,
                             data.frame(Id                                                   = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$Id), NA,dataRaw$result$AdGroups[[adgroups_i]]$Id), 
                                        Name                                                 = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$Name), NA,dataRaw$result$AdGroups[[adgroups_i]]$Name),
                                        CampaignId                                           = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$CampaignId), NA,dataRaw$result$AdGroups[[adgroups_i]]$CampaignId),
                                        Type                                                 = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$Type), NA,dataRaw$result$AdGroups[[adgroups_i]]$Type),
                                        Subtype                                              = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$Subtype), NA,dataRaw$result$AdGroups[[adgroups_i]]$Subtype),
                                        Status                                               = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$Status), NA,dataRaw$result$AdGroups[[adgroups_i]]$Status),
                                        ServingStatus                                        = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$ServingStatus), NA,dataRaw$result$AdGroups[[adgroups_i]]$ServingStatus),
                                        NegativeKeywords                                     = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$NegativeKeywords$Items), NA,paste0(dataRaw$result$AdGroups[[adgroups_i]]$NegativeKeywords$Items, collapse = ",")),
                                        TrackingParams                                       = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$TrackingParams), NA,dataRaw$result$AdGroups[[adgroups_i]]$TrackingParams),
                                        RegionIds                                            = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$RegionIds), NA,paste0(dataRaw$result$AdGroups[[adgroups_i]]$RegionIds, collapse = ",")),
                                        RestrictedRegionIds                                  = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$RestrictedRegionIds$Items), NA,paste0(dataRaw$result$AdGroups[[adgroups_i]]$RestrictedRegionIds$Items, collapse = ",")),
                                        MobileAppAdGroupStoreUrl                             = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$StoreUrl), NA,dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$StoreUrl),
                                        MobileAppAdGroupTargetDeviceType                     = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$TargetDeviceType), NA,dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$TargetDeviceType),
                                        MobileAppAdGroupTargetCarrier                        = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$TargetCarrier), NA,dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$TargetCarrier),
                                        MobileAppAdGroupTargetOperatingSystemVersion         = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$TargetOperatingSystemVersion), NA,dataRaw$result$AdGroups[[adgroups_i]]$TargetOperatingSystemVersion),
                                        MobileAppAdGroupAppIconModerationStatus              = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$AppIconModeration$Status), NA,dataRaw$result$AdGroups[[adgroups_i]]$AppIconModeration$Status),
                                        MobileAppAdGroupAppIconModerationStatusClarification = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$AppIconModeration$StatusClarification), NA,dataRaw$result$AdGroups[[adgroups_i]]$AppIconModeration$StatusClarification),
                                        MobileAppAdGroupAppOperatingSystemType               = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$OperatingSystemType), NA,dataRaw$result$AdGroups[[adgroups_i]]$OperatingSystemType),
                                        MobileAppAdGroupAppAvailabilityStatus                = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$MobileAppAdGroup$AppAvailabilityStatus), NA,dataRaw$result$AdGroups[[adgroups_i]]$AppAvailabilityStatus),
                                        DynamicTextAdGroupDomainUrl                          = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextAdGroup$DomainUrl), NA,dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextAdGroup$DomainUrl),
                                        DynamicTextAdGroupDomainUrlProcessingStatus          = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextAdGroup$DomainUrlProcessingStatus), NA,dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextAdGroup$DomainUrlProcessingStatus),
                                        DynamicTextFeedAdGroupSource                         = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextFeedAdGroup$Source), NA,dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextFeedAdGroup$Source),
                                        DynamicTextFeedAdGroupSourceType                     = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextFeedAdGroup$SourceType), NA,dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextFeedAdGroup$SourceType),
                                        DynamicTextFeedAdGroupSourceProcessingStatus         = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextFeedAdGroup$SourceProcessingStatus), NA,dataRaw$result$AdGroups[[adgroups_i]]$DynamicTextFeedAdGroup$SourceProcessingStatus),
                                        SmartAdGroupFeedId                                   = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$SmartAdGroup$FeedId), NA,dataRaw$result$AdGroups[[adgroups_i]]$SmartAdGroup$FeedId),
                                        SmartAdGroupAdTitleSource                            = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$SmartAdGroup$AdTitleSource), NA,dataRaw$result$AdGroups[[adgroups_i]]$SmartAdGroup$AdTitleSource),
                                        SmartAdGroupAdBodySource                             = ifelse(is.null(dataRaw$result$AdGroups[[adgroups_i]]$SmartAdGroup$AdBodySource), NA,dataRaw$result$AdGroups[[adgroups_i]]$SmartAdGroup$AdBodySource)
                                        )
                             )
      }
      # progressbar
      packageStartupMessage(".", appendLF = F)
      # check last iteration
      lim <- ifelse(is.null(dataRaw$result$LimitedBy), "stoped",dataRaw$result$LimitedBy + 1)
    }
    
    # next campaings
    camp_start <- camp_start + camp_step
  }
  
  # set finish time
  stop_time <- Sys.time()
  
  # message
  packageStartupMessage("Done", appendLF = T)
  packageStartupMessage(paste0("Number of ad groups: ", nrow(result)), appendLF = T)
  packageStartupMessage(paste0("Duration: ", round(difftime(stop_time, start_time , units ="secs"),0), " sec."), appendLF = T)
  # result
  return(result)}

yadirGetReportFun <- 
     function(ReportType        = "CUSTOM_REPORT", 
              DateRangeType     = "CUSTOM_DATE", 
              DateFrom          = Sys.Date() - 31, 
              DateTo            = Sys.Date() - 1,
              FieldNames        = c("CampaignName","Impressions","Clicks","Cost"), 
              FilterList        = NULL,
              Goals             = NULL,
              AttributionModels = NULL,
              IncludeVAT        = "YES",
              IncludeDiscount   = "NO",
              Login             = getOption("ryandexdirect.user"),
              AgencyAccount     = getOption("ryandexdirect.agency_account"),
              Token             = NULL,
              TokenPath         = yadirTokenPath(),
			  SkipErrors        = TRUE){
  
  # start time
  proc_start <- Sys.time()
  
  # field names
  Fields <- paste0("<FieldNames>",FieldNames, "</FieldNames>", collapse = "")
  
  # for limited accounts
  limit_reached <- c()
  
  # filters
  if(!is.null(FilterList)){
    fil_list <- NA
    filt <- FilterList
    for(fil in filt){
      val <- strsplit(paste0(strsplit(fil ," ")[[1]][3:length(strsplit(fil ," ")[[1]])], collapse = ""), split = ",| |;")[[1]]
      fil_list <- paste0(fil_list[!is.na(fil_list)],
                         paste0("<Filter>",
                                paste0("<Field>",strsplit(fil ," ")[[1]][1], "</Field>"),
                                paste0("<Operator>",strsplit(fil ," ")[[1]][2], "</Operator>"),
                                paste0(paste0("<Values>",val, "</Values>"), collapse = ""),"</Filter>"))
    }}
  
  # Goals and attributtion
  Goals             <- ifelse(is.null(Goals), "", paste0("<Goals>",Goals, "</Goals>", collapse = ""))
  AttributionModels <- ifelse(is.null(AttributionModels), "", paste0("<AttributionModels>",AttributionModels, "</AttributionModels>", collapse = ""))
 
   # compose request body
  queryBody <- paste0('
                      <ReportDefinition xmlns="http://api.direct.yandex.com/v5/reports">
                      <SelectionCriteria>',
                      ifelse(DateRangeType == "CUSTOM_DATE",paste0("<DateFrom>",DateFrom,"</DateFrom>","<DateTo>",DateTo,"</DateTo>") ,"" ),
                      ifelse(is.null(FilterList),"",fil_list),
                      '
                      </SelectionCriteria>',
                      Goals,AttributionModels,Fields,'
                      <ReportName>',paste0("MyReport ", Sys.time()),'</ReportName>
                      <ReportType>',ReportType,'</ReportType>
                      <DateRangeType>',DateRangeType ,'</DateRangeType>
                      <Format>TSV</Format>
                      <IncludeVAT>',IncludeVAT,'</IncludeVAT>
                      <IncludeDiscount>',IncludeDiscount,'</IncludeDiscount>
                      </ReportDefinition>')
  
  # result frame
  result <- data.frame()
  
  for(current_login in Login){
    # auth
    curToken <- tech_auth(login = current_login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
   
    # time counter
    parsing_time <- 0
    server_time <- 0
    # start message
    packageStartupMessage("-----------------------------------------------------------")
    packageStartupMessage(paste0("Loading data by ",current_login))
    
	# send request to API
    serv_start_time <- Sys.time()
    
    answer <- POST("https://api.direct.yandex.com/v5/reports", 
	               body = queryBody, 
	               add_headers( Authorization       = paste0("Bearer ",curToken), 
				               'Accept-Language'    = "ru", 
							    skipReportHeader    = "true" ,
								skipReportSummary   = "true" , 
								'Client-Login'      = current_login, 
								returnMoneyInMicros = "false", 
								processingMode      = "auto"))
    
    if(substr(answer$status_code,1,1) == 4){
      packageStartupMessage("Error in request parameters or limit on the number of requests or reports in the queue is exceeded. In this case, analyze the error message, correct the request and send it again.")
      
      # parse error
      content(answer, "parsed","text/xml",encoding = "UTF-8") %>%
          xml_find_all(xpath = ".//reports:ApiError//reports:requestId") %>%
          xml_text() %>%
          message(" > Request Id")
      
      content(answer, "parsed","text/xml",encoding = "UTF-8") %>%
          xml_find_all(xpath = ".//reports:ApiError//reports:errorCode") %>%
          xml_text() %>%
          message(" > Error Code")
      
      content(answer, "parsed","text/xml",encoding = "UTF-8") %>%
          xml_find_all(xpath = ".//reports:ApiError//reports:errorMessage") %>%
          xml_text() %>%
          message(" > Error Message")
      
      content(answer, "parsed","text/xml",encoding = "UTF-8") %>%
          xml_find_all(xpath = ".//reports:ApiError//reports:errorDetail") %>%
          xml_text() %>%
          message(" > Error Detail")
		  
	  if ( SkipErrors == FALSE ) {
                    
	                  err_detail <- content(answer, "parsed","text/xml",encoding = "UTF-8") %>%
              	                    xml_find_all(xpath = ".//reports:ApiError//reports:errorDetail") %>%
              	                    xml_text()
	                  
	                  stop(err_detail)
	  } else { 
      next
	  }
    }
    
    if(answer$status_code == 500){
      packageStartupMessage(paste0(current_login," - ",xml_text(content(answer, "parsed","text/xml",encoding = "UTF-8"))))
      packageStartupMessage("While generating the report an error occurred on the server. If for this report the error on the server occurred for the first time, try to generate a report again. If the error persists, contact support.")
	  
	  if ( SkipErrors == FALSE ) {
	                 
	                  err_detail <- content(answer, "parsed","text/xml",encoding = "UTF-8") %>%
                  	                    xml_find_all(xpath = ".//reports:ApiError//reports:errorDetail") %>%
                  	                    xml_text()
	                  
	                  stop(err_detail)
	  } else {
      next
	    }
    }
    
    if(answer$status_code == 201){
      packageStartupMessage("The report has been successfully queued for offline generation.", appendLF = T)
      packageStartupMessage("Proccesing", appendLF = F)
      packageStartupMessage("|", appendLF = F)
    }
    
    if(answer$status_code == 202){
      packageStartupMessage("Report generation is not completed yet.", appendLF = F)
    }
    
    
    while(answer$status_code != 200){
      answer <- POST("https://api.direct.yandex.com/v5/reports", 
	                 body = queryBody, 
					 add_headers( Authorization       = paste0("Bearer ",curToken), 
					             'Accept-Language'    = "ru", 
								  skipReportHeader    = "true" ,
								  skipReportSummary   = "true" , 
								 'Client-Login'       = current_login, 
								  returnMoneyInMicros = "false", 
								  processingMode      = "auto"))
								  
      packageStartupMessage("=", appendLF = F)
      if(answer$status_code == 500){
        stop("While generating the report an error occurred on the server. If for this report the error on the server occurred for the first time, try to generate a report again. If the error persists, contact support.")
      }
      if(answer$status_code == 502){
        stop("Request processing time exceeded server limit.")
      }
      Sys.sleep(5)
    }
    packageStartupMessage("|", appendLF = T)
    # succes message
    server_time <- round(difftime(Sys.time(), serv_start_time , units ="secs"),0)
    packageStartupMessage("Report successfully generated and transmitted in response body.", appendLF = T)
    packageStartupMessage(paste0("Server time: ",server_time , " sec."), appendLF = T)
    
    # pars start time
    pasr_start_time <- Sys.time()
    
    if(answer$status_code == 200){
      df_new <- suppressMessages(content(answer,  "parsed", "text/tab-separated-values", encoding = "UTF-8"))

      # check data
      if(nrow(df_new) == 0){
        packageStartupMessage("Your request did not return any data, carefully check the specified filter and report period, and then try again.")
        next
      }
	  
	  # check if there are APi limits
      if (nrow(df_new) >= 1000000) { 
        warning(current_login,': You have reached the limits of Yandex.Direct API. Try to use "FetchBy" parameter with DateRangeType = "CUSTOM_DATE", "DateFrom" and "DateTo". If you are already using it, try to choose a smaller value.')
		limit_reached <- append(limit_reached, current_login)
      }
	  
      # timing message
      parsing_time <- round(difftime(Sys.time(), pasr_start_time , units ="secs"),0)
      packageStartupMessage(paste0("Parsing time: ", parsing_time, " sec."), appendLF = T)

      # request id
      packageStartupMessage(paste0("RequestID: ",answer$headers$requestid), appendLF = T)
      
      # add login
      if(length(Login) > 1){
        df_new$Login <- current_login}
      
      # add to result
      result <- rbind(result, df_new)
      # end of cycle
    }
  }
  
  # check limits
  if ( length(limit_reached) > 0 ) {
   attr(result, "limit_reached") <- limit_reached
   packageStartupMessage("Limit of 1 000 000 rows reached by next accounts: ", paste(limit_reached, collapse = ", "))
  }  
  
  # tech messages
  total_work_time <- round(difftime(Sys.time(), proc_start , units ="secs"),0)
  packageStartupMessage(paste0("Total time: ",total_work_time, " sec."))
  packageStartupMessage(paste0(round(as.integer(server_time) / as.integer(total_work_time) * 100, 0), "% server time rate."))
  packageStartupMessage(paste0(round(as.integer(parsing_time) / as.integer(total_work_time) * 100, 0), "% parsing time rate."))
  packageStartupMessage("Number of rows is ", nrow(df_new))
  packageStartupMessage(paste0("RequestID: ",answer$headers$requestid))
  packageStartupMessage("-----------------------------------------------------------")
  # return result
  return(result)
}

yadirGetAds <- function(CampaignIds   = NULL,
                        AdGroupIds    = NA,
                        Ids           = NA,
                        Types         = c("TEXT_AD", "MOBILE_APP_AD", "DYNAMIC_TEXT_AD", "IMAGE_AD",
                                          "CPC_VIDEO_AD", "CPM_BANNER_AD", "CPM_VIDEO_AD", "SMART_AD"),
                        States        = c("OFF","ON","SUSPENDED","OFF_BY_MONITORING","ARCHIVED"),
                        Login         = getOption("ryandexdirect.user"),
                        Token         = NULL,
                        AgencyAccount = getOption("ryandexdirect.agency_account"),
                        TokenPath     = yadirTokenPath()){

  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)

  # check campaign filter
  if ( is.null(CampaignIds) ) {
    message("You dont choised any ids of campaign, adgroup or ad. Loading full campaign list.")
    CampaignIds <-  yadirGetCampaign(Logins        = Login,
                                     AgencyAccount = AgencyAccount,
                                     Token         = Token,
                                     TokenPath     = TokenPath)$Id
  }

  # set start time
  start_time  <- Sys.time()

  # result frame
  result      <- data.frame(Id                            = integer(0),
                            AdGroupId                     = integer(0),
                            CampaignId                    = integer(0),
                            Type                          = character(0),
                            Subtype                       = character(0),
                            Status                        = character(0),
                            AgeLabel                      = character(0),
                            State                         = character(0),
                            TextAdTitle                   = character(0),
                            TextAdTitle2                  = character(0),
                            TextAdText                    = character(0),
                            TextAdHref                    = character(0),
                            TextAdDisplayDomain           = character(0),
                            TextAdMobile                  = character(0),
                            TextImageAdHref               = character(0),
                            TextImageAdTurboPageId         = character(0),
                            TextMediaAdHref               = character(0),
                            TextAdSitelinkSetId           = integer(0),
                            DynamicTextAdText             = character(0),
                            DynamicTextAdSitelinkSetId    = integer(0),
                            SmartAdBuilderAdCreative      = character(0),
                            SmartAdBuilderThumbnailUrl    = character(0),
                            SmartAdBuilderPreviewUrl      = character(0),
                            CpmBannerAdBuilderHref        = character(0),
                            CpmBannerAdBuilderTurboPageId = character(0),
                            CpcVideoAdBuilderHref         = character(0),
                            CpcVideoAdBuilderTurboPageId  = character(0),
                            CpmVideoAdBuilderHref         = character(0),
                            CpmVideoAdBuilderTurboPageId  = character(0))

  # states to json for query
  States          <- paste("\"",States,"\"",collapse=", ",sep="")
  Types           <- paste("\"",Types,"\"",collapse=", ",sep="")

  # camp queue
  camp_num     <- as.integer(length(CampaignIds))
  camp_start   <- 1
  camp_step    <- 10

  packageStartupMessage("Processing", appendLF = F)

  # campaign cicle
  while(camp_start <= camp_num){

    # how ,ani campaing need processing
    camp_step   <-  if(camp_num - camp_start >= 10) camp_step else camp_num - camp_start + 1

    # prepare camp list for iteration
    Ids             <- ifelse(any(is.na(Ids)), NA,paste0(Ids, collapse = ","))
    AdGroupIds      <- ifelse(any(is.na(AdGroupIds)),NA,paste0(AdGroupIds, collapse = ","))
    CampaignIdsTmp  <- paste("\"",CampaignIds[camp_start:(camp_start + camp_step - 1)],"\"",collapse=", ",sep="")

    # lim offset
    lim <- 0

    while(lim != "stoped"){

      queryBody <- paste0("{
                          \"method\": \"get\",
                          \"params\": {
                          \"SelectionCriteria\": {
                          \"CampaignIds\": [", CampaignIdsTmp, "],
                          ", ifelse(is.na(Ids), "", paste0("\"Ids\": [",Ids, "],")), "
                          ",ifelse(is.na(AdGroupIds), "", paste0("\"AdGroupIds\": [", AdGroupIds, "],")),"
                          \"States\": [",States, "],
                          \"Types\": [",Types,"]
    },
                          \"FieldNames\": [
                              \"Id\",
                              \"CampaignId\",
                              \"AdGroupId\",
                              \"Status\",
                              \"State\",
                              \"AgeLabel\",
                              \"Type\",
                              \"Subtype\"],
                          \"TextAdFieldNames\": [
                              \"Title\",
                              \"Title2\",
                              \"Text\",
                              \"Href\",
                              \"Mobile\",
                              \"SitelinkSetId\",
                              \"DisplayDomain\"],
                          \"TextImageAdFieldNames\": [
                              \"Href\"],
                          \"DynamicTextAdFieldNames\": [
                              \"Text\",
                              \"SitelinkSetId\"],
                          \"CpcVideoAdBuilderAdFieldNames\": [
                              \"Creative\",
                              \"Href\",
                              \"TurboPageId\"],
                          \"CpmBannerAdBuilderAdFieldNames\": [
                              \"Creative\",
                              \"Href\",
                              \"TurboPageId\"],
                          \"CpmVideoAdBuilderAdFieldNames\": [
                              \"Creative\",
                              \"Href\",
                              \"TurboPageId\"],
                          \"SmartAdBuilderAdFieldNames\": [
                              \"Creative\"],
                          \"Page\": {
                          \"Limit\": 10000,
                          \"Offset\": ",
                          lim, "}
    }
    }")

      answer <- POST("https://api.direct.yandex.com/json/v5/ads", 
                     body = queryBody, 
                     add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru",'Client-Login' = Login))
      
      stop_for_status(answer)
      
      dataRaw <- content(answer, "parsed", "application/json")

      # check erroe
      if(length(dataRaw$error) > 0){
        stop(paste0(dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
      }

      # parse list and add to result frame
      if (length(dataRaw$result$Ads) > 0){
        for(ads_i in seq_along(1:length(dataRaw$result$Ads))){
          result      <- rbind(result,
                               data.frame(Id                            = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$Id), NA,dataRaw$result$Ads[[ads_i]]$Id),
                                          AdGroupId                     = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$AdGroupId), NA,dataRaw$result$Ads[[ads_i]]$AdGroupId),
                                          CampaignId                    = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CampaignId), NA,dataRaw$result$Ads[[ads_i]]$CampaignId),
                                          Type                          = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$Type), NA,dataRaw$result$Ads[[ads_i]]$Type),
                                          Subtype                       = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$Subtype), NA,dataRaw$result$Ads[[ads_i]]$Subtype),
                                          Status                        = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$Status), NA,dataRaw$result$Ads[[ads_i]]$Status),
                                          AgeLabel                      = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$AgeLabel), NA,dataRaw$result$Ads[[ads_i]]$AgeLabel),
                                          State                         = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$State), NA,dataRaw$result$Ads[[ads_i]]$State),
                                          TextAdTitle                   = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$Title), NA,dataRaw$result$Ads[[ads_i]]$TextAd$Title),
                                          TextAdTitle2                  = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$Title2), NA,dataRaw$result$Ads[[ads_i]]$TextAd$Title2),
                                          TextAdText                    = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$Text), NA,dataRaw$result$Ads[[ads_i]]$TextAd$Text),
                                          TextAdHref                    = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$Href), NA,dataRaw$result$Ads[[ads_i]]$TextAd$Href),
                                          TextAdDisplayDomain           = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$DisplayDomain), NA,dataRaw$result$Ads[[ads_i]]$TextAd$DisplayDomain),
                                          TextAdMobile                  = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$Mobile), NA,dataRaw$result$Ads[[ads_i]]$TextAd$Mobile),
                                          TextImageAdHref               = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextImageAd$Href), NA,dataRaw$result$Ads[[ads_i]]$TextImageAd$Href),
                                          TextImageAdTurboPageId        = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextImageAd$TurboPageId), NA,dataRaw$result$Ads[[ads_i]]$TextImageAd$TurboPageId),
                                          TextMediaAdHref               = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpmBannerAdBuilderAd$Href), NA, dataRaw$result$Ads[[ads_i]]$CpmBannerAdBuilderAd$Href),
                                          TextAdSitelinkSetId           = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$TextAd$SitelinkSetId), NA, dataRaw$result$Ads[[ads_i]]$TextAd$SitelinkSetId),
                                          DynamicTextAdText             = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$DynamicTextAd$Text), NA, dataRaw$result$Ads[[ads_i]]$DynamicTextAd$Text),
                                          DynamicTextAdSitelinkSetId    = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$DynamicTextAd$SitelinkSetId), NA, dataRaw$result$Ads[[ads_i]]$DynamicTextAd$SitelinkSetId),
                                          SmartAdBuilderAdCreativeId    = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$SmartAdBuilderAd$Creative$CreativeId), NA, dataRaw$result$Ads[[ads_i]]$SmartAdBuilderAd$Creative$CreativeId),
                                          SmartAdBuilderThumbnailUrl    = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$SmartAdBuilderAd$Creative$ThumbnailUrl), NA, dataRaw$result$Ads[[ads_i]]$SmartAdBuilderAd$Creative$ThumbnailUrl),
                                          SmartAdBuilderPreviewUrl      = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$SmartAdBuilderAd$Creative$PreviewUrl), NA, dataRaw$result$Ads[[ads_i]]$SmartAdBuilderAd$Creative$PreviewUrl),
                                          CpmBannerAdBuilderHref        = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpmBannerAdBuilderAd$Href), NA, dataRaw$result$Ads[[ads_i]]$CpmBannerAdBuilderAd$Href),
                                          CpmBannerAdBuilderTurboPageId = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpmBannerAdBuilderAd$TurboPageId), NA, dataRaw$result$Ads[[ads_i]]$CpmBannerAdBuilderAd$TurboPageId),
                                          CpcVideoAdBuilderHref         = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpcVideoAdBuilderAd$Href), NA, dataRaw$result$Ads[[ads_i]]$CpcVideoAdBuilderAd$Href),
                                          CpcVideoAdBuilderTurboPageId  = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpcVideoAdBuilderAd$TurboPageId), NA, dataRaw$result$Ads[[ads_i]]$CpcVideoAdBuilderAd$TurboPageId),
                                          CpmVideoAdBuilderHref         = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpmVideoAdBuilderAd$Href), NA, dataRaw$result$Ads[[ads_i]]$CpmVideoAdBuilderAd$Href),
                                          CpmVideoAdBuilderTurboPageId  = ifelse(is.null(dataRaw$result$Ads[[ads_i]]$CpmVideoAdBuilderAd$TurboPageId), NA, dataRaw$result$Ads[[ads_i]]$CpmVideoAdBuilderAd$TurboPageId))
                                          )
        }
        # add point to progress bar
        packageStartupMessage(".", appendLF = F)
      }
      # if last iteration change lim
      lim <- ifelse(is.null(dataRaw$result$LimitedBy), "stoped",dataRaw$result$LimitedBy + 1)
    }

    # set offset to next iteration
    camp_start <- camp_start + camp_step
  }

  # set finish time
  stop_time <- Sys.time()

  # tech messages
  packageStartupMessage("Done", appendLF = T)
  packageStartupMessage(paste0("Number of load ads: ", nrow(result)), appendLF = T)
  packageStartupMessage(paste0("Duration: ", round(difftime(stop_time, start_time , units ="secs"),0), " sec."), appendLF = T)
  # return of result
  return(result)}

ReportID <- NULL
StatusReport <- NULL
ws_data <- NULL
SearchedAlso <- NULL 
SearchedWith <- NULL
. <- NULL

yadirGetWordStatReport <- function(
                             Phrases,
                             GeoID         = 0,
                             Login         = getOption("ryandexdirect.user"),
                             Token         = NULL,
                             AgencyAccount = getOption("ryandexdirect.agency_account"),
                             TokenPath     = yadirTokenPath()) {
  
  
  #start time
  start_time  <- Sys.time()
  
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  # check length
  Phrases <- yadirToList(Phrases)
  GeoID   <- yadirToList(GeoID)
  
  # отправляем отчёт
  message('.Send report')
  send_query <- list(method = "CreateNewWordstatReport",
                     param = list(Phrases = Phrases,
                                  GeoID   = GeoID),
                     locale = "ru",
                     token = Token) %>%
                toJSON(auto_unbox = TRUE)
  
  ans <- POST("https://api.direct.yandex.ru/v4/json/", body = send_query )
  
  rep_id = content(ans, 'parsed')
  
  # check for error
  if ( ! is.null( rep_id$error_str ) ) {
    
    message(rep_id$error_detail)
    stop(rep_id$error_str)
  }
  
  # проверка статуса отчёта
  rep_status <- 'New'
  
  while ( rep_status != 'Done' ) {
    
    get_status <- list(method = "GetWordstatReportList",
                  locale = "ru",
                  token = Token) %>%
             toJSON(auto_unbox = TRUE)
    
    ans <- POST("https://api.direct.yandex.ru/v4/json/", body = get_status )
    
    rep_status <- content(ans, 'parsed') %>% 
                    tibble( report = .$data ) %>% 
                    unnest_wider(report) %>%
                    filter(ReportID == rep_id) %>%
                    select(StatusReport) %>%
                    unlist()
    
    Sys.sleep(1)
    message(".Report status: ", rep_status)
  }
  
  # get query
  message('.Get report')
  get_query <- list(method = "GetWordstatReport",
                    param  = unlist(rep_id),
                    locale = "ru",
                    token = Token) %>%
               toJSON(auto_unbox = TRUE)
  
  raw_data <- POST("https://api.direct.yandex.ru/v4/json/", body = get_query )
  
  # unnesting_report
  message('.Parse report')
  SearchedAlso <- content(raw_data, 'parsed') %>% 
                    tibble(ws_data = .$data) %>%
                    unnest_wider(ws_data) %>%
                    unnest_longer(GeoID) %>%
                    select(-SearchedWith, -".") %>%
                    unnest_longer(SearchedAlso) %>%
                    hoist(SearchedAlso,
                          Phrase     = c("Phrase"),
                          Shows      = c("Shows"))
  
  SearchedWith <- content(raw_data, 'parsed') %>% 
                    tibble(ws_data = .$data) %>%
                    unnest_wider(ws_data) %>%
                    unnest_longer(GeoID) %>%
                    select(-SearchedAlso, -".") %>%
                    unnest_longer(SearchedWith) %>%
                    hoist(SearchedWith,
                          Phrase     = c("Phrase"),
                          Shows      = c("Shows"))
  
  # create object
  report <- list(SearchedWith = SearchedWith,
                 SearchedAlso = SearchedAlso)
  
  # delete report
  message('.Delete report')
  get_query <- list(method = "GetWordstatReport",
                    param  = unlist(rep_id),
                    locale = "ru",
                    token = Token) %>%
    toJSON(auto_unbox = TRUE)
  
  raw_data <- POST("https://api.direct.yandex.ru/v4/json/", body = get_query )
  
  message("Success!")
  stop_time <- Sys.time()
  message(paste0("Duration: ", round(difftime(stop_time, start_time , units ="secs"),0), " sec."))
  message("Request ID: ", ans$headers$requestid)
  message("WordStat Report ID: ", rep_id)
  message("Report SearchedAlso has ", nrow(SearchedAlso), " rows")
  message("Report SearchedWith has ", nrow(SearchedWith), " rows")
  message("For get report data x[['ReportName']]")
  return(report) 
}

# var
ForecastID <- NULL 
StatusForecast <- NULL
Phrases <- NULL
Common <- NULL

# length check function
yadirToList <- function(x) {
  
  if ( length(x) == 1 ) {
    
    x <- as.list(x)
    
  }
  
  return(x)
  
}

# forecast function
yadirGetForecast <- function(
  Phrases,
  GeoID         = 0,
  Currency      = "RUB",
  AuctionBids   = "No",
  Login         = getOption("ryandexdirect.user"),
  Token         = NULL,
  AgencyAccount = getOption("ryandexdirect.agency_account"),
  TokenPath     = yadirTokenPath()) {
  
  
  #start time
  start_time  <- Sys.time()
  
  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  # AuctionBids
  if ( is.logical(AuctionBids) ) {
    
    if ( isTRUE(AuctionBids) ) {
      
      AuctionBids <- "Yes"
    
    } else {
      
      AuctionBids <- "No"
      
    }
  }
  
  # check length
  Phrases <- yadirToList(Phrases)
  GeoID   <- yadirToList(GeoID)
  
  # отправляем отчёт
  message('.Send Query')
  send_query <- list(method = "CreateNewForecast",
                     param = list(Phrases     = Phrases,
                                  GeoID       = GeoID,
                                  Currency    = Currency,
                                  AuctionBids = AuctionBids),
                     locale = "ru",
                     token = Token) %>%
    toJSON(auto_unbox = T, pretty = T)
  
  #return(send_query)
  ans <- POST("https://api.direct.yandex.ru/live/v4/json/", body = send_query )
  
  rep_id <- content(ans, 'parsed')

  # check for error
  if ( ! is.null( rep_id$error_str ) ) {
    
    message(rep_id$error_detail)
    stop(rep_id$error_str)
  }
  
  
  # проверка статуса отчёта
  rep_status <- 'New'

  while ( rep_status != 'Done' ) {
    
    get_status <- list(method = "GetForecastList",
                       locale = "ru",
                       token = Token) %>%
      toJSON(auto_unbox = TRUE)
    
    ans <- POST("https://api.direct.yandex.ru/live/v4/json/", body = get_status )
    
    rep_status <- content(ans, 'parsed') %>% 
      tibble( report = .$data ) %>% 
      unnest_wider(report) %>% 
      filter(ForecastID == rep_id) %>%
      select(StatusForecast) %>%
      unlist()
    
    Sys.sleep(15)
    message(".Forecast status: ", rep_status)
  }
  
  # get query
  message('.Get Forecast')
  get_query <- list(method = "GetForecast",
                    param  = unlist(rep_id),
                    locale = "ru",
                    token = Token) %>%
    toJSON(auto_unbox = TRUE)
  
  raw_data <- POST("https://api.direct.yandex.ru/live/v4/json/", body = get_query )
  
  # unnesting_report
  message('.Parse Forecast')

  PhrasesForecast <- content(raw_data, 'parsed') %>% 
                        tibble(ws_data = list(.$data)) %>% 
                        select(ws_data) %>%
                        unnest_wider(ws_data) %>%
                        select(-Common) %>%
                        unnest_longer(Phrases) %>% 
                        unnest_wider(Phrases)
  
  # if report with AuctionBids unnesting
  if ( tolower(AuctionBids) == 'yes' ) {
    
    PhrasesForecast <-
      PhrasesForecast %>%
        unnest_longer(AuctionBids) %>%
        unnest_wider(AuctionBids)
    
  }
  
  CommonForecast <- content(raw_data, 'parsed') %>% 
                        tibble(ws_data = list(.$data)) %>% 
                        select(ws_data) %>%
                        unnest_wider(ws_data) %>%
                        select(-Phrases) %>%
                        unnest_wider(Common) 
  
  # create object
  report <- list(PhrasesForecast = PhrasesForecast,
                 CommonForecast  = CommonForecast)
  
  # delete report
  message('.Delete Forecast')
  get_query <- list(method = "DeleteForecastReport",
                    param  = unlist(rep_id),
                    locale = "ru",
                    token = Token) %>%
    toJSON(auto_unbox = TRUE)
  
  raw_data <- POST("https://api.direct.yandex.ru/live/v4/json/", body = get_query )
  
  message("Success!")
  stop_time <- Sys.time()
  message(paste0("Duration: ", round(difftime(stop_time, start_time , units ="secs"),0), " sec."))
  message("Request ID: ", ans$headers$requestid)
  message("Forecast ID: ", rep_id)
  message("PhrasesForecast has ", nrow(PhrasesForecast), " rows")
  message("CommonForecast has ", nrow(CommonForecast), " rows")
  message("For get report data x[['ReportName']]")
  return(report) 
}

yadirGetDictionary <- function(DictionaryName = "GeoRegions", 
                               Language       = "ru", 
                               Login          = getOption("ryandexdirect.user"),
                               Token          = NULL,
                               AgencyAccount  = getOption("ryandexdirect.agency_account"),
                               TokenPath      = yadirTokenPath()){
  
  # start

  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  # Check dict name
  if(!DictionaryName %in% c("Currencies",
                            "MetroStations",
                            "GeoRegions",
                            "TimeZones",
                            "Constants",
                            "AdCategories",
                            "OperationSystemVersions",
                            "ProductivityAssertions",
                            "SupplySidePlatforms",
                            "Interests")){
    stop("Error in DictionaryName, select one of Currencies, MetroStations, GeoRegions, TimeZones, Constants, AdCategories, OperationSystemVersions, ProductivityAssertions, SupplySidePlatforms, Interests")
  }

	queryBody <- paste0("{
						  \"method\": \"get\",
						  \"params\": {
						  \"DictionaryNames\": [ \"",DictionaryName,"\" ]
	}
	}")
  
  # send query
  answer <- POST("https://api.direct.yandex.com/json/v5/dictionaries", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = Language, "Client-Login" = Login[1]))
  # check status
  stop_for_status(answer)
  # parse
  dataRaw <- content(answer, "parsed", "application/json")
  
  if(length(dataRaw$error) > 0){
    stop(paste0(dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
  }
  
  # convert to data.frame
  
  # georegions
  if(DictionaryName == "GeoRegions"){
  dictionary_df <- data.frame()

  for(dr in 1:length(dataRaw$result[[1]])){
    dictionary_df_temp <- data.frame(GeoRegionId = dataRaw$result[[1]][[dr]]$GeoRegionId,
                                     ParentId = ifelse(is.null(dataRaw$result[[1]][[dr]]$ParentId),NA , dataRaw$result[[1]][[dr]]$ParentId),
                                     GeoRegionType = dataRaw$result[[1]][[dr]]$GeoRegionType,
                                     GeoRegionName = dataRaw$result[[1]][[dr]]$GeoRegionName)
    dictionary_df <- rbind(dictionary_df, dictionary_df_temp)
    
  }}

  # cur
  if(DictionaryName == "Currencies"){
    dictionary_df <- data.frame()
    for(dr in dataRaw$result$Currencies){
      dictionary_df_temp <- data.frame(Currency                = dr$Currency,
                                       BidIncrement            = as.numeric(dr$Properties[[1]]$Value) / 1000000,
                                       FullName                = dr$Properties[[2]]$Value,
                                       MaximumBid              = as.numeric(dr$Properties[[3]]$Value) / 1000000,
                                       MinimumAverageCPA       = as.numeric(dr$Properties[[4]]$Value) / 1000000,
                                       MinimumAverageCPC       = as.numeric(dr$Properties[[5]]$Value) / 1000000,
                                       MinimumCPM              = as.numeric(dr$Properties[[6]]$Value) / 1000000,
                                       MaximumCPM              = as.numeric(dr$Properties[[7]]$Value) / 1000000,
                                       MinimumBid              = as.numeric(dr$Properties[[8]]$Value) / 1000000,
                                       MinimumDailyBudget      = as.numeric(dr$Properties[[9]]$Value) / 1000000,
                                       MinimumPayment          = as.numeric(dr$Properties[[10]]$Value) / 1000000,
                                       MinimumTransferAmount   = as.numeric(dr$Properties[[11]]$Value) / 1000000,
                                       MinimumWeeklySpendLimit = as.numeric(dr$Properties[[12]]$Value) / 1000000)
      
      dictionary_df <- rbind(dictionary_df, dictionary_df_temp)
    }
    
  }
  
  # Interests
  if(DictionaryName == "Interests"){
    dictionary_df <- data.frame()
    for(dr in 1:length(dataRaw$result[[1]])){
      dictionary_df_temp <- data.frame(Name = dataRaw$result[[1]][[dr]]$Name,
                                       ParentId = ifelse(is.null(dataRaw$result[[1]][[dr]]$ParentId),NA , dataRaw$result[[1]][[dr]]$ParentId),
                                       InterestId = dataRaw$result[[1]][[dr]]$InterestId,
                                       IsTargetable = dataRaw$result[[1]][[dr]]$IsTargetable)
      dictionary_df <- rbind(dictionary_df, dictionary_df_temp)
    }
  }
  
  # pars dict with standart structure
  if(! DictionaryName %in% c("Currencies","GeoRegions","Interests")){
    dictionary_df <- do.call(rbind.data.frame, dataRaw$result[[1]])
    }

  # technical info message
   packageStartupMessage("Directory successfully loaded!", appendLF = T)
   packageStartupMessage(paste0("Points are deducted from: " ,answer$headers$`units-used-login`), appendLF = T)
   packageStartupMessage(paste0("Number of API points spent when executing the request: " ,strsplit(answer$headers$units, "/")[[1]][1]), appendLF = T)
   packageStartupMessage(paste0("Available balance of daily limit API points: " ,strsplit(answer$headers$units, "/")[[1]][2]), appendLF = T)
   packageStartupMessage(paste0("Daily limit of API points:" ,strsplit(answer$headers$units, "/")[[1]][3]), appendLF = T)
   packageStartupMessage(paste0("Reqiest ID that must be specified when contacting support:: ",answer$headers$requestid), appendLF = T)
  
  # result
  return(dictionary_df)
}

# FetchBy
date_ranges <- function(start, end, by) {
  start <- as.Date(start)
  end <- as.Date(end)
  by <- tolower(by)
  by <- match.arg(by, c("day", "week", "month", "quarter", "year"))
  dates <- seq.Date(start, end, by = by)
  res <- cbind(start = as.character(dates),
               end = as.character(c(dates[-1] - 1, end)))
  as.data.frame(res, stringsAsFactors = F)
}

yadirGetReport <- function(ReportType        = "CUSTOM_REPORT", 
                           DateRangeType     = "CUSTOM_DATE", 
                           DateFrom          = Sys.Date() - 31, 
                           DateTo            = Sys.Date() - 1, 
                           FieldNames        = c("CampaignName","Impressions","Clicks","Cost"), 
                           FilterList        = NULL,
                           Goals             = NULL,
                           AttributionModels = NULL,
                           IncludeVAT        = "YES",
                           IncludeDiscount   = "NO",
                           Login             = getOption("ryandexdirect.user"),
                           AgencyAccount     = getOption("ryandexdirect.agency_account"),
                           FetchBy           = NULL,
                           Token             = NULL,
                           TokenPath         = yadirTokenPath(),
						               SkipErrors        = TRUE) {
  st <- Sys.time()
  
  # for limited accounts
  limit_reached <- c()
  
  # FetchBy
  if (!is.null(FetchBy)) {
    if ((DateRangeType != "CUSTOM_DATE") && (is.null(DateFrom)) && (is.null(DateTo)))
        stop('You should use "FetchBy" parameter only with DateRangeType "CUSTOM_DATE" and existing DateFrom and DateTo.')
    #dates df
    dates <- date_ranges(DateFrom, DateTo, FetchBy)
    n <- nrow(dates)
    packageStartupMessage("Batch processing mode is enabled.")
    packageStartupMessage(paste0("Fetching data by ", tolower(FetchBy), ": from ", as.Date(DateFrom), " to ", as.Date(DateTo), "."))
    packageStartupMessage()
  } else {
    #dates df
    dates <- as.data.frame(list(DateFrom, DateTo), col.names = c("start","end"), stringsAsFactors = F)
    n <- 1
  }

  # result
  result <- data.frame()
  for (i in 1:n) {
    df <- yadirGetReportFun(ReportType, 
                            DateRangeType, 
                            dates[i, "start"], 
                            dates[i, "end"], 
                            FieldNames, 
                            FilterList,
                            Goals,
                            AttributionModels,
                            IncludeVAT,
                            IncludeDiscount,
                            Login,
                            AgencyAccount,
                            Token,
                            TokenPath,
						    SkipErrors
    )
	
	limit_reached <- attr(df, "limit_reached")
	
    result <- rbind(result, df)
  }
  
  # logs
  if (!is.null(FetchBy) && length(limit_reached) == 0) {
    packageStartupMessage('Function with batch processing mode has executed successfully.')
    packageStartupMessage("Total number of rows is: ", nrow(result))
    packageStartupMessage("Total executing time is: ", round(difftime(Sys.time(), st , units = "secs"), 0), " sec.")
  }
  
    # check limits
  if ( length(limit_reached) > 0 ) {
   attr(result, "limit_reached") <- unique(limit_reached)
  }  
  return(result)
}

yadirGetLogins <- function(TokenPath = yadirTokenPath(), SetLogin = TRUE) {
  
  if ( is.null(TokenPath) ) {
    TokenPath <- getwd()
  }
  
  if ( ! interactive() ) {
    
    SetLogin <- FALSE
    
  }
  
  files  <- dir(TokenPath, pattern = "\\.yadirAuth\\.RData$")
  logins <- gsub("\\.yadirAuth\\.RData$", replacement = "", files)
  
  if ( length(files) == 0 ) {
    
    message("You does't have any authorize login.")
    auth <- readline("Do you want start authorize process? ( y / n ): ")
    
    if (auth == "y") {
      
      if ( is.null(getOption("ryandexdirect.user")) ) {
        
        login <- readline("Enter your Yandex login:")
        
      } else {
        
        login <- getOption("ryandexdirect.user")
        
      }
      
      yadirAuth(Login = login, NewUser = T, TokenPath = TokenPath)
      
      return(login)
      
    } else {
      
      return(NULL)
      
    }
  }
  
  for ( i in 1:length(logins) ) {
    
    cat(i, ":\t", logins[i], "\n")
    
  }
  
  if (SetLogin) {
    
    len          <- length(logins)
    login_number <- readline(str_glue("Choose login (enter number from 1 to {len}): "))
    login        <- logins[as.integer(login_number)]
    options(ryandexdirect.user = login)
    message(str_glue("Set {login} as default session login"))
    return(login)
    
  }
  
  return(logins)
  
}

#' @title Print yadir_token
#' @description S3 method for printing yadir_token objects
#' @param x yadir_token object
#' @param ... extra arguments
#' @export
print.yadir_token <- function(x, ...) {
  
  cat('Yandex Direct Token:\nExpire at', format(x$expire_at, '%Y-%m-%d %T %Z (UTC %z)'))
  
}

yadirGetClientParam <- function(Language      = "ru", 
                                Logins        = getOption("ryandexdirect.user"), 
                                Token         = NULL,
                                AgencyAccount = getOption("ryandexdirect.agency_account"),
                                TokenPath     = yadirTokenPath()){
  
  
  # auth
  Token <- tech_auth(login = Logins, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  # loading client list
  if ( ! is.null(AgencyAccount) && is.null(Logins)) {
    Logins <- yadirGetClientList(AgencyAccount = AgencyAccount, TokenPath = TokenPath, Token = Token)$Login
  }
  
  # compose query
    queryBody <- paste0("{
  \"method\": \"get\",
                      \"params\": { 
                      \"FieldNames\": [
                      \"AccountQuality\",
                      \"Archived\",
                      \"ClientId\",
                      \"ClientInfo\",
                      \"CountryId\",
                      \"CreatedAt\",
                      \"Currency\",
                      \"Grants\",
                      \"Login\",
                      \"Notification\",
                      \"OverdraftSumAvailable\",
                      \"Phone\",
                      \"Representatives\",
                      \"Restrictions\",
                      \"Settings\",
                      \"VatRate\"]
      }
    }")
  
  result <- data.frame()
  
  for (login in Logins) {
    
    # send query
    answer <- POST("https://api.direct.yandex.com/json/v5/clients", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = Language, "Client-Login" = login))
    # check for error
    stop_for_status(answer)
    
    dataRaw <- content(answer, "parsed", "application/json")
    
    if(length(dataRaw$error) > 0){
      warning(login,paste0(": ",dataRaw$error$error_string, " - ", dataRaw$error$error_detail))
	  next
    }
       
    # result frame
      dictionary_df <- data.frame()
      
      for(dr in 1:length(dataRaw$result[[1]])){
        dictionary_df_temp <- data.frame(Login                       = dataRaw$result[[1]][[dr]]$Login,
                                         ClientId                    = dataRaw$result[[1]][[dr]]$ClientId,
                                         CountryId                   = dataRaw$result[[1]][[dr]]$CountryId,
                                         Currency                    = dataRaw$result[[1]][[dr]]$Currency,
                                         CreatedAt                   = dataRaw$result[[1]][[dr]]$CreatedAt,
                                         ClientInfo                  = dataRaw$result[[1]][[dr]]$ClientInfo,
                                         AccountQuality              = ifelse(is.null(dataRaw$result[[1]][[dr]]$AccountQuality),NA,dataRaw$result[[1]][[dr]]$AccountQuality),
                                         CampaignsTotalPerClient     = dataRaw$result[[1]][[dr]]$Restrictions[[1]]$Value,
                                         CampaignsUnarchivePerClient = dataRaw$result[[1]][[dr]]$Restrictions[[2]]$Value,
                                         APIPoints                   = dataRaw$result[[1]][[dr]]$Restrictions[[10]]$Value)
        
        dictionary_df <- rbind(dictionary_df, dictionary_df_temp)
        
      }
      


    # tech messages
     packageStartupMessage(paste0("API points write off from : " ,answer$headers$`units-used-login`), appendLF = T)
     packageStartupMessage(paste0("Number of points spent on the request: " ,strsplit(answer$headers$units, "/")[[1]][1]), appendLF = T)
     packageStartupMessage(paste0("Available balance daily about limit points: " ,strsplit(answer$headers$units, "/")[[1]][2]), appendLF = T)
     packageStartupMessage(paste0("Daily limit points: " ,strsplit(answer$headers$units, "/")[[1]][3]), appendLF = T)
     packageStartupMessage(paste0("Unique identifier of the request that must be specified when contacting support: ",answer$headers$requestid), appendLF = T)

     # binding resuilt
     result <- rbind(result, dictionary_df)
  }
  return(result)
}

yadirSetLogin <- function(Login, TokenPath = yadirTokenPath()) {
  
  file_name   <- file.path(TokenPath, str_glue("{Login}.yadirAuth.RData"))
     
  if ( file.exists(file_name) ) {

    options(ryandexdirect.user = Login)
    
  } else if ( ! is.null(getOption('ryandexdirect.agency_account')) ) {
    
    clients <- yadirGetClientList()
    client_logins <- clients$Login
    
    # search in agency account
    if ( tolower(Login) %in% tolower(client_logins) ) {
      
      options(ryandexdirect.user = Login)
      message("AgencyAccount: ", getOption('ryandexdirect.agency_account'), "\n",
              "Login: ", Login)
      
    }
  

  } else if ( interactive() ) {
    
    yadirAuth(Login, NewUser = TRUE, TokenPath)
    
    options(ryandexdirect.user = Login)
  
  } else {
    
    stop(str_glue("Your login not found in {TokenPath}, and you using batch mode. Run R in interactive mode and repeat this action."))
    
  }

  return(TRUE)
  
}

# for check
Date <- NULL

yadirGetCostData <- function(DateFrom        = Sys.Date() - 31,
                             DateTo          = Sys.Date() - 1,
                             Source          = 'yandex',
                             Medium          = 'cpc',
                             IncludeVAT      = "YES",
                             IncludeDiscount = "NO",
                             Login           = getOption("ryandexdirect.user"),
                             AgencyAccount   = getOption("ryandexdirect.agency_account"),
                             FetchBy         = NULL,
                             Token           = NULL,
                             TokenPath       = yadirTokenPath()) {
  
  
  # run report loader
  cost_data <- yadirGetReport(ReportType = "CUSTOM_REPORT",
                              DateRangeType = "CUSTOM_DATE",
                              DateFrom = DateFrom,
                              DateTo = DateTo,
                              FieldNames = c("Date",
                                             "CampaignName", 
                                             "Criteria",
                                             "AdId",
                                             "Impressions",
                                             "Clicks", 
                                             "Cost"),
                              IncludeVAT = IncludeVAT,
                              IncludeDiscount = IncludeDiscount,
                              FetchBy = FetchBy,
                              Login = Login,
                              AgencyAccount = AgencyAccount,
                              Token = Token,
                              TokenPath = TokenPath,
                              SkipErrors = FALSE) %>%
    mutate(Date = format(as.Date(Date), "%Y%m%d")) %>%
    rename("ga:date"        = 'Date',
           "ga:campaign"    = 'CampaignName',
           "ga:keyword"     = 'Criteria',
           "ga:adContent"   = 'AdId',
           "ga:adCost"      = 'Cost',
           "ga:impressions" = 'Impressions',
           "ga:adClicks"    = 'Clicks') %>%
    mutate("ga:medium" = Medium,
           "ga:source" = Source) %>%
    select("ga:date",
           "ga:medium",
           "ga:source",
           "ga:adClicks",
           "ga:adCost",
           "ga:impressions",
           "ga:campaign",
           "ga:keyword",
           "ga:adContent") %>%
    mutate_if( is.character, iconv, to = "UTF-8" )%>%
    mutate_if( is.numeric, replace_na, 0)
  
  return(cost_data)
  
  
}

yadirSetAgencyAccount <- function(AgencyAccount, TokenPath = yadirTokenPath()) {
  
  if ( is.null(AgencyAccount) || is.na(AgencyAccount) ) {
    
    message("Unset AgencyAccount")
    options(ryandexdirect.agency_account = NULL)
    
  } else  {
  
    file_name   <- file.path(TokenPath, str_glue("{AgencyAccount}.yadirAuth.RData")) 
  
    if ( file.exists(file_name) ) {
      
      options(ryandexdirect.agency_account = AgencyAccount)
      
    } else if ( interactive() ) {
      
      yadirAuth(Login = AgencyAccount, NewUser = TRUE, TokenPath)
      
      options(ryandexdirect.user = AgencyAccount)
      
    } else {
      
      stop(str_glue("Your login not found in {TokenPath}, and you using batch mode. Run R in interactive mode and repeat this action."))
      
    }
  
  }
  
  return(TRUE)

}

yadirStartAds <-  function(Login = getOption("ryandexdirect.user"), 
                           Ids   = NULL,
                           Token = NULL,
                           AgencyAccount = getOption("ryandexdirect.agency_account"),
                           TokenPath     = yadirTokenPath()){
  
  # auth
  Token <- tech_auth(login = Login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
  
  if(length(Ids) > 10000){
    stop(paste0("In the parameter Ids transferred numbers ",length(Ids), " of ads, the maximum number of ads per request is 10000."))
  }
  
  if(is.null(Ids)){
    stop("In the Ids argument, you must pass the vector containing the Id ads for which you want to resume the show. You have not passed any Id.")
  }
  
  # errors counter
  CounErr <- 0
  
  # Error vector
  errors_id <-  vector()
  
  # start time
  start_time  <- Sys.time()
  
  # start message
  packageStartupMessage("Processing", appendLF = T)
  
  IdsPast <- paste0(Ids, collapse = ",")
  
  # request body
  queryBody <- paste0("{
                      \"method\": \"resume\",
                      \"params\": { 
                      \"SelectionCriteria\": {
                      \"Ids\": [",IdsPast,"]}
}
}")
  
  # send request
  answer <- POST("https://api.direct.yandex.com/json/v5/ads", body = queryBody, add_headers(Authorization = paste0("Bearer ",Token), 'Accept-Language' = "ru","Client-Login" = Login))
  # parse answer
  ans_pars <- content(answer)
  # check answer for errors
  if(!is.null(ans_pars$error)){
    stop(paste0("Error: ", ans_pars$error$error_string,". Message: ",ans_pars$error$error_detail, ". RequestID: ",ans_pars$error$request_id))
  }
  
  # check not starting ads
  for(error_search in 1:length(ans_pars$result$ResumeResults)){
    if(!is.null(ans_pars$result$ResumeResults[[error_search]]$Errors)){
      CounErr <- CounErr + 1
      errors_id <- c(errors_id, Ids[error_search])
      packageStartupMessage(paste0("    AdId: ",Ids[error_search]," - ", ans_pars$result$ResumeResults[[error_search]]$Errors[[1]]$Details))
    }
  }
  
  # message about starting ads number
  out_message <- ""
  
  TotalCampStoped <- length(Ids) - CounErr
  
  if(TotalCampStoped %in% c(2,3,4) & !(TotalCampStoped %% 100 %in% c(12,13,14))){
    out_message <- "start ads"
  } else if(TotalCampStoped %% 10 == 1 & TotalCampStoped %% 100 != 11){
    out_message <- "start ads"
  } else {
    out_message <- "start ads"
  }
  
  # message
  packageStartupMessage(paste0(TotalCampStoped, " ", out_message))
  packageStartupMessage(paste0("Total time: ", as.integer(round(difftime(Sys.time(), start_time , units ="secs"),0)), " sec."))
  return(errors_id)}

yadirSetKeyWordsBids <- function(KeywordIds       = NULL,
                                 AdGroupIds       = NULL,
                                 CampaignIds      = NULL,
                                 StrategyPriority = c(NA, "LOW", "NORMAL", "HIGH" ),
                                 SearchBid        = NULL,
                                 NetworkBid       = NULL,
                                 Login            = getOption("ryandexdirect.user"),
                                 Token            = NULL,
                                 AgencyAccount    = getOption("ryandexdirect.agency_account"),
                                 TokenPath        = yadirTokenPath()) {
  # start time
  start_time  <- Sys.time()
  
  # check str priority
  StrategyPriority <- match.arg(StrategyPriority)
  
  # auth
  Token <- tech_auth(login         = Login, 
                     token         = Token, 
                     AgencyAccount = AgencyAccount, 
                     TokenPath     = TokenPath)
  
  # requests id log
  req_ids <- c()
  
  # out
  out <- list()
  
  # create body shell
  query_list <- list(method = "set",
                     params = 
                       list(KeywordBids = list())
  )
  
  
  # define variables
  if (! is.null(KeywordIds) ) {
    
    if ( "data.frame" %in% class( KeywordIds ) ) {
      KeywordIds <- KeywordIds$Id
    }
    object          <- "keywords"
    ids_parts_limit <- 10000
    ids             <- KeywordIds
    
    
  } else if (! is.null(AdGroupIds) ) {
    
    if ( "data.frame" %in% class( AdGroupIds ) ) {
      AdGroupIds <- AdGroupIds$Id
    }
    
    object          <- "adgroups"
    ids_parts_limit <- 1000
    ids             <- AdGroupIds
    
  } else {
    
    object          <- "campaings"
    ids_parts_limit <- 10
    
    
    if ( "data.frame" %in% class( CampaignIds ) ) {
      CampaignIds <- CampaignIds$Id
    }
    
    # if CampaignIds not set
    if ( is.null(CampaignIds) ) {
      message("You dont choised any ids of campaign, adgroup or ad. Loading full campaign list.")
      if ( interactive() ) {
        load_camp <- readline("Do you want load all campaings and set keywordbids to all account keywords? (y / n)")
        if ( tolower(load_camp) %in% c("y", "yes")) {
          ids <-  yadirGetCampaignList(Logins        = Login,
                                       AgencyAccount = AgencyAccount,
                                       Token         = Token,
                                       TokenPath     = TokenPath)$Id
        } else {
          
          stop("You don`t set any objects ids, please set KeywordIds or AdGroupIds or CampaignIds")
          
        }
        
      } else {
        
        stop("You don`t set any objects ids, please set KeywordIds or AdGroupIds or CampaignIds")
        
      }
      
    } else {
      ids             <- CampaignIds
    }
    
  }
  
  # object for collect answers
  res <- list()
  
  # spliting for obj limit
  id_parts <- split(ids, 
                    ceiling(seq_along(ids) / ids_parts_limit))
  
  # start of coleccting
  for (part_of_id in id_parts) {
    
    # detect ids and object
    if ( object == "keywords" ) {
      query_list_param <- lapply( part_of_id, function(x) list(KeywordId        = x,
                                                               StrategyPriority = StrategyPriority,
                                                               SearchBid        = ifelse( is.null(SearchBid), NA, SearchBid * 1000000 ),
                                                               NetworkBid       = ifelse( is.null(NetworkBid), NA, NetworkBid * 1000000 )))
    } else if ( object == "adgroups" ) {
      query_list_param <- lapply( part_of_id, function(x) list(AdGroupId        = x,
                                                               StrategyPriority = StrategyPriority,
                                                               SearchBid        = ifelse( is.null(SearchBid), NA, SearchBid * 1000000 ),
                                                               NetworkBid       = ifelse( is.null(NetworkBid), NA, NetworkBid * 1000000 )))
    } else if ( object == "campaings" ) {
      query_list_param <- lapply( part_of_id, function(x) list(CampaignId      = x,
                                                               StrategyPriority = StrategyPriority,
                                                               SearchBid        = ifelse( is.null(SearchBid), NA, SearchBid * 1000000 ),
                                                               NetworkBid       = ifelse( is.null(NetworkBid), NA, NetworkBid * 1000000 )))
    }
    
    # param to body
    query_list$params$KeywordBids <- query_list_param
    
    # create body
    queryBody <- toJSON(query_list, 
                        auto_unbox = T, 
                        pretty = T, 
                        null = "null")  
    
    # send request
    answer <- POST("https://api.direct.yandex.com/json/v5/keywordbids", 
                   body = queryBody, 
                   add_headers(Authorization = paste0("Bearer ", Token), 
                               `Accept-Language` = "ru", 
                               `Client-Login` = Login))
    
    # add request id
    req_ids <- append(req_ids, headers(answer)$requestid)
    
    # answer
    dataRaw <- content(answer, "parsed", "application/json")
    
    # check answer for error
    if ( ! is.null(dataRaw$error) ) {
      message("Error:  ", dataRaw$error$error_string)
      message("Code:   ", dataRaw$error$error_code)
      message("Detail: ", dataRaw$error$error_detail)
      stop(dataRaw$error$error_detail)
    }
    
    out     <- append(out, list(list(request_id = headers(answer)$requestid,
                                     data       = list(dataRaw)))) 
    
    # set names
    names(out)  <- c(names(out), headers(answer)$requestid)
  }
  
  # copy for 
  out_ew <- out
  
  # get out data
  out <-
    lapply(out,
           function(x) {
             lapply(x$data,
                    function(y) 
                      sapply(y$result$SetResults,
                             function(z) 
                               z$KeywordId)
             )
     })
  
  # check errors
  Errors <- lapply(out_ew,
                      function(x) {
                        sapply( x$data,
                                function(y) {
                                  sapply(y$result$SetResults,
                                         function(z) {
                                           sapply(z$Errors,
                                                  function(errors) {
                                                    message("!..Error: ", unlist(errors$Details))
                                                    errors$Details
                                                  })})
                                } )
                      }
  )

  # check warnings
  Warnings <- lapply(out_ew,
                     function(x) {
                       sapply( x$data,
                               function(y) {
                                 sapply(y$result$SetResults,
                                        function(z) {
                                          sapply(z$Warnings,
                                                 function(warnings) {
                                                   message("...Warning: ", unlist(warnings$Details))
                                                   warnings$Details
                                                 })})
                               } )
                     }
  )
  
  end_time <- Sys.time()
  
  message("Duration: ", round(difftime(end_time, start_time, units = "secs"), 0), " secs")
  message("Total requests send: ", length(req_ids))
  message("RequestIDs: ", paste(req_ids, collapse = ", "))
  
  message("Ditails ----------------->")
  for ( om in req_ids ) {
    message("RequestID: ", om)
    message("ResultIds: ", paste(unlist(out[[om]]), collapse = ", "))
  }
  
  return(out)
}

yadirGetCampaign <- function(
      Logins = getOption("ryandexdirect.user"), 
      States = c("OFF", "ON", "SUSPENDED", "ENDED", "CONVERTED", "ARCHIVED"),
      Types = c("TEXT_CAMPAIGN", "MOBILE_APP_CAMPAIGN", "DYNAMIC_TEXT_CAMPAIGN", "CPM_BANNER_CAMPAIGN", "SMART_CAMPAIGN", "UNIFIED_CAMPAIGN"),
      Statuses = c("ACCEPTED", "DRAFT", "MODERATION", "REJECTED"),
      StatusesPayment = c("DISALLOWED", "ALLOWED"),
      Token = NULL,
      AgencyAccount = getOption("ryandexdirect.agency_account"),
      TokenPath = yadirTokenPath()
  ) {
    start_time <- Sys.time()
    
    # Вспомогательная функция для обработки null значений
    handle_null <- function(value, default = NA) {
      if (is.null(value)) return(default)
      return(value)
    }
    
    create_query_body <- function(States, Types, Statuses, StatusesPayment, lim) {
      list(
        method = "get",
        params = list(
          SelectionCriteria = list(
            States = States,
            Types = Types,
            StatusesPayment = StatusesPayment,
            Statuses = Statuses
          ),
          FieldNames = c(
            "Id", "Name", "Type", "StartDate", "Status", "StatusPayment", "SourceId", "State",
            "Statistics", "Funds", "Currency", "DailyBudget", "ClientInfo"
          ),
          TextCampaignFieldNames = c("BiddingStrategy", "AttributionModel"),
          MobileAppCampaignFieldNames = list("BiddingStrategy"),
          DynamicTextCampaignFieldNames = c("BiddingStrategy", "AttributionModel"),
          CpmBannerCampaignFieldNames = list("BiddingStrategy"),
          Page = list(Limit = 10000, Offset = lim)
        )
      )
    }
    
    process_campaign_data <- function(campaign, login) {
      tibble(
        Id = handle_null(campaign$Id),
        Name = handle_null(campaign$Name),
        Type = handle_null(campaign$Type),
        Status = handle_null(campaign$Status),
        State = handle_null(campaign$State),
        StatusPayment = handle_null(campaign$StatusPayment),
        SourceId = handle_null(campaign$SourceId),
        DailyBudgetAmount = handle_null(campaign$DailyBudget$Amount) / 1000000,
        DailyBudgetMode = handle_null(campaign$DailyBudget$Mode),
        Currency = handle_null(campaign$Currency),
        StartDate = handle_null(campaign$StartDate),
        Impressions = handle_null(campaign$Statistics$Impressions),
        Clicks = handle_null(campaign$Statistics$Clicks),
        ClientInfo = handle_null(campaign$ClientInfo),
        FundsMode = handle_null(campaign$Funds$Mode),
        CampaignFundsBalance = handle_null(campaign$Funds$CampaignFunds$Balance) / 1000000,
        CampaignFundsBalanceBonus = handle_null(campaign$Funds$CampaignFunds$BalanceBonus) / 1000000,
        CampaignFundsSumAvailableForTransfer = handle_null(campaign$Funds$CampaignFunds$SumAvailableForTransfer) / 1000000,
        SharedAccountFundsRefund = handle_null(campaign$Funds$SharedAccountFunds$Refund) / 1000000,
        SharedAccountFundsSpend = handle_null(campaign$Funds$SharedAccountFunds$Spend) / 1000000,
        TextCampBidStrategySearchType = handle_null(campaign$TextCampaign$BiddingStrategy$Search$BiddingStrategyType, ""),
        TextCampBidStrategyNetworkType = handle_null(campaign$TextCampaign$BiddingStrategy$Network$BiddingStrategyType, ""),
        TextCampAttributionModel = handle_null(campaign$TextCampaign$AttributionModel, ""),
        DynCampBidStrategySearchType = handle_null(campaign$DynamicTextCampaign$BiddingStrategy$Search$BiddingStrategyType, ""),
        DynCampBidStrategyNetworkType = handle_null(campaign$DynamicTextCampaign$BiddingStrategy$Network$BiddingStrategyType, ""),
        DynCampAttributionModel = handle_null(campaign$DynamicTextCampaign$AttributionModel, ""),
        MobCampBidStrategySearchType = handle_null(campaign$MobileAppCampaign$BiddingStrategy$Search$BiddingStrategyType, ""),
        MobCampBidStrategyNetworkType = handle_null(campaign$MobileAppCampaign$BiddingStrategy$Network$BiddingStrategyType, ""),
        CpmBannerBidStrategySearchType = handle_null(campaign$CpmBannerCampaign$BiddingStrategy$Search$BiddingStrategyType, ""),
        CpmBannerBidStrategyNetworkType = handle_null(campaign$CpmBannerCampaign$BiddingStrategy$Network$BiddingStrategyType, ""),
        Login = login
      )
    }
    
    lim <- 0
    packageStartupMessage("Processing", appendLF = FALSE)
    
    result <- tibble()
    
    while (lim != "stoped") {
      query_body <- create_query_body(States, Types, Statuses, StatusesPayment, lim)
      print(query_body)
      
      for (login in Logins) {
        Token <- tech_auth(login = login, token = Token, AgencyAccount = AgencyAccount, TokenPath = TokenPath)
        
        response <- request("https://api.direct.yandex.com/json/v5/campaigns") %>%
          req_method("POST") %>%
          req_headers(
            Authorization = paste("Bearer", Token),
            `Accept-Language` = "ru",
            `Client-Login` = login
          ) %>%
          req_body_json(query_body) %>%
          req_perform()
        
        data_raw <- resp_body_json(response)
        
        if (!is.null(data_raw$error)) {
          stop(paste0(data_raw$error$error_string, " - ", data_raw$error$error_detail))
        }
        
        if (!is.null(data_raw$result$Campaigns)) {
          df <- map_dfr(data_raw$result$Campaigns, process_campaign_data, login = login)
          result <- bind_rows(result, df)
        }
      }
      
      packageStartupMessage(".", appendLF = FALSE)
      lim <- ifelse(is.null(data_raw$result$LimitedBy), "stoped", data_raw$result$LimitedBy + 1)
    }
    
    result <- result %>%
      mutate(
        Type = as.factor(Type),
        Status = as.factor(Status),
        State = as.factor(State),
        Currency = as.factor(Currency)
      )
    
    stop_time <- Sys.time()
    
    packageStartupMessage("Done", appendLF = TRUE)
    packageStartupMessage(paste0("Number of loaded campaigns: ", nrow(result)), appendLF = TRUE)
    packageStartupMessage(paste0("Processing duration: ", round(difftime(stop_time, start_time, units = "secs"), 0), " sec."), appendLF = TRUE)
    
    return(result)
}

yadirGetKeyWordsBids <- function(KeywordIds    = NULL,
                                 AdGroupIds    = NULL,
                                 CampaignIds   = NULL,
                                 AuctionBids   = c(NA, "search", "network"),
                                 Login         = getOption("ryandexdirect.user"),
                                 Token         = NULL,
                                 AgencyAccount = getOption("ryandexdirect.agency_account"),
                                 TokenPath     = yadirTokenPath()) {
  # start time
  start_time  <- Sys.time()
  
  # auth
  Token <- tech_auth(login = Login, 
                     token = Token, 
                     AgencyAccount = AgencyAccount, 
                     TokenPath = TokenPath)
  
  # auction
  AuctionBids <- match.arg(AuctionBids)
  
  # requests id log
  req_ids <- c()
  
  # create body
  query_list <- list(method = "get",
                     params = 
                        list(SelectionCriteria = list(ServingStatuses = list("ELIGIBLE", "RARELY_SERVED")),
                             FieldNames = list( "KeywordId", "AdGroupId", "CampaignId", "ServingStatus", "StrategyPriority"),
                             SearchFieldNames = list("Bid", "AuctionBids"),
                             NetworkFieldNames  = list("Bid", "Coverage"))
                        )
  
  # define variables
  if (! is.null(KeywordIds) ) {
    
    if ( "data.frame" %in% class( KeywordIds ) ) {
      KeywordIds <- KeywordIds$Id
    }
      object          <- "keywords"
      ids_parts_limit <- 10000
      ids             <- KeywordIds
      
    
  } else if (! is.null(AdGroupIds) ) {
    
    if ( "data.frame" %in% class( AdGroupIds ) ) {
      AdGroupIds <- AdGroupIds$Id
    }
    
    object          <- "adgroups"
    ids_parts_limit <- 1000
    ids             <- AdGroupIds
    
  } else {

    object          <- "campaings"
    ids_parts_limit <- 10
    
    
    if ( "data.frame" %in% class( CampaignIds ) ) {
      CampaignIds <- CampaignIds$Id
    }
    
    # if CampaignIds not set
    if ( is.null(CampaignIds) ) {
      message("You dont choised any ids of campaign, adgroup or ad. Loading full campaign list.")
      ids <-  yadirGetCampaignList(Logins        = Login,
                                           AgencyAccount = AgencyAccount,
                                           Token         = Token,
                                           TokenPath     = TokenPath)$Id
    } else {
      ids             <- CampaignIds
    }
    
  }
  
  # object for collect answers
  res <- list()
  
  # spliting for obj limit
  id_parts <- split(ids, 
                    ceiling(seq_along(ids) / ids_parts_limit))
  
  # start of coleccting
  for (part_of_id in id_parts) {
    
    if ( length(part_of_id) == 1 ) {
    
	part_of_id <- list(part_of_id)
	 
    }	
	
    # detect ids and object
    if ( object == "keywords" ) {
      query_list$params$SelectionCriteria$KeywordIds  <- part_of_id
    } else if ( object == "adgroups" ) {
      query_list$params$SelectionCriteria$AdGroupIds  <- part_of_id
    } else {
      query_list$params$SelectionCriteria$CampaignIds <- part_of_id
    }
    
    # create body
    queryBody <- jsonlite::toJSON(query_list, 
                                  auto_unbox = T, 
                                  pretty = T, 
                                  null = "null")  
    
    # send request
    answer <- POST("https://api.direct.yandex.com/json/v5/keywordbids", 
                   body = queryBody, 
                   add_headers(Authorization = paste0("Bearer ", Token), 
                               `Accept-Language` = "ru", 
                               `Client-Login` = Login))
    
    # add request id
    req_ids <- append(req_ids, headers(answer)$requestid)
    
    # check answer
    stop_for_status(answer)
    # pars answer
    dataRaw <- content(answer, "parsed", "application/json")
    
    # check answer for error
    if ( ! is.null(dataRaw$error) ) {
      message("Error:  ", dataRaw$error$error_string)
      message("Code:   ", dataRaw$error$error_code)
      message("Detail: ", dataRaw$error$error_detail)
      stop(dataRaw$error$error_detail)
    }
    
    # add to results list
    res <- append(res, list(dataRaw))
    
    
  }
  
  # answer level parsing result
  res_flatting <- list()
  
  # answer level
  for ( part_of_res in res ) {
    
    # keywordbid level
    for( kw_bid in part_of_res$result$KeywordBids ) {
      
      ## check AuctionBids for Search auctions
      if ( tolower(AuctionBids) == "search" && !is.na(AuctionBids) ) {
        # auction level
        for( auction_ser in kw_bid$Search$AuctionBids$AuctionBidItems ) {
          
             res_flatting <- append(res_flatting,
                               list(      
                               list(KeywordId            = kw_bid$KeywordId,
                                    AdGroupId            = kw_bid$AdGroupId,
                                    CampaignId           = kw_bid$CampaignId,
                                    ServingStatus        = kw_bid$ServingStatus,
                                    StrategyPriority     = kw_bid$StrategyPriority,
                                    SearchBid            = kw_bid$Search$Bid / 1000000,
                                    NetworkBid           = kw_bid$Network$Bid / 1000000,
                                    AuctionTrafficVolume = auction_ser$TrafficVolume,
                                    AuctionBid           = auction_ser$Bid / 1000000,
                                    AuctionPrice         = auction_ser$Price / 1000000)
                                )
                              )
        }
        
      ## check AuctionBids for Network auctions
      } else if (  tolower(AuctionBids) == "network"  && !is.na(AuctionBids) ) {
        # auction level
        for( auction_ser in kw_bid$Network$Coverage$CoverageItems ) {
          
          res_flatting <- append(res_flatting,
                                 list(      
                                   list(KeywordId        = kw_bid$KeywordId,
                                        AdGroupId        = kw_bid$AdGroupId,
                                        CampaignId       = kw_bid$CampaignId,
                                        ServingStatus    = kw_bid$ServingStatus,
                                        StrategyPriority = kw_bid$StrategyPriority,
                                        SearchBid        = kw_bid$Search$Bid / 1000000,
                                        NetworkBid       = kw_bid$Network$Bid / 1000000,
                                        Probability      = auction_ser$Probability,
                                        AuctionBid       = auction_ser$Bid / 1000000)
                                 )
          )
          
        }
        
      ## check AuctionBids without auctions
      } else {
        
        res_flatting <- append(res_flatting,
                               list(      
                                 list(KeywordId        = kw_bid$KeywordId,
                                      AdGroupId        = kw_bid$AdGroupId,
                                      CampaignId       = kw_bid$CampaignId,
                                      ServingStatus    = kw_bid$ServingStatus,
                                      StrategyPriority = kw_bid$StrategyPriority,
                                      SearchBid        = kw_bid$Search$Bid  / 1000000,
                                      NetworkBid       = kw_bid$Network$Bid / 1000000)
                               )
        )
        
    }
      
      
  }
}
  
  out <- map_df(res_flatting, flatten)
  end_time <- Sys.time()
  
  message("Duration: ", round(difftime(end_time, start_time, units = "secs"), 0), " secs")
  message("Total requests send: ", length(req_ids))
  message("RequestIDs: ", paste(req_ids, collapse = ", "))
  
  return(out)

}

---
title: "Авторизация в Яндекс Директ"
author: "Alexey Seleznev"
output: rmarkdown::html_vignette
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{ryandexdirect: Авторизация в Яндекс Директ}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Видео инструкция по авторизации
Для быстрого старта вы можете посмотреть этот [видео урок](https://youtu.be/hbUXttDFKNE) посвящённый авторизации в API Яндекс.Директ с помощью пакета `ryandexdirect`.

## Получение токена
Для генерации токена в пакете есть две функции, `yadirAuth()` и `yadirGetToken()`. Получить токен можно с помощью любой из этих функций, но я рекомендую использовать `yadirAuth`, функция `yadirGetToken()` считается устарвшей начиная с ryandexdirect 3.0.0.

Разница между ними состоит в том, что `yadirGetToken()` просто генерирует токен и перенаправляет вас на страницу где вы можете его скопировать, а `yadirAuth` первый раз направляет вас на старницу где будет сгенерирован код для получения токена, после чего сама запрашивает токен и сохраняет в файл, в дальнейшем сама его обновляет и работает с сохранённым файлом.

Вы можете передавать токен в виде строки в агрумент Token при работе с любой из функций, но этот метод является устаревшим и не эффективным способом работы с функциями пакета ryandexdirect.

В версии 3.5.0 в пакет была добавлена новая функция для прохождения аутентификации в API `yadirAuth()`, но отдельно вызывать её для получения токена больше не требуется, при любом обращении к API если вы в используемой функции явно не указываете никакой токен, любая функция пакета сначала будет искать файл с сохранёнными учётными данными в рабочей директории, или в папке которую вы укажите в аргументе *AgencyAccount*, если файл найден то учётные данные будут загружены из него, если файл не был найден то будет открыт браузер и вам потребуется разрешить приложению **ryandexdirect** доступ к своему рекламному аккауну, далее будет сформирован код авторизации, который вам необходимо вставить в R консоль в ответ на запрос *"Enter authorize code:"*, после чего функция сама сохранит учётные данные в файл с именем *login.yadirAuth.RData*, где вместо login будет подставлен логин аккаунта к которому вы предоставили доступ, это необходимо для сохранения токенов полученных для разных аккаунтов. При следующих обращениях, все функции пакета будут запрашивать токен из сохранённого файла.

### Пример кода для прохождения авторизации
`aut <- yadirAuth(Login = "Ваш логин в яндексе", NewUser = TRUE, TokenPath = "token_yandex")`

### Обновление токена
При каждом обращении к API проверяется количество дней до того как используемый токен станет просроченным, если остаётся менее 30 дней токен автоматически будет обновлён, и файл с учётными данными перезаписан.

### Аргументы функции yadirAuth
+ **Login** - имя пользователя или электронный адрес на Яндексе, с которого есть доступ к нужному вам рекламному аккаунту
+ **NewUser** - логическое выражаение TRUE или FALSE, признак того, что у пользователя обязательно нужно запросить разрешение на доступ к аккаунту (даже если пользователь уже разрешил доступ данному приложению). Получив этот параметр, Яндекс.OAuth предложит пользователю разрешить доступ приложению и выбрать нужный аккаунт Яндекса. Параметр полезен, например, если вы хотите переключиться на другой аккаунт. По умолчанию установлено значение FALSE.
+ **TokenPath** - путь к папке, в которой будут сохраняться учётные данные, по умолчанию это рабочая директория, но можно указывать любой путь, как абсолютный например "C:/my_r_files/tokens", так и относительный "tokens", при использовании относительного пути в рабочей директории будет создана папка с установленным вами названием, и в неё будут сохраняться файлы с расширением .RData в которых хранятся учётные данные.

Ещё раз обращаю внимание на то, что хоть вы в любой момент при желании можете пройти аутентификацию в API Яндекс Директ с помощью функции yadirAuth, но это действие не является обязательным, т.к. при использовании любой функции пакета будет запущен поиск файла с учётными данными, и если файл не будет найден, то автоматически будет запущен процесс авторизации и запроса токена.


### Авторизация с помощью функции `yadirGetToken()`
Функция для получения токена для доступа к API Яндекс.Директ с помощью упрощённой одноэтапной аутентификации. Полученый токен используется во всех остальных функциях.
Перед запуском функции необходимо авторизироваться в яндексе под логином через который есть доступ к аккаунту Яндекс.Директ, далее запустить функции, в окне браузера подтвердить разрешение на доступ к данным.


<center>
<img src="https://selesnow.github.io/ryandexdirect/getToken/gettoken2.gif" alt="авторизация" width="700">
</center>


Полученный токен необходимо скопировать с сайта и ввести в консоль R, далее вам будет предложено сохранить полученный токен в файл, рекомендуется отвечать `y`, в таком случае вам не придётся проходить авторизации при старте новой R сессии.

#### Можно получить токен воспользовавшись кнопкой
Ещё один способ получить токен, скопировать его, и использовать во всех остальных функциях как обычную текстовую строку.

<center>
<Br>
<p class="buttond"  style="text-align:center;"><a href="https://oauth.yandex.ru/authorize?response_type=token&client_id=365a2d0a675c462d90ac145d4f5948cc" target="_self" style="cursor: pointer; font-size:14px;  text-decoration: none; padding:10px 20px; color:#383a3b; background-color:#bbc0c4; border-radius:5px; border: 3px solid #383a3b;">Получить токен доступа к API!</a></p>
<Br>
</center>

Если у вас не отображается кнопка перейдите на страницу официальной документации ryandexdirect по [этой ссылке](https://selesnow.github.io/ryandexdirect/#yadirgettoken).

## Аргументы, общие для всех функций
Во всех функции пакета существуют общие аргументы, в дальнейшем эти аргументы рассматриваться в описании функций не будут.

+ **Login / Logins** - Логин или вектор содержащий логины клиентов агентского аккаунта, в этот аргумент необходимо указывать логин в случае если вы запрашиваете данные из обычного рекламного аккаунта Яндекс Директ, или же логин клиентского аккаунта привязаного к вашему агентскому аккаунту, в таком случае в рабочей директории будет создан отдельный файл под каждый рекламный аккаунт, в котором будут хранится нужные для работы учётные данные.
+ **Token** - С версии 3.0.0 этот аргумент больше не является обязательным, вы по прежнему можете передавать в него токены в виде строки, полученные с помощью функции yadirGetToken, так же можете передавать объекты учётных данных полученные с помощью новой функции yadirAuth, либо просто опустить данный агрумент.
+ **AgencyAccount** - В случае если вы работаете с агентского аккаунта в этот аргумент необходимо передавать логин вашего агентского аккаунта, для каждого агентского аккаунта с которым вы работаете так же будет создан отдельный файл для хранения учётных данных.
+ **TokenPath** - Путь к папке в которой хранятся все файлы с учётными данными, по умолчанию все функции пакета пытаются найти нужный файл с хранящимися учётными данными в текущей рабочей директории, но вы можете указывать абсолютный или условный путь к совершенно любой папке на вашем ПК, в которой вы хотите хранить все учётные данные ваших аккаунтов Яндекс Директ. Пример условного пути TokenPath = "yandex_token", в таком случае в рабочей директории будет создана папка yandex_token в которую и будут сохраняться все учётные данные, указав этот путь во всех функциях пакета вам не понадобится повторно запрашивать токены.

### Работа с обычным рекламным аккаунтом
Таким образом при работе с обычным рекламным аккаунтом вам необходимо передавать логин этого аккаунта в аргументе Login или Logins. При запуске любой функции в первую очередь будет запущен процесс поиска учётных данных для заданного логина в папке, название или путь к которой указан в аргументе TokenPath. Если файл с учётными данными для заданного логина был найден то работа функции будет продолжена без вашего вмешательства, если учётные данные для данного логина ещё не были получены, или были получены но вы не указали путь к папке в которой данный файл был сохранён то автоматически будет запущен веб браузер и вам потребуется пройти авторизацию и разрешить доступ пакету ryandexdirect к вашему рекламному аккаунту, учётные данные при этом будут сохранены в локальный файл и при дальнейшей работе будут загружаться именно из файла.

### Работа с агентским аккаунтом
Для получения статистики по рекламным аккаунтам прикреплённым к агентскому аккаунту необходимо использовать аргумент AgencyAccount, в который необходимо передать логин или почту для вашего агентского аккаунта, а в аргумент Login / Logins передавать логин клиента, или вектор содержащий логины клиентов данного агентского аккаунта, по которым вы хотите запросить какие либо данные или совершить ещё какие либо действия.

Авторизация в данном случае будет проходить по агентскому аккаунту.

## Функции для переключения между аккаунтами
Прописывать в каждой отдельной функции аргументы `Login`, `TokenPath`, `AgencyAccount` неудобно, особенно если в ходе сессии работаете только с одним аккаунтом. 

Вы можете использовать следующие, вспомогательные функции для того, что бы посмотреть список всех аккаунтов к которым вы сгенерировали файлы с учётными данными, и для переключения между этими аккаунтами.

* `yadirSetLogin()` - позволяет вам установить логин который будет использоваться в ходе сессии по умолчанию, так же в ходе сессии с помощью этой функции вы можете переключаться между аккаунтами.
* `yadirSetAgencyAccount() - позволяет вам установить логин центра клиентов который будет использоваться в ходе сессии по умолчанию, так же в ходе сессии с помощью этой функции вы можете переключаться между центрами клиентов к которым у вас есть доступ.
* `yadirGetLogins()` - позволяет вам запросить логины всех аккаунтов к которым вы сгенерировали и созранили учётные данные, далее, при работе в интерактивном режиме функция выведит их список, и вы можете указать номер аккаунта, который будет установлен в текущей сессии по умолчанию. 

## Опции и переменные среды
Ещё одна возможность избежать дублирования кода - установить дефолтные значения через опции или переменные среды. 

### Опции
Пакет `ryandexdirect` на данный момент поддерживает следующие опции:

* `ryandexdirect.user` - логин по умолчанию
* `ryandexdirect.agency_account` - логин центра клиентов по умолчанию
* `ryandexdirect.token_path` - путь к директории в которой хранятся файлы с полученными учётными данными

Устанавливаются опции с помощью команды `options()`.

```{r eval=FALSE}
options(ryandexdirect.user = 'my_yandex_login',
        ryandexdirect.agency_account = 'my_client_centre_login',
        ryandexdirect.token_path = 'C:/auth/yandex_direct')
```

Используя пример кода для установки опций вам необходимо подставить значения вместо:

* my_yandex_login - логин подчинённого аккаунта, или обычного аккаунта яндекс директ
* my_client_centre_login - логин центра клиентов, если указанный вами подчинённый аккаунт является клиентом в вашем центре клиентов
* 'C:/auth/yandex_direct' - путь к директории в которой хранятся, или будут хранится учётные данные полученные вами при авторизации

### Переменные среды
Помимо опций `ryandexdirect` поддерживает работу с переменными среды, их можно установить, что бы не прописывать в каждом отдельном скрипте опции.

На данный момент поддерживаются следующие переменные среды:

* RYD_USER - логин по умолчанию
* RYD_AGENCY - логин центра клиентов по умолчанию
* RYD_TOKEN_PATH - путь к директории в которой хранятся файлы с полученными учётными данными

Прописать переменные среды можно либо в файле `.Renviron`

```
RFB_USER="my_yandex_login"
RFB_TOKEN_PATH="C:/auth/yandex_direct"
```

Либо вы можете использовать команду `Sys.setenv()`:

```{r eval = FALSE}
Sys.setenv(RFB_USER="my_yandex_login"
           RFB_TOKEN_PATH="C:/auth/yandex_direct")
```

Либо использовать возможности вашей операционной системы, для создания переменных среды.

## Функции для быстрого переключения между аккаунтами
В принципе вы можете указывать нужный аккаунт с помощью аргумента `Login`, в любой из функций пакета, так же вы можете указать нужный вам аккаунт через опции или переменные среды, но начиная с версии 3.6.0 в пакет были включены новые функции, которые позволяют повысить читаемость вашего кода, и упростить переход между аккаунтами.

+ `yadirGetLogins()` - Показывает все логины под которыми вы прошли авторизацию, при этом запрашиваются те логины которые учётные данные которых сохраненны в указанной папке с помощью аргумента `TokenPath`, либо опции `ryandexdirect.token_path` или переменной среды `RYD_TOKEN_PATH`.
+ `yadirSetLogin()` - Позволяет установить дефолтный логин, который будет использоваться в рамках сессии.
+ `yadirSetAgencyAccount()` - Позволяет установить дефолтный центр клиентов, который будет использоваться в рамках сессии. 

Обо всех подробностях использования данных функций вы можете узнать из данного [видео урока](https://youtu.be/hbUXttDFKNE).

---
title: "Прогноз бюджета и отчёты сервиса WordStat"
author: "Alexey Seleznev"
output: rmarkdown::html_vignette
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{ryandexdirect: Прогноз бюджета и отчёты сервиса WordStat}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(ryandexdirect)
```

# Отчёт WordStat
WordStat — это бесплатный сервис Яндекса, предназначенный для оценки пользовательского интереса к различным тематикам и подбора ключевых слов для SEO-оптимизации и контекстной рекламы.

Для того, что бы получить отчёты из данного сервиса с помощью пакета `ryandexdirect` необходимо использовать функцию `ydirGetWordStatReport()`.

Функция `ydirGetWordStatReport()` возвращает те же сведения, которые доступны в сервисе [Подбор слов](https://wordstat.yandex.ru/). Статистика рассчитывается за прошедший месяц.

## Видео урок
Помимо данной виньетки данная тема рассмотренна в видео уроке, который доступен по [ссылке](https://youtu.be/g9lwjsgAhyY).

## Аргументы функции ydirGetWordStatReport
* Phrases - Вектор ключевых фраз, по которым требуется получить статистику поисковых запросов (не более 10 фраз в кодировке UTF-8). Ключевые фразы могут содержать минус-фразы. Минус-фразу из нескольких слов следует заключить в скобки, например: `холодильник -морозильник -(морозильная камера) -ремонт`.
* GeoID - Вектор идентификаторов регионов. Позволяет получать статистику поисковых запросов, сделанных только в указанных регионах. Чтобы исключить регион, перед его идентификатором ставят минус, например [1,-219] — получить статистику для Москвы и Московской области, кроме Черноголовки. Статистика выдается суммарно по всем плюс-регионам, из которых исключены все минус-регионы. Получить справочник регионов можно с помощью функции `yadirGetDictionary()`

Остальные аргументы функции отвечают исключительно за авториацию, и с ними вы можете ознакомится в виньетке `vignette('yandex-direct-auth', "ryandexdirect")`.

## Пример загрузки отчёта по ключевым словам из Wordstat
В одном запросе вы можете получитьданные максимум по 10 ключевым фразам. 

Ключевые фразы могут содержать минус-фразы. Минус-фразу из нескольких слов следует заключить в скобки, например: `холодильник -морозильник -(морозильная камера) -ремонт`.

Вы можете не указывать регион, и получить общие данные о популярности поисковых запросов.

```{r}
s_ws_rep <- yadirGetWordStatReport(c('купить смартфон',
                                     'купить сотовый телефон',
                                     'купить мобильный',
                                     'samsung -(серого цвета)'),
                                   Login = "selesnow")
```

В ходе формирования и парсинга запроса в консоли вы можете наблюдать за прогрессом выполнения операции.
```
.Send report
.Report status: Pending
.Report status: Pending
.Report status: Pending
.Report status: Done
.Get report
.Parse report
.Delete report
Success!
Duration: 29 sec.
Request ID: 2685785695974693399
WordStat Report ID: 2016016472
Report SearchedAlso has 55 rows
Report SearchedWith has 900 rows
For get report data x[['ReportName']]
```

* Send report - Отчёт отправлен на API сервер
* Report status: Pending - Отчёт формируется на сервере API
* Report status: Done - Отчёт сформирован
* Get report - Получен отчёт от API сервера
* Parse report - Выполняется преобразование полученного от API результата в таблицы
* Delete report - Выполняется удаление отчёта с сервера API

В результате работы вы получите список, который состоит из двух таблиц: *SearchedWith* и *SearchedAlso*.

Обратиться к каждой из этих таблиц можно двумя способами:
```{r}
# Что искали с заданными ключевыми словами
s_ws_rep$SearchedWith
s_ws_rep[['SearchedWith']]

# Запросы, похожие на заданные ключевые слова
s_ws_rep$SearchedAlso
s_ws_rep[['SearchedAlso']]

```

Образение через `$` и по имени в двойных квадратных скобках равнозначны.

Если вам необходимо получить справочник проектов, и получить статистику по Москве, Санкт - Петербургу, Екатеринбург и Владивостоку, то изначально вам необходимо запросить справочник регионов с помощью функции `yadirGetDictionary()` и получить идентификаторы нужных регионов, а потом передать их в аргумент `GeoId`.
```{r}
library(dplyr)

# запрашиваем справочник регионов
regions <- yadirGetDictionary(Login = "selesnow")

# оставляем толлько нужные регионы
rep_regions <- regions %>%
                  filter(GeoRegionName %in% c("Москва",
                                              "Санкт-Петербург",
                                              "Екатеринбург",
                                              "Владивосток"))

# запрашиваем отчёт
reg_ws <- yadirGetWordStatReport(Phrases = c('купить смартфон -xiaomi', 
                                             'купить xiaomi'),
                                 GeoID   = rep_regions$GeoRegionId,
                                 Login   = "selesnow")
```

Так же вы можете указать минус регионы, например если вы хотите получить статистику по всей России без Москвы, Санкт - Петербуруа, Екатеринбурга и Владивостока. Для этого запросите справочник регионов, сначала получите идентификатор России (225), затем идентификаторы минус регионов, и перед идентификаторами минус регинов поставьте знак '-'. 

В приведённом ниже примере кода это реализуется с помощью функции `mutate()` из пакета `dplyr`.
```
library(dplyr)

# запрашиваем справочник регионов
regions <- yadirGetDictionary(Login = "selesnow")

# получаем идентификатор России
regions <- regions %>%
  filter(GeoRegionName %in% c("Россия"))

# Получаем идентификаторы минус регионов, и ставим перед ними минус
minus_regions <- regions %>%
                  filter(GeoRegionName %in% c("Москва",
                                              "Санкт-Петербург",
                                              "Екатеринбург",
                                              "Владивосток")) %>%
                  mutate(GeoRegionId = paste0("-", GeoRegionId))

# формируем запрос
reg_ws <- yadirGetWordStatReport(Phrases = c('купить смартфон -xiaomi', 
                                             'купить xiaomi'),
                                 GeoID   = c(regions$GeoRegionId, minus_regions$GeoRegionId),
                                 Login   = "selesnow")
```

# Получить прогноз показов, кликов и затрат
С помощью функции `yadirGetForecast()` вы можете получить прогнозируемые данные по расходам, показам и кликам указав интересующие вас ключевые фразы и регионы.

## Аргументы функции yadirGetForecast
* Phrases - Вектор ключевых фраз, для которых требуется получить прогноз (не более 100 фраз)
* GeoID - Вектор, содержащий идентификаторы регионов для составления прогноза. Если не задан, прогноз составляется по всем регионам.
* Currency - Валюта, в которой должны быть выражены цены клика и суммарные затраты в отчете. Возможные значения: RUB, CHF, EUR, KZT, TRY, UAH, USD, BYN.
* AuctionBids - Выводить ли в отчете результаты торгов — Yes/No или `TRUE` / `FALSE`. Если не задано, подразумевается No.

Пример кода для получения прогноза с использованием всех аргументов, минус слов и минус регионов с результататми торгов:
```{r}
library(dplyr)

# запрашиваем справочник регионов
regions <- yadirGetDictionary(Login = "selesnow")

# получаем идентификатор России
regions <- regions %>%
  filter(GeoRegionName %in% c("Россия"))

# получаем идентификатор минус регионов и помечаем их минусом
minus_regions <- regions %>%
  filter(GeoRegionName %in% c("Москва",
                              "Санкт-Петербург",
                              "Екатеринбург",
                              "Владивосток")) %>%
  mutate(GeoRegionId = paste0("-", GeoRegionId))

# запрашиваем прогноз с результататми торгов
forecast <- yadirGetForecast2(Phrases = c('купить смартфон -xiaomi', 
                                         'купить xiaomi',
                                         'самсунг -(серого цвета)'),
                             GeoID   = c(regions$GeoRegionId, minus_regions$GeoRegionId),
                             AuctionBids = 'Yes',
                             Login   = "selesnow")
```

В результате в объекте *forecast* будет список из двух таблиц:
* PhrasesForecast - Таблица прогноза в разреще всех указанных фраз
* CommonForecast - Общий суммарный прогноз

Обратится к таблицам можно по имени указав его через `$` или в двойных квадратных скобках.
```r
# прогноз в разрезе фраз
forecast$PhrasesForecast

# оющий прогнох
forecast[['CommonForecast']]
```

## Описание полей полученной таблицы прогнозов
Таблица *PhrasesForecast* сожержит следующие поля:

* PhrasesForecast - Ключевая фраза, для которой составлен прогноз.
* IsRubric - Признак того, что фраза является рубрикой Яндекс.Каталога.
* Min - Средневзвешенная цена клика в нижнем блоке на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* Max - Средневзвешенная цена клика на первом месте в нижнем блоке на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* PremiumMin - Средневзвешенная цена клика в спецразмещении на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* PremiumMax - Средневзвешенная цена клика на первом месте в спецразмещении на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* Shows - Возможное количество показов объявления по данной фразе за прошедший месяц.
* Clicks - Возможное количество кликов по объявлению в нижнем блоке (кроме первого места) за прошедший месяц.
* FirstPlaceClicks - Возможное количество кликов по объявлению на первом месте в нижнем блоке, за прошедший месяц.
* PremiumClicks - Возможное количество кликов по объявлению в спецразмещении за прошедший месяц.
* CTR - CTR при показе в нижнем блоке, в процентах. Рассчитывается по формуле: `Clicks/Shows * 100`
* FirstPlaceCTR - CTR при показе на первом месте в нижнем блоке. Рассчитывается по формуле: `FirstPlaceClicks/Shows * 100`
* Currency - Валюта, в которой выражены цены клика и суммарные затраты в отчете. Возможные значения: RUB, CHF, EUR, KZT, TRY, UAH, USD, BYN.
* Position - Позиция показа: *Pmn*, где: m — номер блока (1 — спецразмещение, 2 — блок гарантированных показов); 
n — номер позиции в рамках блока. Например, P12 — второе место в спецразмещении, P21 — первое место в блоке гарантированных показов.
* Bid - Минимальная ставка за указанную позицию (в валюте, указанной в параметре `Currency`). Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* Price - Списываемая цена для указанной позиции (в валюте, указанной в параметре `Currency`). Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.

Таблица *CommonForecast* сожержит следующие поля:

* Geo - Список регионов, которые указаны при формировании отчета 
* Min - Средневзвешенная цена клика в нижнем блоке на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* Max - Средневзвешенная цена клика на первом месте в нижнем блоке на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* PremiumMin - Средневзвешенная цена клика в спецразмещении на момент составления прогноза. Значение конвертируется в валюту, указанную в параметре `Currency`, и округляется вверх с точностью до шага торгов в этой валюте.
* Shows - Возможное количество показов объявления по данной фразе за прошедший месяц.
* Clicks - Возможное количество кликов по объявлению в нижнем блоке (кроме первого места) за прошедший месяц.
* FirstPlaceClicks - Возможное количество кликов по объявлению на первом месте в нижнем блоке, за прошедший месяц.
* PremiumClicks - Возможное количество кликов по объявлению в спецразмещении за прошедший месяц.

---
title: "Загрузка статистики из рекламного аккаунта Яндекс Директ"
author: "Alexey Seleznev"
output: rmarkdown::html_vignette
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{ryandexdirect: Загрузка статистики из рекламного аккаунта Яндекс Директ}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

Для начала работы с пакетом предварительно его требуется подключить.

```{r setup}
library(ryandexdirect)
```

Для загрузки статистики  =в пакете `ryandexdirect` предназначена функция `yadirGetReport`. Данная функция взаимодействует с API 'Сервис Reports', подробно о нём можно узнав в офйициальной [документации](https://yandex.ru/dev/direct/doc/reports/reports-docpage), в текущей виньетке так же используется материал официальной спавки по работе с API сервиса 'Reports'.

## Аргументы функции yadirGetReports

* ReportType - Текстовое значение, тип отчёта. Возможные значения: ACCOUNT_PERFORMANCE_REPORT, CAMPAIGN_PERFORMANCE_REPORT, ADGROUP_PERFORMANCE_REPORT, AD_PERFORMANCE_REPORT, CRITERIA_PERFORMANCE_REPORT, CUSTOM_REPORT, SEARCH_QUERY_PERFORMANCE_REPORT. Более подробно о типах отчёта будет рассказано ниже.
* DateRangeType - Строковое значение, тип временного диапазона. Возможные значения: TODAY, YESTERDAY, LAST_3_DAYS, LAST_5_DAYS, LAST_7_DAYS, LAST_14_DAYS, LAST_30_DAYS, LAST_90_DAYS, LAST_365_DAYS, THIS_WEEK_MON_TODAY ,THIS_WEEK_SUN_TODAY, LAST_WEEK, LAST_BUSINESS_WEEK, LAST_WEEK_SUN_SAT, THIS_MONTH, LAST_MONTH, ALL_TIME, CUSTOM_DATE, AUTO. По умолчанию LAST_30_DAYS.
* DateFrom - Дата начала отчётного периода в формате ГГГГ-ММ-ДД. Аргумент используется только если DateRangeType равен CUSTOM_DATE.
* DateTo - Дата завершение отчётного периода в формате ГГГГ-ММ-ДД. Аргумент используется только если DateRangeType равен CUSTOM_DATE.
* FieldNames - Текстовый вектор содержащий названия полей статистику по которым вы хотите получить. Пример:  `c("Date","CampaignName","Impressions","Clicks")`. Актуальный список полей можно найти по [ссылке](https://yandex.ru/dev/direct/doc/reports/fields-list-docpage).
* FilterList - Текстовый вектор содержащий логические выражения для фильтрации данных. Например `c("Clicks GREATER_THAN 99","Impressions LESS_THAN 1000")`.
* Goals - Целочисленный вектор содержащий идентификаторы целей Яндекс.Метрики по которым вы планируете получить статистики. Пример `c(182453, 182452, 23458860)`. Получить список идентификаторов целей можно с помозью функции `rym_get_goals` входящей в пакет `rym`.
* AttributionModels - Текстовое значение содержащее аббревиатуру названия нужной модели атрибуции для расчёта количества достигнутых целей. Пример ` c("LSC", "LC", "FC")`. Более подробно можно узнать в разделе "Модели атрибуции".
* IncludeVAT - Текстовое значение. Включать ли НДС в денежные суммы в отчете. Возможные вариенты: YES, NO. Влияет на расчёт данных в полях AvgCpc, AvgCpm, Cost, CostPerConversion, а также Profit и Revenue.
* IncludeDiscount - Текстовое значение. Учитывать ли скидку при расчёте денежных данных. Возможные значения: YES, NO.
* FetchBy - Строковое значение. Позволяет разбивать запрос на подзапросы по временному интервалу.
* Login - Текстовый вектор, содержащий логины аккаунтов из которых необходимо получить статистику.
* AgencyAccount - Текстовое значение, логин агентского аккаунта, к которому подключены аккаунты перечисленные в Login.
* TokenPath - Текстовое значение, путь к папке в которой храняться, или будут сохраненны учётные данные для работы с API.
* Token - Текстовое значение или объект полученный с помощью функции `yadirAuth` или `yadirGetToken`. Авториа=зационный токен для работы с API Яндекс.Директ.
* SkipErrors - Логическое значение, используется как правило при загрузке статистики из множества аккаунтов, и позволяет игнорировать ошибки полученные в ходе загрузки данных из некоторых аккаунтов из списка. При этом в консоль будут выводиться сообщения об ошибках.

## Тип отчета
В параметре ReportType укажите тип отчета. Тип отчета влияет на набор доступных полей и группировку данных.

Например, если выбран тип отчета SEARCH_QUERY_PERFORMANCE_REPORT, то данные в отчете будут сгруппированы по идентификатору группы AdGroupId и поисковому запросу Query. Обратите внимание, что добавление группировок данных не приводит к автоматическому добавлению соответствующих полей в отчет: отчет содержит только поля, явным образом перечисленные в параметре FieldNames.

Наиболее общий тип отчета — CUSTOM_REPORT. Он не добавляет никаких дополнительных группировок.

Типы отчетов представлены в таблице.

```{r, eval = TRUE, results = "asis", echo=FALSE}
library(magrittr)
table_report_types <- data.frame(reptype = c("ACCOUNT_PERFORMANCE_REPORT", 
                                             "CAMPAIGN_PERFORMANCE_REPORT", 
                                             "ADGROUP_PERFORMANCE_REPORT",
                                             "AD_PERFORMANCE_REPORT",
                                             "CRITERIA_PERFORMANCE_REPORT",
                                             "CUSTOM_REPORT",
                                             "REACH_AND_FREQUENCY_PERFORMANCE_REPORT",
                                             "SEARCH_QUERY_PERFORMANCE_REPORT"), 
                                 descr    = c("Статистика по аккаунту рекламодателя",
                                              "Статистика по кампаниям",
                                              "Статистика по группам объявлений",
                                              "Статистика по объявлениям",
                                              "Статистика по условиям показа",
                                              "Статистика с произвольными группировками",
                                              "Статистика по медийным кампаниям. Отчет содержит только данные по кампаниям с типом «Медийная кампания», кампании остальных типов игнорируются",
                                              "Статистика по поисковым запросам"),
                                 grouping = c("–", 
                                              "CampaignId", 
                                              "AdGroupId",
                                              "AdId",
                                              "AdGroupId, CriteriaId, CriteriaType",
                                              "–",
                                              "В запросе на формирование отчета необходимо указать в поле FieldNames значение CampaignId", 
                                              "AdGroupId, Query"))

names(table_report_types) <- c("Тип отчёта", "Описание", "Добавляется группировка")
table_report_types <- kableExtra::kable(table_report_types)
kableExtra::kable_styling(table_report_types, bootstrap_options = c("striped", "hover"))
```

Во всех типах отчетов применяется единичная атрибуция (single attribution): каждый показ и клик относится только к одному условию показа, региону, возрасту пользователя и т.д.

## Период отчёта
* TODAY — текущий день;
* YESTERDAY — вчера;
* LAST_3_DAYS, LAST_5_DAYS, LAST_7_DAYS, LAST_14_DAYS, LAST_30_DAYS, LAST_90_DAYS, LAST_365_DAYS — указанное количество предыдущих дней, не включая текущий день;
* THIS_WEEK_MON_TODAY — текущая неделя начиная с понедельника, включая текущий день;
* THIS_WEEK_SUN_TODAY — текущая неделя начиная с воскресенья, включая текущий день;
* LAST_WEEK — прошлая неделя с понедельника по воскресенье;
* LAST_BUSINESS_WEEK — прошлая рабочая неделя с понедельника по пятницу;

![Последняя рабочая неделя](http://img.netpeak.ua/alsey/155897903366_kiss_10kb.png)

* LAST_WEEK_SUN_SAT — прошлая неделя с воскресенья по субботу;
* THIS_MONTH — текущий календарный месяц;
* LAST_MONTH — полный предыдущий календарный месяц;
* ALL_TIME — вся доступная статистика, включая текущий день;
* CUSTOM_DATE — произвольный период. При выборе этого значения необходимо указать даты начала и окончания периода в параметрах DateFrom и DateTo.
* AUTO — период, за который статистика показов и кликов могла измениться. Период выбирается автоматически в зависимости от того, произошла ли в предыдущий день корректировка статистики. 

## Поля отчёта
Полный список допустимых полей можно найти в официальной [документации](https://yandex.ru/dev/direct/doc/reports/fields-list-docpage).

Поля указываются в следующих параметрах отчета:

* FieldNames — поля, которые выводятся в отчете (столбцы);
* Field в структуре Filter — поля, которые используются для фильтрации данных в отчете;
* Field в структуре OrderBy — поля, которые используются для сортировки строк в отчете.

Наборы допустимых полей в этих параметрах различаются. Например, поле *CampaignName* может быть добавлено как столбец в отчете, но не может использоваться для фильтрации данных, а поле *Keyword* — наоборот, используется только для фильтрации данных и в отчете не выводится.

Тип отчета также влияет на набор допустимых полей и их назначение, смотрите таблицу по [ссылке](https://yandex.ru/dev/direct/doc/reports/fields-list-docpage).

Каждое поле в зависимости от выбранного типа отчёта может иметь свой тип:


* фильтр — поле используется только для фильтрации данных и не может быть выведено в отчете;
* сегмент — при выводе в отчете поле используется для группировки данных;
* метрика — поле содержит числовое значение, рассчитанное в соответствии с заданными фильтрами и группировками;
* атрибут — поле содержит фиксированное значение в соответствии с заданными группировками, добавление такого поля в отчет не приводит к появлению новой группировки.

Например, поле *CampaignId* для типа отчета *CUSTOM_REPORT* является сегментом: если его добавить в отчет, данные будут сгруппированы по кампаниям. А для типа отчета *ADGROUP_PERFORMANCE_REPORT* поле *CampaignId* является атрибутом: данные уже сгруппированы по *AdGroupId*, а идентификатор кампании является для каждой группы фиксированным значением.

## Фильтрация данных
Для фильтрации данных в отчете используйте аргумент `FilterList`. Каждый фильтр представляет собой критерий отбора данных. Фильтры объединяются по условию AND: в отчет попадают данные, для которых выполнены все фильтры. Фильтр состоит из трех параметров:

* Field — значение какого поля сравнивается;
* Operator — каким образом сравнивается;
* Values — массив символьных значений, с которыми выполняется сравнение.

Например, чтобы отобрать в отчет строки, в которых количество целевых визитов больше 10, используйте фильтр `FilterList = "Conversions GREATER_THAN 10"`. Для применения нескольких фильтров перечислите их выражения в векторе, например `FilterList = c("Clicks GREATER_THAN 99","Impressions LESS_THAN 1000")` оставит строки в которых более 99 кликов и менее 1000 показов.

Соответствие полей и операторов представлено в таблице.

```{r, eval = TRUE, results = "asis", echo=FALSE}
library(magrittr)
table_report_types <- data.frame(field = c("AdNetworkType, CampaignId, CampaignType", 
                                             "AdFormat, AdGroupId, AdId, Age, AudienceTargetId, CarrierType, ClickType, CriteriaType, CriterionType, Device, DynamicTextAdTargetId, ExternalNetworkName, Gender, LocationOfPresenceId, MatchType, MobilePlatform, Placement, RlAdjustmentId, Slot, SmartBannerFilterId, TargetingLocationId", 
                                             "Clicks, Conversions, ImpressionReach, Impressions",
                                             "AvgClickPosition, AvgCpc, AvgCpm, AvgImpressionFrequency, AvgImpressionPosition, AvgPageviews, AvgTrafficVolume, BounceRate, ConversionRate, Cost, CostPerConversion, Ctr, GoalsRoi, ImpressionShare, Profit, Revenue, WeightedCtr, WeightedImpressions, ",
                                             "Keyword, MatchedKeyword, Query"), 
                                 operators    = c("EQUALS, IN",
                                              "EQUALS, IN, NOT_EQUALS, NOT_IN",
                                              "EQUALS, IN, GREATER_THAN, LESS_THAN",
                                              "GREATER_THAN, LESS_THAN",
                                              "EQUALS, IN, NOT_EQUALS, NOT_IN, STARTS_WITH_IGNORE_CASE, STARTS_WITH_ANY_IGNORE_CASE, DOES_NOT_START_WITH_IGNORE_CASE, DOES_NOT_START_WITH_ALL_IGNORE_CASE"))

names(table_report_types) <- c("Имя поля", "Доступные операторы")
table_report_types <- kableExtra::kable(table_report_types)
kableExtra::kable_styling(table_report_types, bootstrap_options = c("striped", "hover"))
```

Все денежные значения в фильтрах следует указывать в виде целых чисел: сумм в валюте, умноженных на 1 000 000.

## Цели
Вы можете запрашивать количество достижений каждой отдельно взятой цели с помощью аргумента *Goals*.

Данный аргумент принимает вектор из набора идентификаторов целей, при этом в одном запросе можно запрашивать данные не более чем по 10 целям. Получить идентификаторы цели можно несколькими способами:

* С помощью пакета `rym` и входящей в него функции `rym_get_goals`;
* В веб интерфейсе Яндекс Метрики: Настройка -> Цели -> Номер цели (в таблице целей).

Если параметр указан, то в отчете вместо полей *ConversionRate*, *Conversions*, *CostPerConversion*, *GoalsRoi* и *Revenue* с агрегированными данными по всем целям будут выведены аналогичные поля с именами вида *<поле>_<id_цели>_<модель_атрибуции>* и данными по каждой цели в отдельности.

## Модели атрибуции
Модель атрибуции — это правило, какой переход считать источником визита:

* **Первый переход** (FC) — источником любых визитов посетителя считается его первый переход на сайт за последние 180 дней. Эта модель позволяет отследить источник рекламы, который впервые привел клиента и тем самым повлиял на все последующие взаимодействия с сайтом.
* **Последний переход** (LC) — источником визита считается переход, в результате которого посетитель в данный момент пришел на сайт, без учета истории визитов. Эта модель может использоваться при техническом анализе сайта. Для оценки эффективности работы рекламных кампании используйте модель Последний значимый переход.
* **Последний значимый переход** (LSC) — все источники переходов за последние 90 дней условно разделяются на значимые (ваши объявления) и незначимые (переходы с сохраненных страниц, внутренние переходы или прямые заходы на сайт). Источником визита считается переход по объявлению. Если визит произошел после незначимого перехода, то его источником считается последний переход по объявлению. Эта модель позволяет учитывать конверсии, которые теряются в модели Последний переход из-за технического ограничения продолжительности визита.
* **Последний переход из Директа** (LYDC) - из всех последних значимых переходов учитывается только Директ. Если пользователь хотя бы раз перешел на сайт по объявлению в Директе, то именно этот переход считается источником всех последующих визитов — пока пользователь не перейдет по объявлению еще раз. Эта модель помогает увидеть все визиты, связанные с Директом, в том числе те, которые в других моделях атрибуции были бы отнесены к другим источникам.

Модели атрибуции, используемые при расчете данных по целям Яндекс.Метрики. Аргумент *AttributionModels* можно указать, только если указан параметр Goals. Если параметр Goals указан, а параметр AttributionModels — нет, по умолчанию используется значение LSC.

Если указано несколько моделей атрибуции, данные будут выведены по каждой модели в отдельности.

## FetchBy разбивка запросов по времени
В сервисе 'API Reports' есть недокументированное ограничение на максимальное количество строк которое можно получить в ответе. По умолчанию оно равняется 1 000 000 строк. В большинстве случаев это незначительное ограничение, но при загрузке отчётов из больших аккаунтов с глубокой детализацией, за длительный период скорее всего вы столкнётесь с этой проблемой, и даже её не заметите.

Функция `yadirGetReports` столкнувшись с этим ограничением выведет уведомление `'You have reached the limits of Yandex.Direct API. Try to use "FetchBy" parameter with DateRangeType = "CUSTOM_DATE", "DateFrom" and "DateTo". If you are already using it, try to choose a smaller value.'`.


Т.е. функция столкнувшись с лимитом предложит вам повторно запустить запрос с использованием аргумента *FetchBy*. Данный аргумент позволяет вам разбить запрос на части по временному интервалу. 

Возможные значения: "DAY", "WEEK", "MONTH", "QUARTER", "YEAR"

Загрузка в таком случае потребует большего времени но вернёт полные данные.

Помимо уведомления, таблица которую вы получите в рзельтате работы функции `yadirGetReports` будет иметь атрибут *limit_reached*. В данном атрибуте хранится список логинов тех аккаунтов по котором был достигнул лимит при загрузке данных. Получить этот список можно с помощью `attr(data, "limit_reached")`, в том случае если объект с полученной статистикой имеет имя *data*, в другом случае `attr(имя_полученного_объекта, "limit_reached")`.

Наиболее правильный вариант использования этого функцияала заключается в следующем:

1. Создаём результирующий дата фрейм.
2. Внутри цикла `while` запускает сбор статистики из любого колличества аккаунтов. Первый раз без разбивки запроса на временные интервалы.
3. После загрузки данных проверяется атрибут *limit_reached*, на наличие аккаунтов достигших лимита.
4. Из полученной таблицы со статистикой по всем аккаунтам исключаются данные по аккаунтам достигшим лимита.
5. В результирующий фрем дописываются данные по аккаунтам НЕ достигшим лимита.
6. Включаем следующий уровень временной разбивки, рекомедуемая последовательность: МЕСЯЦ -> НЕДЕЛЯ -> ДЕНЬ.
7. Переходим на следующую итерации, запуская сбор статистики из аккаунтов которые без разбивки запроса достигли лимита.

Данный цикл должен работать до тех пор пока не будет выполненно одно из следующих условий:

1. Мы получим данные в которых ни один аккаунт не достигнет лимита, и атрибут *limit_reached* будет пустым.
2. При запуске сбора данных с подневной разбивкой отчёта всё равно будут аккаунты достигшие лимита.

Ниже приведён пример кода реализующего описанный выше механизм.

```{r}
library(ryandexdirect)

# создаём результирующий фрейм
res   <- data.frame()

# список логинов
log_list <- c("login1", "login2","login3", "login4")

# проверка лиситов
# отмечаем что это первый запуск
check <- "first"

# создаём последовательность уровней временной разбивки запросов
fetching_seq <- c("OFF", "MONTH", "WEEK", "DAY")

# счётчик последовательностей разбивки
fetch_id <- 1

# запускаем цикл загрузки данных с проверкой лимитов
while ( ! is.null( log_list ) ) {
  
  # определяем уровень разбивки запроса
  if ( fetching_seq[fetch_id] == "OFF" ) fetching <- NULL else fetching <- fetching_seq[fetch_id]
  
  # запускаем сбор данных
  data <- yadirGetReport(DateRangeType = "CUSTOM_DATE", 
                         DateFrom      = "2018-06-01", 
                         DateTo        = "2019-05-31", 
                         FieldNames    = c("Date","CampaignName","Impressions","Clicks"),
                         Login         = log_list,
                         FetchBy       = fetching)
  
  # если загрузка была по одному аккаунту добавляем его логин
  if ( length(log_list) == 1 ) {
    data$Login <- log_list
  }
  
  # проверяем список аккаунтов достигших лимита
  log_list <- attr(data, "limit_reached")
  
  # выводим список аккаунтов достигших лимита
  print(log_list)
  
  # если есть аккаунты достигшие лимита
  if ( length(log_list) > 0 ) {
    # очищаем от них общую таблицу
    data <- data[ ! data$Login %in% log_list, ]
    # переключаем уровень разбивки на более мелкий
    fetch_id <- fetch_id + 1
  } 
  
  # дописываем в результирующий фрейм данные
  # по тем аккаунтам которые не упёрлись в лимит
  if ( nrow(data) > 0 ) {
    res <- dplyr::bind_rows(res, data)
  }
  
  # проверяем модно ли разбить запрос на более мелкие части
  if ( fetch_id > length(fetching_seq) && length(log_list) > 0 ) {
    message("Запрос невозможно разбить на меньшие части")
    message("Аккаунты которые достигли лимита при загрузке данных по дням: ", paste(log_list, collapse = ", "))
    limits_login <- log_list
    break
  }
}
```

## Аргументы доступа к API Login, AgencyAccount, Token, TokenPath
Все перечисленные в заголовке аргументы используются для авторизации в API, хранении и использовании учётных данных. Более подробно о них можно узнать из виньетке посвящённой процессу авторизации, с помощью `vignette("yandex-direct-auth", package = "ryandexdirect")`.

---
title: "Загрузка данных о расходах на рекламу из Яндекс Директ в Google Analytics"
author: "Алексей Селезнёв"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ryandexdirect: Загрузка данных о расходах на рекламу из Яндекс Директ в Google Analytics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  eval=FALSE,
  collapse = TRUE,
  comment = "#>"
)
```

Начиная с версии `ryandexdirect 3.6.0` в пакет была добавлена функция `yadirGetCostData()`, которая позволяет вам получить данные о расходах для дальнейшей их загрузки в Google Analytics.

## Видео уроки
Вы можете посмотреть видео урок по данной теме, перейдя по [этой ссылке](https://youtu.be/T9y0Iuh5EWk).

## Создаём набор данных в Google Analytics
Изначально вам необходимо создать набор данных в Google Analytics.

1. Заходим в панель администратора
2. В области ресурс выбираем пункт "Импорт данных"

![Панель администратора](http://img.netpeak.ua/alsey/159056878065_kiss_43kb.png)

3. Нажимае **Создать / Create**
4. Среды предложенных вариантов выбираем **Данные о расходах / Cost Data**

![Выбираем тип набора данных](http://img.netpeak.ua/alsey/159056895184_kiss_45kb.png)

5. Задаём имя набора данных
6. Выбираем из выпадающего списка представления, в которые будут импортироваться данные о расодах

![Выбираем представления](http://img.netpeak.ua/alsey/159171156636_kiss_26kb.png)

7. Настраиваем схему данных и выбираем перезаписывать данные если они соответвуют уже существующим строкам *Owerwrite*

![Настраиваем схему данных](http://img.netpeak.ua/alsey/159056941892_kiss_43kb.png)

8. Жмём сохранить.

Далее в интерфейсе появится созданный набор данных.

![Созданный набор данных](http://img.netpeak.ua/alsey/159171162087_kiss_25kb.png)

В дальнейшем нам понадобится его идентификаторв.

## Запрашиваем данные о расхода из рекламного кабинета Facebook
Далее мы можем запросить данные о расходах из любого аккаунта к которому у нас есть доступ, и с помощью пакета `googleAnalyticsR` загрузить их в созданный источник данных.

```{r}
library(rfacebookstat)
library(googleAnalyticsR)

# fb options
options(ryandexdirect.username = 'your_yandex_username')

# auth
fbAuth()
ga_auth(email = "google_analytics@gmail.com")

# read data from facebook
cost_data <- yadirGetCostData(DateFrom        = '2020-03-01',
                              DateTo          = '2020-03-10',
                              Source          = 'yandex', 
                              Medium          = 'cpc',
                              IncludeVAT      = "YES",
                              IncludeDiscount = "NO")

# upload into GA source
ga_custom_upload_file(accountId          = 2222222, 
                      webPropertyId      = "UA-2222222-1", 
                      customDataSourceId = 'M_XXXXX_Qudk-8g', 
                      cost_data)
```

Если вы всё сделали правильно в разделе "Manage Uploads" вы увидите загруженные данные, и сообщение о статусе загрузке.

![Раздел "Manage Uploads"](http://img.netpeak.ua/alsey/159171178055_kiss_29kb.png)

![Статус загрузки](https://img.netpeak.ua/alsey/159057047222_kiss_16kb.png)

## Где искать данные о расходах на Яндекс Директ в Google Analytics
После загрузки в Google Analytics данные появляются не сразу, как правило должно пройти 1 - 1.5 часа.
Найти данные о расходах можно в отчёте Анализ расходов:

![Данные о расходах в Google Analytics](http://img.netpeak.ua/alsey/159178300398_kiss_26kb.png)

---
title: "Управление ставками ключевых слов с помощью API Яндекс Директ"
author: "Alexey Seleznev"
output: rmarkdown::html_vignette
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{ryandexdirect: Управление ставками ключевых слов с помощью API Яндекс Директ}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  collapse = TRUE,
  comment = "#>"
)
```
Значительная часть представленной в этой виньетке информации взаимствована из [официальной документации](https://yandex.ru/dev/direct/doc/ref-v5/bids/bids-docpage) к API ЯндексюДирект версии 5.

Для управления ставками в пакете `ryandexdirect` существуют 3 функции:

* `yadirGetKeyWordsBids` - Возвращает ставки и приоритеты для ключевых фраз и автотаргетингов, отвечающих заданным критериям, а также данные торгов: ставки и списываемые цены для различных объемов трафика на поиске и ставки для охвата различных долей аудитории в сетях.
* `yadirSetKeyWordsBids` - Назначает фиксированные ставки и приоритеты для ключевых фраз и автотаргетингов.
* `yadirSetAutoKeyWordsBids` - Назначает для фраз ставки на поиске в зависимости от желаемого объема трафика или ставки в сетях в зависимости от желаемой частоты показа (доли аудитории).

## Загрузка ставок и данных торгов, функция yadirGetKeyWordBids

### Аргументы

* KeywordIds - Идентификатор ключевой фразы или автотаргетинга;
* AdGroupIds - Идентификатор группы объявлений, к которой относится ключевая фраза или автотаргетинг;
* CampaignIds - Идентификатор кампании, к которой относится ключевая фраза или автотаргетинг;
* AuctionBids - Тип аукционов данные о торгах на которым вы хотите получить, возможные значения:
    * `NA` - Не загружать данные торгов;
    * `search` - Получить данные ставок и списываемых цен на поиске, соответствующих различным объемам трафика;
    * `network` - Получить массив ставок, соответствующих охвату различных долей аудитории.

### Описание данных возвращаемых функцией yadirGetKeyWordBids

* CampaignId - Идентификатор кампании, к которой относится ключевая фраза или автотаргетинг;
* AdGroupId - Идентификатор группы объявлений, к которой относится ключевая фраза или автотаргетинг;
* KeywordId - Идентификатор ключевой фразы или автотаргетинга;
* ServingStatus - Статус возможности показов группы объявлений:
    * ELIGIBLE - Показы возможны;
    * RARELY_SERVED - "Мало показов" — при текущих настройках показы происходят крайне редко. Например, ключевые фразы не встречаются в поисковых запросах пользователей.
* StrategyPriority - Приоритет ключевой фразы или автотаргетинга: LOW, NORMAL или HIGH.
* SearchBib - Ставка на поиске, назначенная рекламодателем;
* NetworkBid - Ставка в сетях, назначенная рекламодателем.

#### Дополнительные поля при запросе данных торгов на поиске `AuctionBids = "search"`

* AuctionTrafficVolume - Объем трафика;
* AuctionBid - Ставка на поиске, соответствующая указанному объему трафика;
* AuctionPrice - Списываемая цена, соответствующая указанному объему трафика.

#### Дополнительные поля при запросе данных торгов на поиске `AuctionBids = "network"`

* Probability - Частота показа (доля аудитории) в сетях. Указывается в процентах от 0 до 100;
* AuctionBid - Ставка в сетях, соответствующая указанной частоте показа.

### Пример кода для запроса ставок
```{r}
# подключаем пакет
library(ryandexdirect)

# запрашиваем список ключевых слов
kw <- yadirGetKeyWords()
                       
# запрашиваем ставки
bids  <- yadirGetKeyWordsBids(kw$Id)

# запрашиваем ставки и данные торго на поиске
bids_search <- yadirGetKeyWordsBids(kw$Id,
                                    AuctionBids = "search")
                                    
# запрашиваем ставки и данные торго в сетях
network_bids <- yadirGetKeyWordsBids(kw$Id,
                                     AuctionBids = "network")
```


## Назначение фиксированных ставок и приоритетов для ключевых фраз и автотаргетингов, функция yadirSetKeyWordsBids

### Аргументы

* KeywordIds - Идентификатор ключевой фразы или автотаргетинга;
* AdGroupIds - Идентификатор группы объявлений, к которой относится ключевая фраза или автотаргетинг;
* CampaignIds - Идентификатор кампании, к которой относится ключевая фраза или автотаргетинг;
* StrategyPriority - Приоритет фразы: LOW, NORMAL или HIGH. Только для автоматической стратегии.
* SearchBid - Ставка на поиске. Только для ручной стратегии. Указывается в валюте рекламодателя.
* NteworkBid - Ставка в сетях. Только для ручной стратегии с независимым управлением ставками в сетях. Указывается в валюте рекламодателя. 

Обязательно указать хотя бы один из аргументов *SearchBid*, *NetworkBid*, *StrategyPriority*.

Ставку или приоритет можно назначить:

* для отдельной ключевой фразы или автотаргетинга с помощью аргумента *KeywordIds*;

* для всех ключевых фраз и автотаргетинга в группе объявлений с помощью аргумента *AdGroupIds*;

* для всех ключевых фраз и автотаргетингов в кампании с помощью аргумента *CampaignIds*.

Ставку или приоритет можно назначить в зависимости от того, какая стратегия показа выбрана в кампании:

Если выбрана стратегия показа на поиске HIGHEST_POSITION, то можно указать аргумент *SearchBid*.

Если выбрана стратегия показа в сетях MAXIMUM_COVERAGE или MANUAL_CPM, то можно указать аргумент *NetworkBid*.

Если выбрана автоматическая стратегия, то можно указать параметр *StrategyPriority*: фразы с более высоким приоритетом по возможности получают больший объем трафика, а при недостатке бюджета отключаются в последнюю очередь.

В случае если элемент входного массива содержит параметры, не соответствующие стратегии, то значения этих параметров изменены не будут.

Если элемент входного массива содержит одновременно и параметры, соответствующие стратегии, и параметры, не соответствующие стратегии (например, и ставку, и приоритет), то в результате операции будут изменены значения только параметров, соответствующих стратегии, и выдано предупреждение.

Если элемент входного массива содержит только параметры, не соответствующие стратегии (например, только приоритет при ручной стратегии), то операция не будет выполнена и будет возвращена ошибка.

### Возвращаемый результат

В результате работы функции будет возвращён объект класса `list`. Данный объект будет состоять из ответов полученных на каждый запрос, название узла списка соттветвует ID вашего запроса.

Внутри лист с ответом на каждый запрос может содержать следующие данные:

* SetResults - Результаты назначения ставок и/или приоритетов
    * Warnings - Предупреждения, возникшие при выполнении операции
    * Errors - Ошибки, возникшие при выполнении операции.
    * CampaignId - Идентификатор кампании. Возвращается в случае отсутствия ошибок.
    * AdGroupId - Идентификатор группы объявлений. Возвращается в случае отсутствия ошибок.
    * KeywordId - Идентификатор ключевой фразы или автотаргетинга. Возвращается в случае отсутствия ошибок.

Соответвенно, если в ответе вы получили список в котором содержится перечень KeywordId, AdGroupId и CampaignId, это говорит о том, что вы успешно назначили ставки.

Пример ответа в случае успешной устновки ставки:
```{r}
## $`2874076041280427473`
## $`2874076041280427473`[[1]]
## $`2874076041280427473`[[1]]$KeywordId
## [1] 6402098519
##
##
## $`2874076041280427473`[[2]]
## $`2874076041280427473`[[2]]$KeywordId
## [1] 6402098520
```

Пример ответа в случае ошибок и предупреждений при устновке ставки:
```{r}
## $`2874442110113212447`
## $`2874442110113212447`[[1]]
## $`2874442110113212447`[[1]]$Warnings
## $`2874442110113212447`[[1]]$Warnings[[1]]
## $`2874442110113212447`[[1]]$Warnings[[1]]$Code
## [1] 10160
## 
## $`2874442110113212447`[[1]]$Warnings[[1]]$Message
## [1] "Ставка не будет применена"
## 
## $`2874442110113212447`[[1]]$Warnings[[1]]$Details
## [1] "Ставки на сети не будут изменены, так как отключены показы на сети"
## 
## 
## 
## $`2874442110113212447`[[1]]$Errors
## $`2874442110113212447`[[1]]$Errors[[1]]
## $`2874442110113212447`[[1]]$Errors[[1]]$Code
## [1] 9600
## 
## $`2874442110113212447`[[1]]$Errors[[1]]$Message
## [1] "Поле не соответствует установленной стратегии"
## 
## $`2874442110113212447`[[1]]$Errors[[1]]$Details
## [1] "Для стратегии с ручным управлением ставками на поиске поле SearchBid должно быть указано"
## 
## 
## 
## 
## $`2874442110113212447`[[2]]
## $`2874442110113212447`[[2]]$Warnings
## $`2874442110113212447`[[2]]$Warnings[[1]]
## $`2874442110113212447`[[2]]$Warnings[[1]]$Code
## [1] 10160
## 
## $`2874442110113212447`[[2]]$Warnings[[1]]$Message
## [1] "Ставка не будет применена"
## 
## $`2874442110113212447`[[2]]$Warnings[[1]]$Details
## [1] "Ставки на сети не будут изменены, так как отключены показы на сети"
## 
## 
## 
## $`2874442110113212447`[[2]]$Errors
## $`2874442110113212447`[[2]]$Errors[[1]]
## $`2874442110113212447`[[2]]$Errors[[1]]$Code
## [1] 9600
## 
## $`2874442110113212447`[[2]]$Errors[[1]]$Message
## [1] "Поле не соответствует установленной стратегии"
## 
## $`2874442110113212447`[[2]]$Errors[[1]]$Details
## [1] "Для стратегии с ручным управлением ставками на поиске поле SearchBid должно быть указано"
```

К тому же при возникновении ошибок и предупреждений вы получите следующие сообщения:
```{r}
!..Error: Для стратегии с ручным управлением ставками на поиске поле SearchBid должно быть указано
!..Error: Для стратегии с ручным управлением ставками на поиске поле SearchBid должно быть указано
...Warning: Ставки на сети не будут изменены, так как отключены показы на сети
...Warning: Ставки на сети не будут изменены, так как отключены показы на сети
```

### Пример кода установки ставок

Если вы хотите установить ставку в поиске на уровне 9 рублей на все ключевые слова в одной рекламной кампании можете воспользоваться следующим кодом:

```{r}
# подключаем пакет
library(ryandexdirect)

# запрашиваем список рекламных кампаний
camp <- yadirGetCampaign()

# меняем ставки
setbid     <- yadirSetKeyWordsBids(CampaignIds = camp$Id[1],
                                   SearchBid   = 9)
```

## Назначить для фраз ставки на поиске в зависимости от желаемого объема трафика или ставки в сетях в зависимости от желаемой частоты показа (доли аудитории)., функция yadirSetKeyWordsBids

### Аргументы

* KeywordIds - Идентификатор ключевой фразы или автотаргетинга;
* AdGroupIds - Идентификатор группы объявлений, к которой относится ключевая фраза или автотаргетинг;
* CampaignIds - Идентификатор кампании, к которой относится ключевая фраза или автотаргетинг;
* TargetTrafficVolume - Желаемый объем трафика на поиске. Указывается в процентах от 5 до 100. К ставке, соответствующей выбранному объему трафика, прибавляется надбавка (см. аргумент *SearchIncreasePercent*).
* SearchIncreasePercent - Процент надбавки от 0 до 1000. Если не задан, надбавка не рассчитывается.
* SearchBidCeiling - Ограничение на ставку. Целое число. Указывается в валюте рекламодателя.
* TargetCoverage - Желаемая частота показа (доля аудитории) в сетях. Указывается в процентах от 1 до 100. К ставке, соответствующей выбранной частоте показа, прибавляется надбавка (см. параметр *NetworkIncreasePercent*).
* NetworkIncreasePercent - Процент надбавки от 0 до 1000. Если не задан, надбавка не рассчитывается.
* NetworkBidCeiling - Ограничение на ставку, умноженное на 1 000 000. Целое число. Указывается в валюте рекламодателя. 

#### Наборы аргументов

Все аргументы функции `yadirSetKeyWordsBids` можно объединить в блоки. 

* Вы **обязательно** должны указать только один из аргументов: *KeywordIds*, *AdGroupIds*, *CampaignIds*.
* Аргументы содержащие параметры для формулы расчета ставок деляться на два блока. В одном запросе вы должны использовать только один из блоков.
    * Блок аргументов установки параметров для формулы расчета ставок на поиске:
        * TargetTrafficVolume (**обязательный**)
        * SearchIncreasePercent
        * SearchBidCeiling
    * Блок аргументов установки параметров для формулы расчета ставок  в сетях:
        * TargetCoverage (**обязательный**)
        * NetworkIncreasePercent
        * NetworkBidCeiling
        
### Формула расчета ставки

**Расчёт ставки на поиске:**

Ставка, соответствующая объему трафика `TargetTrafficVolume × (1 + SearchIncreasePercent / 100)`, но не более `SearchBidCeiling`.

Со временем активность конкурентов может поднять ставку за желаемый объем трафика, и она превысит ставку рекламодателя. Чем выше надбавка, тем больше объем трафика, но и выше возможные расходы.

**Расчёт ставки в сетях:**

Ставка, соответствующая частоте показа `TargetCoverage × (1 + NetworkIncreasePercent / 100)`, но не более `NetworkBidCeiling`.

Со временем активность конкурентов может поднять ставку за желаемую частоту показа, и она превысит ставку рекламодателя. Чем выше надбавка, тем выше частота показа, но и выше возможные расходы.

#### Ограничения

Ставку на поиске можно назначить, только если в кампании выбрана стратегия показа на поиске HIGHEST_POSITION. В противном случае возвращается ошибка.

Ставку в сетях можно назначить, только если в кампании выбрана стратегия показа в сетях MAXIMUM_COVERAGE или MANUAL_CPM. В противном случае возвращается ошибка.

В одном запросе можно назначить ставки только для однородных объектов — либо только для кампаний, либо только для групп, либо только для фраз.

Функция не поддерживает назначение ставки отдельному автотаргетингу. Обновление ставок на поиске для всех фраз группы объявлений или кампании может повлиять на ставку для автотаргетинга.

### Пример кода автоматической установки ставок

Если вы хотите поднять ставку по всем ключевым словам в аккаунте до уровня, при котором ваши объявления гарантированно будут показаны в 50% соответвующих поисковых запросах воспользуйтесь кодом.

```{r}
# подключаем пакет
library(ryandexdirect)

# запрашиваем список ключевых слов
kw <- yadirGetKeyWords()

# меняем ставки
autosetbids <- yadirSetAutoKeyWordsBids(KeywordIds = kw$Id,
                                        TargetTrafficVolume = 50)
```

## Работа с пайпланами %>%


Все функции, предназначенные для работы со ставками (`yadirGetKeyWordsBids`, `yadirSetKeyWordsBids`, `yadirSetAutoKeyWordsBids`) поддерживают работу с пайплайнами `%>%` из пакета `magrittr`. Данный функционал широко известен всем, кто привык использовать пакеты входящие в состав библиотеки `tidyverse`.

### Пример кода загрузки ставок с использованием пайплана.
```{r}
# подключаем пакет
library(ryandexdirect)
library(magrittr)

# загрузкса скписка ключевых слов и ставок по ним с помощью пайплайна
kw_bids <- yadirGetKeyWords() %>%
           yadirGetyadirGetKeyWordsBids()

# загрузкса скписка групп объявлений с типом "TEXT_AD_GROUP" и ставок по ключевым словам входящим в эти группы
autosetbids <-  yadirGetAdGroups(Types = "TEXT_AD_GROUP")  %>%
                yadirGetyadirGetKeyWordsBids(AdGroupIds = .)
```

## Ссылки

* [Справка Яндекс.Директ по сервису KeywordBids.](https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/keywordbids-docpage)
* [Справка Яндекс.Директ по методу get сервиса KeywordBids.](https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/get-docpage)
* [Справка Яндекс.Директ по методу set сервиса KeywordBids.](https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/set-docpage)
* [Справка Яндекс.Директ по методу setAuto сервиса KeywordBids.](https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/setAuto-docpage)
* [Руководство разработчика Яндекс.Директ по ставкам и приоритетам.](https://yandex.ru/dev/direct/doc/dg/objects/bid-priority-docpage)

\name{yadirGetDictionary}
\alias{yadirGetDictionary}
\title{Get Dictionary from 'Yandex Direct'}
\description{Returns reference data: regions, time zones, exchange rates, list of metro stations, restrictions on parameter values, external networks (SSP), Crypt segments for targeting by user profile, etc.}
\usage{
yadirGetDictionary(DictionaryName = "GeoRegions", 
                   Language      = "ru", 
                   Login         = getOption("ryandexdirect.user"),
                   Token         = NULL,
                   AgencyAccount = getOption("ryandexdirect.agency_account"),
                   TokenPath     = yadirTokenPath())
}

\arguments{
  \item{DictionaryName}{character string with Dictionary name, one of Currencies, MetroStations, GeoRegions, TimeZones, Constants, AdCategories, OperationSystemVersions, ProductivityAssertions, SupplySidePlatforms, Interests}
  \item{Language}{Answer message lenguage, by default "ru", support one of en, ru, tr, uk}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
}
\references{
For more details go \href{https://yandex.ru/dev/direct/doc/ref-v5/dictionaries/get-docpage}{link}
}
\value{Data Frame}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
geo <- yadirGetDictionary(Login = "login", 
                          DictionaryName = "GeoRegions",
                          TokenPath = tempdir())
}
}

\name{yadirGetCostData}
\alias{yadirGetCostData}
\title{Loading Cost Data for Upload to Google Analytics}
\description{Get data on expenses, impressions and clicks on advertising campaigns, keywords and ads for further loading into 'Google Analytics' source.}
\usage{
yadirGetCostData(
    DateFrom = Sys.Date() - 31,
    DateTo = Sys.Date() - 1,
    Source = 'yandex',
    Medium = 'cpc',
    IncludeVAT        = "YES",
    IncludeDiscount   = "NO",
    Login             = getOption("ryandexdirect.user"),
    AgencyAccount     = getOption("ryandexdirect.agency_account"),
    FetchBy           = NULL,
    Token             = NULL,
    TokenPath         = yadirTokenPath())
}

\arguments{
  \item{DateFrom}{If your select CUSTOM_DATE in DateRangeType you must enter start date in this argument in YYYY-MM-DD format else you can not used this argument}
  \item{DateTo}{If your select CUSTOM_DATE in DateRangeType you must enter end date in this argument in YYYY-MM-DD format else you can not used this argument}
  \item{Source}{Character, UTM source}
  \item{Medium}{Character, UTM medium}
  \item{IncludeVAT}{Include or exclude VAT, one of "NO" or "YES"}
  \item{IncludeDiscount}{Include or exclude Discount, one of "NO" or "YES"}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{FetchBy}{Split the query by date range. Allowed values: "DAY", "WEEK", "MONTH", "QUARTER", "YEAR".}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
}

\value{Data frame with cost data}

\author{Alexey Seleznev}

\examples{
\dontrun{
### 
# Use googleAnalyticsR for upload data into Google Analytics
library(googleAnalyticsR)

# load cost data
cost_data <- yadirGetCostData(DateFrom = "2020-05-10", 
                              DateTo = '2020-05-14')
                              
# upload into GA source
ga_custom_upload_file(accountId          = xxxx, 
                      webPropertyId      = "UA-xxxx-1", 
                      customDataSourceId = 'abcdefg', 
                      cost_data)

}
}

\name{yadirGetAds}
\alias{yadirGetAds}
\title{Get Ads List From 'Yandex Direct' account}
\description{Returns the parameters of ads that match the specified criteria.}
\usage{
yadirGetAds(CampaignIds   = NULL, 
            AdGroupIds    = NA, 
            Ids           = NA, 
            Types         = c("TEXT_AD", "MOBILE_APP_AD", "DYNAMIC_TEXT_AD", 
                              "IMAGE_AD", "CPC_VIDEO_AD", "CPM_BANNER_AD", 
                              "CPM_VIDEO_AD", "SMART_AD"),
            States        = c("OFF","ON","SUSPENDED","OFF_BY_MONITORING","ARCHIVED"), 
            Login         = getOption("ryandexdirect.user"),
            Token         = NULL,
            AgencyAccount = getOption("ryandexdirect.agency_account"),
            TokenPath     = yadirTokenPath())
}

\arguments{
  \item{CampaignIds}{character vector with yandex direct campaign ID, require}
  \item{AdGroupIds}{character vector with yandex direct adgroup ID}
  \item{Ids}{character vector with yandex direct ad ID}
  \item{Types}{character vector, types of ads: TEXT_AD, MOBILE_APP_AD, DYNAMIC_TEXT_AD, IMAGE_AD, CPC_VIDEO_AD, CPM_BANNER_AD, CPM_VIDEO_AD, SMART_AD}
  \item{States}{character vector with ad states}
  \item{Login}{character, your login at Yandex, require}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{Data Frame}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
#For get ads data frame
# Get ads from client account
aut <- yadirAuth(Login = "login", NewUser = TRUE,
                 TokenPath = tempdir())
my_ads  <- yadirGetAds(Login = "login",
                       Token = aut)
					   
# Get data from agency account
aut <- yadirAuth(Login = "agency_login", NewUser = TRUE,
                 TokenPath = tempdir())

my_ads  <- yadirGetAds(Login = "client_login",
					   AgencyAccount = "agency_login",
                       Token = aut)
}
}

\name{yadirGetForecast}
\alias{yadirGetForecast}
\title{Get Forecast}
\description{Returns a forecast for cost, click and impression}
\usage{
yadirGetForecast(Phrases,
                 GeoID         = 0,
                 Currency      = "RUB",
                 AuctionBids   = "No",
                 Login         = getOption("ryandexdirect.user"),
                 Token         = NULL,
                 AgencyAccount = getOption("ryandexdirect.agency_account"),
                 TokenPath     = yadirTokenPath())
                        }

\arguments{
  \item{Phrases}{character vector of key phrases for which you want to get a forecast, require}
  \item{GeoID}{character vectora, an array of key phrases for which you want to obtain statistics on search queries}
  \item{Currency}{The currency in which click prices and total costs in the report should be expressed. Possible values: RUB, CHF, EUR, KZT, TRY, UAH, USD, BYN.}
  \item{AuctionBids}{Whether to display trading results in the report (Yes / No)}
  \item{Login}{integer, An array of region identifiers. Allows you to get statistics on search queries made only in the specified regions, see yadirGetDictionary}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{Data Frame}
\author{Alexey Seleznev}

\examples{
\dontrun{
# get forecast
forecast <- yadirGetForecast(c('buy smartphone -xiaomi', 
                               'buy samsung'),
                             Login = "selesnow")
}
}

\name{yadirStartKeyWords}
\alias{yadirStartKeyWords}
\title{Resume KeyWords}
\description{Resumes impressions on previously stopped keyword phrases and auto-targeting.}
\usage{
yadirStartKeyWords(Login = getOption("ryandexdirect.user"), Ids = NULL, 
        Token = NULL, 
        AgencyAccount = getOption("ryandexdirect.agency_account"), 
        TokenPath = yadirTokenPath())
}

\arguments{
  \item{Ids}{Integer or Character vector with KeyWords Ids, who will resume impressions}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Function resume impressions and return vector with KeyWords Ids with error.
}
\value{Vector with keywords ids which failed to resume impressions}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# Get API token
tok <- yadirGetToken(TokenPath = tempdir())

# Get camp dictionary
my_camp <- yadirGetCampaignList(Login = "Login", 
                                Token = tok,
								TokenPath = tempdir())

# Get keywords dictionary
my_kw <- yadirGetKeyWords(Login = "Login", Token = tok, 
                          CampaignIds = my_camp$Id[1:10],
						  TokenPath = tempdir())

# Resume impressions
err <- yadirStartKeyWords(Login = "LOGIN", Token =  tok, 
                          Ids = my_kw$Id,
						  TokenPath = tempdir()) 
}
}

\name{yadirGetClientParam}
\alias{yadirGetClientParam}
\title{Get params list from 'Yandex Direct' agency clients}
\description{Returns the parameters of the advertiser and the settings of the user - the representative of the advertiser or the parameters of the agency and the settings of the user - the representative of the agency.}
\usage{
yadirGetClientParam(Language = "ru", 
                    Logins = getOption("ryandexdirect.user"), 
                    Token = NULL,
                    AgencyAccount = getOption("ryandexdirect.agency_account"),
                    TokenPath     = yadirTokenPath())
}

\arguments{
  \item{Logins}{Character vector of yandex logins}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
  \item{Language}{Answer language one of en, ru, tr, uk}
}

\value{data frame with campaings names and parameters}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
#Get client param from agency account
client_param <- yadirGetClientParam(AgencyAccount = "agency_login", 
                                    TokenPath     = tempdir())
}
}

\name{ryandexdirect-package}
\alias{ryandexdirect-package}
\alias{ryandexdirect}
\docType{package}
\title{
\packageTitle{ryandexdirect}
}
\description{
\packageDescription{ryandexdirect}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{ryandexdirect}
\packageIndices{ryandexdirect}
}
\author{
\packageAuthor{ryandexdirect}

Maintainer: \packageMaintainer{ryandexdirect}
}

\keyword{ package }

\name{yadirGetCampaign}
\alias{yadirGetCampaignList}
\alias{yadirGetCampaign}
\title{Get Campaigns List}
\description{Returns the parameters of campaigns that meet the specified criteria.}
\usage{
yadirGetCampaignList(Logins = getOption("ryandexdirect.user"), 
                   States = c("OFF", "ON", "SUSPENDED",
                   "ENDED", "CONVERTED", "ARCHIVED"), Types =
                   c("TEXT_CAMPAIGN", "MOBILE_APP_CAMPAIGN",
                   "DYNAMIC_TEXT_CAMPAIGN", "CPM_BANNER_CAMPAIGN"),
                   Statuses = c("ACCEPTED", "DRAFT", "MODERATION",
                   "REJECTED"), StatusesPayment = c("DISALLOWED",
                   "ALLOWED"), Token = NULL, 
                   AgencyAccount = getOption("ryandexdirect.agency_account"),
                   TokenPath = yadirTokenPath())
				   
yadirGetCampaign(Logins = getOption("ryandexdirect.user"), 
                  States = c("OFF", "ON", "SUSPENDED",
                   "ENDED", "CONVERTED", "ARCHIVED"), Types =
                   c("TEXT_CAMPAIGN", "MOBILE_APP_CAMPAIGN", "DYNAMIC_TEXT_CAMPAIGN", 
                   "CPM_BANNER_CAMPAIGN", "SMART_CAMPAIGN", "UNIFIED_CAMPAIGN"),
                   Statuses = c("ACCEPTED", "DRAFT", "MODERATION",
                   "REJECTED"), StatusesPayment = c("DISALLOWED",
                   "ALLOWED"), Token = NULL, 
                   AgencyAccount = getOption("ryandexdirect.agency_account"),
                   TokenPath = yadirTokenPath())
}

\arguments{
  \item{Logins}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
  \item{States}{character vector, filter by campaign states, for example c("OFF", "ON", "SUSPENDED", "ENDED", "CONVERTED", "ARCHIVED")}
  \item{Types}{character vector with campaign types, example c("TEXT_CAMPAIGN", "MOBILE_APP_CAMPAIGN", "DYNAMIC_TEXT_CAMPAIGN")}
  \item{Statuses}{character vector, filter campaign list by status, for example c("ACCEPTED", "DRAFT", "MODERATION", "REJECTED")}
  \item{StatusesPayment}{character vector, filter campaign list by payment status, for example c("DISALLOWED", "ALLOWED")}
}

\value{data frame with campaings names and parameters}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
#Get data from client accounts
my_ad_group <- yadirGetCampaign(Login = "login",
                                TokenPath = tempdir())

#Get data from agency account
# Auth
aut <- yadirAuth(Login = "agency_login", 
                 NewUser = TRUE, 
				 TokenPath = tempdir())

# Load Ad Group List
my_ad_group <- yadirGetCampaign(Login = "client_login",
                                Token = aut,
								TokenPath     = tempdir())
}
}

\name{yadirGetWordStatReport}
\alias{yadirGetWordStatReport}
\title{Get WordStat Report}
\description{Returns a search query statistics report from 'Wordstat'.}
\usage{
yadirGetWordStatReport( 
                 Phrases,
                 GeoID         = 0,
                 Login         = getOption("ryandexdirect.user"),
                 Token         = NULL,
                 AgencyAccount = getOption("ryandexdirect.agency_account"),
                 TokenPath     = yadirTokenPath())
}

\arguments{
  \item{Phrases}{character vector with yandex direct campaign ID, require}
  \item{GeoID}{character vectora, an array of key phrases for which you want to obtain statistics on search queries}
  \item{Login}{integer, An array of region identifiers. Allows you to get statistics on search queries made only in the specified regions, see yadirGetDictionary}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{Data Frame}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# get wordstat report
wsrep <- yadirGetWordStatReport(
            Phrases = 'samsung smartphone',
            Login = "abekker.ru.2017")

wsrep[['SearchedAlso']]
wsrep[['SearchedWith']]
}
}

\name{yadirGetSiteLinks}
\alias{yadirGetSiteLinks}
\title{Get Fast Links}
\description{Returns sets of quick links that meet the specified criteria.}
\usage{
yadirGetSiteLinks(
     Login = getOption("ryandexdirect.user"),
     Token = NULL,
     Ids   = NULL,
     AgencyAccount = getOption("ryandexdirect.agency_account"),
     TokenPath     = yadirTokenPath())
}

\arguments{
  \item{Login}{character, your logins at Yandex Direct, require}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{Ids}{integer vector, ids of sitelinks sets}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{data frame with SiteLinks parameters, id, title, href and description }
\author{Alexey Seleznev}

\examples{
\dontrun{
#  Recomendation, please choose another TokenPath to save the Login permanently.
sl <- yadirGetSiteLinks(Login         = "your.yandex.login",
						Ids           = c(5276537, 
						                  8927897239, 
						         	      252627, 
								          33123),
                        TokenPath     = tempdir())
 }
}

\name{yadirGetClientList}
\alias{yadirGetClientList}
\title{Get Client List From Agency Account}
\description{Returns a list of advertisers - agency clients, their parameters and settings of the main representatives of the advertiser.}
\usage{
yadirGetClientList(
         AgencyAccount = getOption("ryandexdirect.agency_account"),
         Token         = NULL,
         TokenPath     = yadirTokenPath())
}

\arguments{
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
}

\value{data frame with client list}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
my_client <- yadirGetClientList(AgencyAccount = "agency_login", 
                                TokenPath = tempdir())
}
}

\name{yadirGetKeyWords}
\alias{yadirGetKeyWords}
\title{Get Keywords}
\description{Returns the parameters of key phrases or autotargets that match the specified criteria: the values of the placeholder variables, status and status, statistics of impressions and clicks, rates and priorities.}
\usage{
yadirGetKeyWords(CampaignIds = NULL, AdGroupIds = NA, Ids = NA, States
                 = c("OFF", "ON", "SUSPENDED"), WithStats = TRUE, 
                 Login = getOption("ryandexdirect.user"), Token = NULL, 
                 AgencyAccount = getOption("ryandexdirect.agency_account"), 
                 TokenPath = yadirTokenPath())
}

\arguments{
  \item{CampaignIds}{character vector with yandex direct campaign ID, require}
  \item{AdGroupIds}{character vector with yandex direct adgroup ID}
  \item{Ids}{character vector with yandex direct ad ID}
  \item{States}{character vector with ad states}
  \item{WithStats}{logical, if TRUE then load stat by impressions and click but load process will be long durations}
  \item{Login}{character, your login at Yandex, require}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{data frame with campaings names and parameters}
\author{Alexey Seleznev}

\examples{
\dontrun{

# get data from agency account
### Please choose another TokenPath to save the Login permanently.
my_keywords <- yadirGetKeyWords(Login = "client_login",
                                AgencyAccount = "agency_login",
                                TokenPath     = tempdir())
}
}

\name{yadirStopCampaigns}
\alias{yadirStopCampaigns}
\title{Suspend impressions by Campaigns}
\description{Suspend impressions by campaings in 'Yandex Direct' account}
\usage{
yadirStopCampaigns(
 Login = getOption("ryandexdirect.user"), Ids = NULL, 
 Token = NULL, 
 AgencyAccount = getOption("ryandexdirect.agency_account"),
 TokenPath = yadirTokenPath())
}

\arguments{
  \item{Ids}{Integer or Character vector with Campaigns Ids, who will suspend impressions}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Function suspend impressions and return vector with Campaigns Ids with error.
}
\value{Vector with campaigns ids which failed to suspend impressions}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# Get API token
tok <- yadirGetToken(TokenPath = tempdir())

# Get camp dictionary
my_camp <- yadirGetCampaignList(Login = "Login", Token = tok)

# Resume impressions
err <- yadirStopCampaigns(Login = "LOGIN", Token =  tok, Ids = my_camp$Id) 
}
}

\name{yadirStopAds}
\alias{yadirStopAds}
\title{Suspend impressions by Ads}
\description{Stops Ad Impressions in 'Yandex Direct' accounts.}
\usage{
yadirStopAds(Login = getOption("ryandexdirect.user"), Ids = NULL,
      Token = NULL, AgencyAccount = getOption("ryandexdirect.agency_account"), 
      TokenPath = yadirTokenPath())
}

\arguments{
  \item{Ids}{Integer or Character vector with Ads Ids, who will suspend impressions}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Function resume impressions and return vector with Ads Ids with error.
}
\value{Vector with ads ids which failed to suspend impressions}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# Get API token
tok <- yadirGetToken(Login = "LOGIN",
                     TokenPath = tempdir())

# Get Campaigns
my_camp <- yadirGetCampaignList(Login = "LOGIN", 
                                Token = tok)

# Get ads dictionary
my_ads <- yadirGetAds(Login = "LOGIN", 
                      Token = tok, 
					  States = "ON", 
					  CampaignIds = my_camp$Id[1:10])

# Resume impressions
err <- yadirStopAds(Login = "LOGIN", Token =  tok, Ids = my_ads$Id) 
}
}

\name{yadirGetLogins}
\alias{yadirGetLogins}
\title{Get Login List or Set Login}
\description{Get and set List of accessed yandex direct login from token path.}
\usage{
yadirGetLogins(TokenPath = yadirTokenPath(), SetLogin = TRUE)
}

\arguments{
  \item{TokenPath}{Path to directory where you save credential data}
  \item{SetLogin}{Boolean, if TRUE you will choose default login}
}
\value{vector with logins}
\author{Alexey Seleznev}

\examples{
\dontrun{
logins <- yadirGetLogins()
}
}

\name{ryandexdirect-deprecated}
\alias{ryandexdirect-deprecated}
\title{Deprecated functions in ryandexdirect}
\description{List of depricated functions and their new versions.}
\section{Deprecated fnctions}{
Next function is deprecated:
\describe{
  \item{\code{yadirGetCampaignList()}}{use \code{\link[=yadirGetCampaign]{yadirGetCampaign()}}, from september 2019}
  \item{yadirGetCampaignListOld()}{use \code{\link[=yadirGetCampaign]{yadirGetCampaign()}}, from april 2019}
  \item{yadirGetLogsData()}{use \code{\link[rym:rym_get_logs]{rym_get_logs()}}, from april 2019}
  \item{yadirGetMetrikaGAData()}{use \code{\link[rym:rym_get_ga]{rym_get_ga()}}, from april 2019}
  \item{yadirGetSummaryStat()}{use \code{\link[=yadirGetReport]{yadirGetReport()}}, from march 2017}
}

}

\name{yadirSetAgencyAccount}
\alias{yadirSetAgencyAccount}
\title{Set Agency Account}
\description{Set yandex direct Agency Account in current R session.}
\usage{
yadirSetAgencyAccount(AgencyAccount, TokenPath = yadirTokenPath())
}

\arguments{
  \item{TokenPath}{Path to directory where you save credential data}
  \item{AgencyAccount}{Character, username of agency account in Yandex}
}
\value{TRUE}
\author{Alexey Seleznev}

\examples{
\dontrun{
yadirSetAgencyAccount('my_agency')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.yadir_token}
\alias{print.yadir_token}
\title{Print yadir_token}
\usage{
\method{print}{yadir_token}(x, ...)
}
\arguments{
\item{x}{yadir_token object}

\item{...}{extra arguments}
}
\description{
S3 method for printing yadir_token objects
}

\name{yadirStopKeyWords}
\alias{yadirStopKeyWords}
\title{Suspend impressions by KeyWords}
\description{Stops impressions by keyword phrases and autotargeting.}
\usage{
yadirStopKeyWords(Login = getOption("ryandexdirect.user"), Ids = NULL, 
                 Token = NULL, AgencyAccount
                 = getOption("ryandexdirect.agency_account"), 
                 TokenPath = yadirTokenPath())
}

\arguments{
  \item{Ids}{Integer or Character vector with KeyWords Ids, who will suspend impressions}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Function suspend impressions and return vector with KeyWords Ids with error.
}
\value{Vector with keywords ids which failed to suspend impressions}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# Get API token
tok <- yadirGetToken(TokenPath = tempdir())

# Get camp dictionary
my_camp <- yadirGetCampaignList(Login = "Login", Token = tok)

# Get keywords dictionary
my_kw <- yadirGetKeyWords(Login = "Login", Token = tok, CampaignIds = my_camp$Id[1:10])

# Resume impressions
err <- yadirStopKeyWords(Login = "LOGIN", Token =  tok, Ids = my_kw$Id) 
}
}

\name{yadirStartAds}
\alias{yadirStartAds}
\title{Start Ads}
\description{Resumes ad impressions previously stopped by the owner.}
\usage{
yadirStartAds(Login = getOption("ryandexdirect.user"), Ids = NULL,
           Token = NULL, 
           AgencyAccount = getOption("ryandexdirect.agency_account"), 
           TokenPath = yadirTokenPath())
}

\arguments{
  \item{Ids}{Integer or Character vector with Ads Ids, who will resume impressions}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Function resume impressions and return vector with Ads Ids with error.
}
\value{Vector with ads ids which failed to resume impressions}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# Get API token
tok <- yadirGetToken(TokenPath = tempdir())

# Get ads dictionary
my_ads <- yadirGetAds(Login = "LOGIN", 
                      Token = tok,
                      States = c("SUSPENDED","OFF"))

# Resume impressions
err <- yadirStartAds(Login = "LOGIN", 
                     Token =  tok, 
					 Ids = my_ads$Id) 
}
}

\name{yadirGetBalance}
\alias{yadirGetBalance}
\title{Get 'Yandex Direct' account balance}
\description{Returns settings for shared accounts.}
\usage{
yadirGetBalance(Logins        = getOption("ryandexdirect.user"), 
                Token         = NULL,     
                AgencyAccount = getOption("ryandexdirect.agency_account"),
                TokenPath     = yadirTokenPath())
}

\arguments{
  \item{Logins}{character vector, your logins at Yandex Direct, require}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{Data frame}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
aut <- yadirAuth(Login = "login", NewUser = TRUE, 
        TokenPath = tempdir())
#For get accounts from client account use
my_balance <- yadirGetBalance(Logins = "login",
                              Token = aut)

#For get accounts from agancy account use
library(ryandexdirect)
aut <- yadirAuth(Login = "agency_login", NewUser = TRUE, 
                 TokenPath = tempdir())
client <- yadirGetClientList(Token = aut)
client_balance <- yadirGetBalance(Logins = client$Login,
                                  AgencyAccount = "agency_login",
                                  Token = aut)
}
}

\name{yadirStartCampaigns}
\alias{yadirStartCampaigns}
\title{Start Campaigns}
\description{Resumes impressions at the campaign level.}
\usage{
yadirStartCampaigns(Login = getOption("ryandexdirect.user"), 
           Ids = NULL, Token = NULL, 
           AgencyAccount = getOption("ryandexdirect.agency_account"), 
           TokenPath = yadirTokenPath())
}

\arguments{
  \item{Ids}{Integer or Character vector with Campaigns Ids, who will resume impressions}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Function resume impressions and return vector with Campaigns Ids with error.
}
\value{Vector with campaigns ids which failed to resume impressions}
\author{Alexey Seleznev}

\examples{
\dontrun{
# Get API token
### !!!Please choose another TokenPath to save the Login permanently.
tok <- yadirGetToken(TokenPath = tempdir())

# Get camp dictionary
my_camp <- yadirGetCampaignList(Login = "Login", Token = tok,
                                TokenPath = tempdir())

# Resume impressions
err <- yadirStartCampaigns(Login = "LOGIN", Token =  tok, Ids = my_camp$Id,
                           TokenPath = tempdir()) 
}
}

\name{yadirGetToken}
\alias{yadirGetToken}
\title{Auth in 'Yandex Direct' API}
\description{Simplified, one-step API authorization.}
\usage{
yadirGetToken(Login = NULL, TokenPath = yadirTokenPath())
}
\arguments{
  \item{Login}{character, your login at Yandex, require}
  \item{TokenPath}{Path to directory where you save credential data}
}
\value{R object with api token}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
myToken <- yadirGetToken(TokenPath = tempdir())
}
}

\name{yadirGetAdGroups}
\alias{yadirGetAdGroups}
\title{Get AdGroups List}
\description{Returns parameters of adgroups that meet specified criteria.}
\usage{
yadirGetAdGroups(CampaignIds   = NULL, 
                 Ids           = NA, 
                 Types         = c("TEXT_AD_GROUP", 
                                   "MOBILE_APP_AD_GROUP", 
                                   "DYNAMIC_TEXT_AD_GROUP",
                                   "CPM_BANNER_AD_GROUP", 
                                   "CPM_VIDEO_AD_GROUP",
                                   "SMART_AD_GROUP"),
                 Statuses      = c("ACCEPTED", 
				                   "DRAFT", 
			                       "MODERATION", 
								   "PREACCEPTED", 
								   "REJECTED"), 
                 Login         = getOption("ryandexdirect.user"),
                 AgencyAccount = getOption("ryandexdirect.agency_account"),
                 Token         = NULL,
                 TokenPath     = yadirTokenPath())
}

\arguments{
  \item{CampaignIds}{character vector with yandex direct campaign ID, require}
  \item{Ids}{character vector with yandex direct adgroup ID}
  \item{Types}{character vector with adgroup types states}
  \item{Statuses}{character vector with adgroup types statuses}
  \item{Login}{character, your login at Yandex, require}
  \item{Token}{character, your Yandex Direct API Token, require}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
}

\value{data frame with adgroups names and parameters}
\author{Alexey Seleznev}

\examples{
\dontrun{
#For get adgroups data frame
# Get data from clients account
my_ad_group <- yadirGetAdGroups(Login = "login")

# Get data from agency account
### !!!Please choose another TokenPath to save the Login permanently.
my_ad_group <- yadirGetAdGroups(Login         = "client_login",
                                AgencyAccount = "agency_login",
								TokenPath     =  tempdir())

}
}

\name{yadirGetReport}
\alias{yadirGetReport}
\title{Get Report From 'Yandex Direct Report Service'}
\description{The 'Service Report' is designed to get statistics on the advertiser's account. You must set period, choose report type and needed fields, and get statistic.}
\usage{
yadirGetReport(ReportType = "CUSTOM_REPORT", DateRangeType =
                 "CUSTOM_DATE", DateFrom = Sys.Date() - 31, 
                 DateTo = Sys.Date() - 1,
                 FieldNames = c("CampaignName", "Impressions",
                 "Clicks", "Cost"), FilterList = NULL, Goals = NULL,
                 AttributionModels = NULL, IncludeVAT = "YES",
                 IncludeDiscount = "NO", Login = getOption("ryandexdirect.user"), 
                 AgencyAccount = getOption("ryandexdirect.agency_account"), 
                 FetchBy = NULL, Token = NULL, TokenPath = yadirTokenPath(), 
				 SkipErrors = TRUE)
}

\arguments{
  \item{ReportType}{Report level, one from ACCOUNT_PERFORMANCE_REPORT, CAMPAIGN_PERFORMANCE_REPORT, ADGROUP_PERFORMANCE_REPORT, AD_PERFORMANCE_REPORT, CRITERIA_PERFORMANCE_REPORT, CUSTOM_REPORT, SEARCH_QUERY_PERFORMANCE_REPORT. For detail go https://tech.yandex.ru/direct/doc/reports/type-docpage/}
  \item{DateRangeType}{Reporting period, one from  TODAY, YESTERDAY, LAST_3_DAYS, LAST_5_DAYS, LAST_7_DAYS, LAST_14_DAYS, LAST_30_DAYS, LAST_90_DAYS, LAST_365_DAYS, THIS_WEEK_MON_TODAY ,THIS_WEEK_SUN_TODAY, LAST_WEEK, LAST_BUSINESS_WEEK, LAST_WEEK_SUN_SAT, THIS_MONTH, LAST_MONTH, ALL_TIME, CUSTOM_DATE, AUTO. For detail go https://tech.yandex.ru/direct/doc/reports/period-docpage/}
  \item{DateFrom}{If your select CUSTOM_DATE in DateRangeType you must enter start date in this argument in YYYY-MM-DD format else you can not used this argument}
  \item{DateTo}{If your select CUSTOM_DATE in DateRangeType you must enter end date in this argument in YYYY-MM-DD format else you can not used this argument}
  \item{FieldNames}{Character vector with field names list, for example c("Date","CampaignName","Impressions","Clicks"), for more ditails go https://yandex.ru/dev/direct/doc/reports/fields-list-docpage}
  \item{FilterList}{Character vector with rows filters, for example c("Clicks GREATER_THAN 99","Impressions LESS_THAN 1000"), for more details go https://tech.yandex.ru/direct/doc/reports/filters-docpage/}
  \item{Goals}{integer vector, id of the Yandex.Metric goals for which you want to get statistics. No more than 10 elements in the array. For example c(182453, 182452, 23458860).}
  \item{AttributionModels}{Attribution models used in the calculation of data on Yandex.Metrica goals. For example c("LSC", "LC", "FC"), see detail for more information.}
  \item{IncludeVAT}{Include or exclude VAT, one of "NO" or "YES"}
  \item{IncludeDiscount}{Include or exclude Discount, one of "NO" or "YES"}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{FetchBy}{Split the query by date range. Allowed values: "DAY", "WEEK", "MONTH", "QUARTER", "YEAR".}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
  \item{SkipErrors}{Logical, if TRUE function skip all errors of API answer}
}
\details{
Attribution model is the rule, what transition to consider the source of the visit.

Possible Values:

\describe{
  \item{FC}{The first transition. The source of any visitor visits is his first visit to the site in the last 180 days. This model allows you to track the source of advertising, which first brought the client and thereby influenced all subsequent interactions with the site.}
  \item{LC}{The last transition. The source of the visit is considered to be a transition, as a result of which the visitor is currently visiting the site, regardless of the history of visits. This model can be used in the technical analysis of the site. To assess the performance of advertising campaigns, use the Last significant transition model.}
  \item{LSC}{Is the last significant transition. All sources of transitions for the last 90 days are conventionally divided into significant (your ads) and insignificant (transitions from saved pages, internal transitions or direct visits to the site). The source of the visit is considered to be an ad transition. If the visit occurred after an insignificant transition, then its source is considered the last transition on the ad. This model allows you to take into account the conversions that are lost in the model Last transition due to technical limitations on the duration of the visit.}
  \item{LYDC}{Last click from Yandex.Direct. Of all the recent significant transitions, only Direct is taken into account. If a user at least once went to the site by an ad in Yandex.Direct, then this transition is considered the source of all subsequent visits - until the user clicks on the ad again. This model helps to see all the visits associated with the Directorate, including those that would be attributed to other sources in other attribution models.}
}
 
The AttributionModels parameter can be specified only if the Goals parameter is specified. If the Goals parameter is specified, and the AttributionModels parameter is not, the default value is LSC.

If several attribution models are specified, the data will be displayed for each model separately.

}
\seealso{
For get more ditails see vignett \code{vignette("yandex-direct-get-stat", package = "ryandexdirect")}
}
\references{
\href{https://yandex.ru/dev/direct/doc/reports/reports-docpage}{Official docs of 'Service Reports'}

\href{https://yandex.ru/support/metrika/general/goals.html}{What are goals?}

\href{https://yandex.ru/support/direct/statistics/attribution-model.html}{What are attribution models?}

\href{https://selesnow.github.io/ryandexdirect/}{Docs on ryandexdirect website}
}

\value{Data frame with the requested fields}

\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# For get Report from client account
statAgency <- yadirGetReport(ReportType       = "ACCOUNT_PERFORMANCE_REPORT", 
                             DateRangeType    = "CUSTOM_DATE", 
                             DateFrom         = "2018-01-01", 
                             DateTo           = "2018-05-10", 
                             FieldNames       = c("AdNetworkType",
                                                  "Impressions",
                                                  "Clicks",
                                                  "Cost"), 
							 Goals = c(182453, 
							           182452, 
							           234588),
							 AttributionModels = c("LSC", "LC"),
                             IncludeVAT        = "YES", 
                             IncludeDiscount   = "NO", 
                             Login             = "client_login", 
                             TokenPath         = tempdir())
							 
# For get data from agency client account
statAgency <- yadirGetReport(ReportType = "ACCOUNT_PERFORMANCE_REPORT", 
                             DateRangeType = "CUSTOM_DATE", 
                             DateFrom = "2018-01-01", 
                             DateTo = "2018-05-10", 
                             FieldNames = c("AdNetworkType",
                                            "Impressions",
                                            "Clicks",
                                            "Cost"), 
                             IncludeVAT = "YES", 
                             IncludeDiscount = "NO", 
                             Login = "client_login", 
                             AgencyAccount = "aguncy_login",
                             TokenPath = tempdir())
}
}

\name{yadirCurrencyRates}
\alias{yadirCurrencyRates}
\title{Get currencies}
\description{Exchange rates, currency parameters and restrictions.}
\usage{
yadirCurrencyRates(
     Login = getOption("ryandexdirect.user"), 
     AgencyAccount = getOption("ryandexdirect.agency_account"), 
     Token = NULL,
     TokenPath = yadirTokenPath())
}

\arguments{
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Token}{character or list object, your Yandex API Token, you can get this by function yadirGetToken or yadirAuth}
}
\value{data frame with campaings names and parameters}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
# from storage tokrn
library(ryandexdirect)
cur <- yadirCurrencyRates(Login = "login" ,
                          TokenPath = tempdir())

# from auth object

aut <- yadirAuth(Login = "login", NewUser = TRUE, 
        TokenPath = tempdir())
cur2 <- yadirCurrencyRates(Login =  "login" ,
                           Token = aut)
}
}

\name{yadirSetAutoKeyWordsBids}
\alias{yadirSetAutoKeyWordsBids}
\title{Auto Set KeyWords Bids}
\description{Assigns search phrases for phrases depending on the desired volume of traffic or rates in networks depending on the desired frequency of display (audience share).}
\usage{
yadirSetAutoKeyWordsBids(
    KeywordIds             = NULL,
    AdGroupIds             = NULL,
    CampaignIds            = NULL,
    TargetTrafficVolume    = NULL,
    SearchIncreasePercent  = NULL,
    SearchBidCeiling       = NULL,
    TargetCoverage         = NULL,
    NetworkIncreasePercent = NULL,
    NetworkBidCeiling      = NULL,
    Login                  = getOption("ryandexdirect.user"),
    Token                  = NULL,
    AgencyAccount          = getOption("ryandexdirect.agency_account"),
    TokenPath              = yadirTokenPath())
}

\arguments{
  \item{KeywordIds}{Phrase ID. Specified if you want to update the bid for an individual phrase.}
  \item{AdGroupIds}{Ad Group ID. Specified if you need to update bids for all group phrases.}
  \item{CampaignIds}{Campaign ID. Specified if you need to update bids for all campaign phrases.}
  \item{TargetTrafficVolume}{The desired amount of traffic on the search. It is specified in percents from 5 to 100. The surcharge is added to the rate corresponding to the selected traffic volume (see the SearchIncreasePercent parameter).}
  \item{SearchIncreasePercent}{Percentage of markup on search from 0 to 1000. If not specified, the markup is not calculated.}
  \item{SearchBidCeiling}{Limit on bid on search. Integer. Indicated in the currency of the advertiser.}
  \item{TargetCoverage}{The desired frequency of display (audience share) in the networks. It is specified in percents from 1 to 100. The surcharge is added to the rate corresponding to the selected display frequency (see the NetworkIncreasePercent parameter).}
  \item{NetworkIncreasePercent}{Percentage of markup on network from 0 to 1000. If not specified, the markup is not calculated.}
  \item{NetworkBidCeiling}{Limit on network bid. Integer. Indicated in the currency of the advertiser.}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Search.

The rate corresponding to the traffic volume \code{TargetTrafficVolume × (1 + SearchIncreasePercent / 100)}, but no more \code{SearchBidCeiling}.

Over time, the activity of competitors can raise the rate for the desired amount of traffic, and it will exceed the advertiser's rate. The higher the surcharge, the greater the volume of traffic, but the higher the possible costs.

Network.

The rate corresponding to the display frequency \code{TargetCoverage × (1 + NetworkIncreasePercent / 100)}, but no more \code{NetworkBidCeiling}.

Over time, the activity of competitors can raise the bid for the desired frequency of display, and it will exceed the advertiser's bid. The higher the surcharge, the higher the frequency of display, but the higher the possible costs.
}
\value{List with result data, object ids, warnings and errors.}
\author{Alexey Seleznev}
\references{
\href{https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/keywordbids-docpage}{KeyWord Bids API documentation}
\href{https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/setAuto-docpage}{KeyWord Bids setAuto method API documentation}
}
\examples{
\dontrun{
# loading keywords list
kw <- yadirGetKeyWords()
# set bid on search
autosetbids_search <- 
   yadirSetAutoKeyWordsBids(TargetTrafficVolume = 5, 
                            KeywordIds = kw$Id)
# set bid on network
autosetbids_network <- 
   yadirSetAutoKeyWordsBids(TargetCoverage = 15,
                            KeywordIds = KeywordIds)
}
}

\name{yadirSetKeyWordsBids}
\alias{yadirSetKeyWordsBids}
\title{Set KeyWords Bids}
\description{Assigns fixed bids and priorities for keyword phrases and auto-targeting.}
\usage{
yadirSetKeyWordsBids(
    KeywordIds       = NULL,
    AdGroupIds       = NULL,
    CampaignIds      = NULL,
    StrategyPriority = c(NA,
                        "LOW",
                        "NORMAL",
                        "HIGH"),
    SearchBid        = NULL,
    NetworkBid       = NULL,
    Login            = getOption("ryandexdirect.user"),
    Token            = NULL,
    AgencyAccount    = getOption("ryandexdirect.agency_account"),
    TokenPath        = yadirTokenPath())
}

\arguments{
  \item{KeywordIds}{Phrase ID. Specified if you want to update the bid for an individual phrase.}
  \item{AdGroupIds}{Ad Group ID. Specified if you need to update bids for all group phrases.}
  \item{CampaignIds}{Campaign ID. Specified if you need to update bids for all campaign phrases.}
  \item{StrategyPriority}{The priority of the phrase: LOW, NORMAL or HIGH. Only for automatic strategy.}
  \item{SearchBid}{Bid on search.}
  \item{NetworkBid}{Bid on network.}
  \item{Login}{Your Yandex Login.}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account.}
  \item{Token}{Your Yandex API Token.}
  \item{TokenPath}{Path to directory where you save credential data.}
}

\details{

You can set a bet or priority:

\itemize{
  \item for a separate keyword phrase or autotarget.
  \item for all keyword phrases and autotargeting in an ad group.
  \item for all keyword phrases and autotargets in the campaign.
} 

You can set a bid or priority depending on which display strategy is selected in the campaign:

\itemize{
  \item if the display strategy is selected on the \code{HIGHEST_POSITION} search, you can specify the SearchBid parameter.
  \item if you selected a display strategy on \code{MAXIMUM_COVERAGE} or \code{MANUAL_CPM} networks, you can specify the NetworkBid parameter.
  \item if the automatic strategy is selected, you can specify the StrategyPriority parameter: phrases with a higher priority receive as much traffic as possible, and in case of a budget deficit, they are turned off last.
}

If the element of the input array contains parameters that do not correspond to the strategy, the values of these parameters will not be changed:

\itemize{
  \item If the input array element contains both parameters corresponding to the strategy and parameters that do not correspond to the strategy (for example, both the bid and priority), then as a result of the operation, the values of only the parameters corresponding to the strategy will be changed and a warning will be issued.
  \item If the input array element contains only parameters that do not correspond to the strategy (for example, only the priority for a manual strategy), the operation will fail and an error will be returned.
}

}

\seealso{
See for get and autoset keyword bids \code{\link[ryandexdirect:yadirGetKeyWordsBids]{yadirGetKeyWordsBids()}} and \code{\link[ryandexdirect:yadirSetAutoKeyWordsBids]{yadirSetAutoKeyWordsBids()}}
}

\value{List with result data, object ids, warnings and errors.}
\author{Alexey Seleznev}
\references{
\itemize{
  \item \href{https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/keywordbids-docpage}{KeyWord Bids API documentation}
  \item \href{https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/set-docpage}{KeyWord Bids set method API documentation}
}
}
\examples{
\dontrun{
# loading keywords list
kw <- yadirGetKeyWords()
# set bid on search
setbid <- yadirSetKeyWordsBids(KeywordIds = kw$Id,
                               SearchBid  = 9)
}
}

\name{yadirAuth}
\alias{yadirAuth}
\title{Authentication in 'Yandex Direct' API}
\description{Get API Token for 'Yandex Direct' account}
\usage{
yadirAuth(Login = getOption("ryandexdirect.user"), 
          NewUser = FALSE, 
          TokenPath = yadirTokenPath())
}
\arguments{
  \item{Login}{character, your login at Yandex, require}
  \item{NewUser}{logical, use TRUE if you want login into new account}
  \item{TokenPath}{Path to directory where you save credential data}
}
\value{R object with api token}
\author{Alexey Seleznev}

\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
library(ryandexdirect)
yadirAuth(Login = "login", NewUser = TRUE,
          TokenPath = tempdir())
}
}

\name{yadirSetLogin}
\alias{yadirSetLogin}
\title{Set Login}
\description{Set yandex direct login in current R session.}
\usage{
yadirSetLogin(Login, TokenPath = yadirTokenPath())
}

\arguments{
  \item{TokenPath}{Path to directory where you save credential data}
  \item{Login}{Character, your username in Yandex}
}
\value{TRUE}
\author{Alexey Seleznev}

\examples{
\dontrun{
yadirSetLogin('my_name')
}
}

\name{yadirGetKeyWordsBids}
\alias{yadirGetKeyWordsBids}
\title{Get KeyWords Bids}
\description{Returns bids and priorities for key phrases and auto-targeting that meet specified criteria, as well as bidding data: bids and write-off prices for different traffic volumes on a search and bids to reach different audience shares in networks.}
\usage{
yadirGetKeyWordsBids(
    KeywordIds    = NULL,
    AdGroupIds    = NULL,
    CampaignIds   = NULL,
    AuctionBids   = c(NA, 
		                  "search", 
		          			  "network"),
    Login         = getOption("ryandexdirect.user"),
    Token         = NULL,
    AgencyAccount = getOption("ryandexdirect.agency_account"),
    TokenPath     = yadirTokenPath())
}

\arguments{
  \item{KeywordIds}{Phrase ID. Specified if you want to update the bid for an individual phrase.}
  \item{AdGroupIds}{Ad Group ID. Specified if you need to update bids for all group phrases.}
  \item{CampaignIds}{Campaign ID. Specified if you need to update bids for all campaign phrases.}
  \item{AuctionBids}{Data on auctions, specify information about which auctions you are requesting. The values are: NA, "search", "network"}
  \item{Login}{Your Yandex Login}
  \item{AgencyAccount}{Your agency account login, if you get statistic from client account}
  \item{Token}{Your Yandex API Token}
  \item{TokenPath}{Path to directory where you save credential data}
}
\details{
Use AuctionBids = "search" when you need traffic volume, bid on the search corresponding to the specified amount of traffi and charged price corresponding to the specified amount of traffic.
Use AuctionBids = "search" when you need get frequency of the show (audience share) in networks (specified in percent from 0 to 100) and network bid that matches the frequency shown.
}
\value{Data Frame with keyword bids and auction data}
\author{Alexey Seleznev}
\references{
\href{https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/keywordbids-docpage}{KeyWord Bids API documentation}
\href{https://yandex.ru/dev/direct/doc/ref-v5/keywordbids/get-docpage}{KeyWord Bids get method API documentation}
}
\examples{
\dontrun{
### Please choose another TokenPath to save the Login permanently.
## Only bids for keywords with ids 123, 234, 345
bids <- yadirGetKeyWordsBids(KeywordIds = c(123, 234, 345))

### Get bids with search auction data
search_bids <- 
    yadirGetKeyWordsBids(KeywordIds = c(123, 234, 345)
	                     AuctionBids = "search")
						 
### Get bids with network auction data
network_bids <- 
    yadirGetKeyWordsBids(KeywordIds = c(123, 234, 345)
	                     AuctionBids = "network")
}
}
